@@ -14,8 +14,6 @@
  * limitations under the License.
  */
 
-// @author Andrei Alexandrescu (andrei.alexandrescu@fb.com)
-
 #include <folly/Benchmark.h>
 
 #include <algorithm>
@@ -33,7 +31,7 @@
 #include <folly/MapUtil.h>
 #include <folly/String.h>
 #include <folly/detail/PerfScoped.h>
-#include <folly/json.h>
+#include <folly/json/json.h>
 
 // This needs to be at the end because some versions end up including
 // Windows.h without defining NOMINMAX, which breaks uses
@@ -81,7 +79,7 @@ FOLLY_GFLAGS_DEFINE_string(
     bm_json_verbose,
     "",
     "File to write verbose JSON format (for BenchmarkCompare / --bm_relative_to). "
-    "NOTE: this is written independent of the above --json / --bm_relative_to.");
+    "NOTE: this file is written regardless of options --json and --bm_relative_to.");
 
 FOLLY_GFLAGS_DEFINE_string(
     bm_regex, "", "Only benchmarks whose names match this regex will be run.");
@@ -658,8 +656,20 @@ namespace {
 struct BenchmarksToRun {
   const detail::BenchmarkRegistration* baseline = nullptr;
   std::vector<const detail::BenchmarkRegistration*> benchmarks;
+  std::vector<size_t> separatorsAfter;
 };
 
+void addSeparator(BenchmarksToRun& res) {
+  size_t separatorAfter = res.benchmarks.size();
+  if (separatorAfter == 0) {
+    return;
+  }
+  if (res.separatorsAfter.empty() ||
+      res.separatorsAfter.back() != separatorAfter) {
+    res.separatorsAfter.push_back(res.benchmarks.size() - 1);
+  }
+}
+
 BenchmarksToRun selectBenchmarksToRun(
     const std::vector<detail::BenchmarkRegistration>& benchmarks) {
   BenchmarksToRun res;
@@ -673,7 +683,8 @@ BenchmarksToRun selectBenchmarksToRun(
   }
 
   for (auto& bm : benchmarks) {
-    if (bm.name == "-") { // skip separators
+    if (bm.name == "-") {
+      addSeparator(res);
       continue;
     }
 
@@ -687,6 +698,10 @@ BenchmarksToRun selectBenchmarksToRun(
     }
   }
 
+  if (bmRegex) {
+    res.separatorsAfter.clear();
+  }
+
   CHECK(res.baseline);
 
   return res;
@@ -708,6 +723,30 @@ void maybeRunWarmUpIteration(const BenchmarksToRun& toRun) {
   }
 }
 
+class ShouldDrawLineTracker {
+ public:
+  explicit ShouldDrawLineTracker(const BenchmarksToRun& toRun)
+      : separatorsAfter_(&toRun.separatorsAfter) {}
+
+  bool operator()() {
+    std::size_t i = curI_++;
+    if (drawAfterI_ >= separatorsAfter_->size()) {
+      return false;
+    }
+    std::size_t nextToDrawAfter = (*separatorsAfter_)[drawAfterI_];
+    if (i == nextToDrawAfter) {
+      ++drawAfterI_;
+      return true;
+    }
+    return false;
+  }
+
+ private:
+  const std::vector<std::size_t>* separatorsAfter_;
+  std::size_t curI_ = 0;
+  std::size_t drawAfterI_ = 0;
+};
+
 std::pair<std::set<std::string>, std::vector<detail::BenchmarkResult>>
 runBenchmarksWithPrinterImpl(
     BenchmarkResultsPrinter* FOLLY_NULLABLE printer,
@@ -721,9 +760,11 @@ runBenchmarksWithPrinterImpl(
       runBenchmarkGetNSPerIteration(toRun.baseline->func, 0);
 
   std::set<std::string> counterNames;
-  for (const auto bmPtr : toRun.benchmarks) {
+  ShouldDrawLineTracker shouldDrawLineTracker(toRun);
+  for (std::size_t i = 0; i != toRun.benchmarks.size(); ++i) {
     std::pair<double, UserCounters> elapsed;
-    const detail::BenchmarkRegistration& bm = *bmPtr;
+    const detail::BenchmarkRegistration& bm = *toRun.benchmarks[i];
+    bool shoudDrawLineAfter = shouldDrawLineTracker();
 
     if (FLAGS_bm_profile) {
       elapsed = runProfilingGetNSPerIteration(bm.func, globalBaseline.first);
@@ -738,6 +779,9 @@ runBenchmarksWithPrinterImpl(
     // counters have been used, then the header can be printed out properly
     if (printer != nullptr) {
       printer->print({{bm.file, bm.name, elapsed.first, elapsed.second}});
+      if (shoudDrawLineAfter) {
+        printer->separator('-');
+      }
     }
     results.push_back({bm.file, bm.name, elapsed.first, elapsed.second});
 
@@ -855,6 +899,12 @@ void runBenchmarks() {
         "WARNING: Running with constant number of iterations. Results might be jittery.\n");
   }
 
+  if (FLAGS_bm_min_iters >= FLAGS_bm_max_iters) {
+    std::cerr << "WARNING: bm_min_iters > bm_max_iters; increasing the max"
+              << std::endl;
+    FLAGS_bm_max_iters = FLAGS_bm_min_iters + 1;
+  }
+
   checkRunMode();
 
   auto& state = detail::globalBenchmarkState();
