@@ -29,7 +29,10 @@
 #include <cassert>
 #include <chrono>
 #include <functional>
+#include <iosfwd>
 #include <limits>
+#include <mutex>
+#include <set>
 #include <type_traits>
 #include <unordered_map>
 
@@ -67,6 +70,13 @@ class UserMetric {
   UserMetric() = default;
   /* implicit */ UserMetric(int64_t val, Type typ = Type::CUSTOM)
       : value(val), type(typ) {}
+
+  friend bool operator==(const UserMetric& x, const UserMetric& y) {
+    return x.value == y.value && x.type == y.type;
+  }
+  friend bool operator!=(const UserMetric& x, const UserMetric& y) {
+    return !(x == y);
+  }
 };
 
 using UserCounters = std::unordered_map<std::string, UserMetric>;
@@ -92,30 +102,26 @@ struct BenchmarkResult {
   std::string name;
   double timeInNs;
   UserCounters counters;
+
+  friend std::ostream& operator<<(std::ostream&, const BenchmarkResult&);
+
+  friend bool operator==(const BenchmarkResult&, const BenchmarkResult&);
+  friend bool operator!=(const BenchmarkResult& x, const BenchmarkResult& y) {
+    return !(x == y);
+  }
 };
 
-/**
- * Adds a benchmark wrapped in a std::function. Only used
- * internally. Pass by value is intentional.
- */
-void addBenchmarkImpl(
-    const char* file, StringPiece name, BenchmarkFun, bool useCounter);
+struct BenchmarkSuspenderBase {
+  /**
+   * Accumulates time spent outside benchmark.
+   */
+  static std::chrono::high_resolution_clock::duration timeSpent;
+};
 
-/**
- * Runs all benchmarks defined in the program, doesn't print by default.
- * Usually used when customized printing of results is desired.
- */
-std::vector<BenchmarkResult> runBenchmarksWithResults();
-
-} // namespace detail
-
-/**
- * Supporting type for BENCHMARK_SUSPEND defined below.
- */
-struct BenchmarkSuspender {
-  using Clock = std::chrono::high_resolution_clock;
-  using TimePoint = Clock::time_point;
-  using Duration = Clock::duration;
+template <typename Clock>
+struct BenchmarkSuspender : BenchmarkSuspenderBase {
+  using TimePoint = std::chrono::high_resolution_clock::time_point;
+  using Duration = std::chrono::high_resolution_clock::duration;
 
   struct DismissedTag {};
   static inline constexpr DismissedTag Dismissed{};
@@ -159,7 +165,9 @@ struct BenchmarkSuspender {
 
   template <class F>
   auto dismissing(F f) -> invoke_result_t<F> {
-    SCOPE_EXIT { rehire(); };
+    SCOPE_EXIT {
+      rehire();
+    };
     dismiss();
     return f();
   }
@@ -170,11 +178,6 @@ struct BenchmarkSuspender {
    */
   explicit operator bool() const { return false; }
 
-  /**
-   * Accumulates time spent outside benchmark.
-   */
-  static Duration timeSpent;
-
  private:
   void tally() {
     auto end = Clock::now();
@@ -185,89 +188,156 @@ struct BenchmarkSuspender {
   TimePoint start;
 };
 
+class PerfScoped;
+
+class BenchmarkingStateBase {
+ public:
+  template <typename Printer>
+  std::pair<std::set<std::string>, std::vector<BenchmarkResult>>
+  runBenchmarksWithPrinter(Printer* printer) const;
+
+  std::vector<BenchmarkResult> runBenchmarksWithResults() const;
+
+  static folly::StringPiece getGlobalBaselineNameForTests();
+
+  bool useCounters() const;
+
+  void addBenchmarkImpl(
+      const char* file, StringPiece name, BenchmarkFun, bool useCounter);
+
+ protected:
+  // There is no need for this virtual but we overcome a check
+  virtual ~BenchmarkingStateBase() = default;
+
+  PerfScoped setUpPerfScoped() const;
+
+  // virtual for purely testing purposes.
+  virtual PerfScoped doSetUpPerfScoped(
+      const std::vector<std::string>& args) const;
+
+  mutable std::mutex mutex_;
+  std::vector<BenchmarkRegistration> benchmarks_;
+};
+
+template <typename Clock>
+class BenchmarkingState : public BenchmarkingStateBase {
+ public:
+  template <typename Lambda>
+  typename std::enable_if<folly::is_invocable_v<Lambda, unsigned>>::type
+  addBenchmark(const char* file, StringPiece name, Lambda&& lambda) {
+    auto execute = [=](unsigned int times) {
+      BenchmarkSuspender<Clock>::timeSpent = {};
+      unsigned int niter;
+
+      // CORE MEASUREMENT STARTS
+      auto start = Clock::now();
+      niter = lambda(times);
+      auto end = Clock::now();
+      // CORE MEASUREMENT ENDS
+      return detail::TimeIterData{
+          (end - start) - BenchmarkSuspender<Clock>::timeSpent,
+          niter,
+          UserCounters{}};
+    };
+
+    this->addBenchmarkImpl(file, name, detail::BenchmarkFun(execute), false);
+  }
+
+  template <typename Lambda>
+  typename std::enable_if<folly::is_invocable_v<Lambda>>::type addBenchmark(
+      const char* file, StringPiece name, Lambda&& lambda) {
+    addBenchmark(file, name, [=](unsigned int times) {
+      unsigned int niter = 0;
+      while (times-- > 0) {
+        niter += lambda();
+      }
+      return niter;
+    });
+  }
+
+  template <typename Lambda>
+  typename std::enable_if<
+      folly::is_invocable_v<Lambda, UserCounters&, unsigned>>::type
+  addBenchmark(const char* file, StringPiece name, Lambda&& lambda) {
+    auto execute = [=](unsigned int times) {
+      BenchmarkSuspender<Clock>::timeSpent = {};
+      unsigned int niter;
+
+      // CORE MEASUREMENT STARTS
+      auto start = std::chrono::high_resolution_clock::now();
+      UserCounters counters;
+      niter = lambda(counters, times);
+      auto end = std::chrono::high_resolution_clock::now();
+      // CORE MEASUREMENT ENDS
+      return detail::TimeIterData{
+          (end - start) - BenchmarkSuspender<Clock>::timeSpent,
+          niter,
+          counters};
+    };
+
+    this->addBenchmarkImpl(
+        file,
+        name,
+        std::function<detail::TimeIterData(unsigned int)>(execute),
+        true);
+  }
+
+  template <typename Lambda>
+  typename std::enable_if<folly::is_invocable_v<Lambda, UserCounters&>>::type
+  addBenchmark(const char* file, StringPiece name, Lambda&& lambda) {
+    addBenchmark(file, name, [=](UserCounters& counters, unsigned int times) {
+      unsigned int niter = 0;
+      while (times-- > 0) {
+        niter += lambda(counters);
+      }
+      return niter;
+    });
+  }
+};
+
+BenchmarkingState<std::chrono::high_resolution_clock>& globalBenchmarkState();
+
 /**
- * Adds a benchmark. Usually not called directly but instead through
- * the macro BENCHMARK defined below. The lambda function involved
- * must take exactly one parameter of type unsigned, and the benchmark
- * uses it with counter semantics (iteration occurs inside the
- * function).
+ * Runs all benchmarks defined in the program, doesn't print by default.
+ * Usually used when customized printing of results is desired.
  */
-template <typename Lambda>
-typename std::enable_if<folly::is_invocable_v<Lambda, unsigned>>::type
-addBenchmark(const char* file, StringPiece name, Lambda&& lambda) {
-  auto execute = [=](unsigned int times) {
-    BenchmarkSuspender::timeSpent = {};
-    unsigned int niter;
+std::vector<BenchmarkResult> runBenchmarksWithResults();
 
-    // CORE MEASUREMENT STARTS
-    auto start = std::chrono::high_resolution_clock::now();
-    niter = lambda(times);
-    auto end = std::chrono::high_resolution_clock::now();
-    // CORE MEASUREMENT ENDS
-    return detail::TimeIterData{
-        (end - start) - BenchmarkSuspender::timeSpent, niter, UserCounters{}};
-  };
-
-  detail::addBenchmarkImpl(file, name, detail::BenchmarkFun(execute), false);
+/**
+ * Adds a benchmark wrapped in a std::function.
+ * Was designed to only be used internally but, unfortunately,
+ * is not.
+ */
+inline void addBenchmarkImpl(
+    const char* file, StringPiece name, BenchmarkFun f, bool useCounter) {
+  globalBenchmarkState().addBenchmarkImpl(file, name, std::move(f), useCounter);
 }
 
+} // namespace detail
+
+/**
+ * Supporting type for BENCHMARK_SUSPEND defined below.
+ */
+struct BenchmarkSuspender
+    : detail::BenchmarkSuspender<std::chrono::high_resolution_clock> {
+  using Impl = detail::BenchmarkSuspender<std::chrono::high_resolution_clock>;
+  using Impl::Impl;
+};
+
 /**
  * Adds a benchmark. Usually not called directly but instead through
- * the macro BENCHMARK defined below. The lambda function involved
- * must take zero parameters, and the benchmark calls it repeatedly
- * (iteration occurs outside the function).
+ * the macro BENCHMARK defined below.
+ * The lambda function involved can have one of the following forms:
+ *  * take zero parameters, and the benchmark calls it repeatedly
+ *  * take exactly one parameter of type unsigned, and the benchmark
+ *    uses it with counter semantics (iteration occurs inside the
+ *    function).
+ *  * 2 versions of the above cases but also accept UserCounters& as
+ *    as their first parameter.
  */
 template <typename Lambda>
-typename std::enable_if<folly::is_invocable_v<Lambda>>::type addBenchmark(
-    const char* file, StringPiece name, Lambda&& lambda) {
-  addBenchmark(file, name, [=](unsigned int times) {
-    unsigned int niter = 0;
-    while (times-- > 0) {
-      niter += lambda();
-    }
-    return niter;
-  });
-}
-
-/**
- * similar as previous two template specialization, but lambda will also take
- * customized counters in the following two cases
- */
-template <typename Lambda>
-typename std::enable_if<
-    folly::is_invocable_v<Lambda, UserCounters&, unsigned>>::type
-addBenchmark(const char* file, StringPiece name, Lambda&& lambda) {
-  auto execute = [=](unsigned int times) {
-    BenchmarkSuspender::timeSpent = {};
-    unsigned int niter;
-
-    // CORE MEASUREMENT STARTS
-    auto start = std::chrono::high_resolution_clock::now();
-    UserCounters counters;
-    niter = lambda(counters, times);
-    auto end = std::chrono::high_resolution_clock::now();
-    // CORE MEASUREMENT ENDS
-    return detail::TimeIterData{
-        (end - start) - BenchmarkSuspender::timeSpent, niter, counters};
-  };
-
-  detail::addBenchmarkImpl(
-      file,
-      name,
-      std::function<detail::TimeIterData(unsigned int)>(execute),
-      true);
-}
-
-template <typename Lambda>
-typename std::enable_if<folly::is_invocable_v<Lambda, UserCounters&>>::type
-addBenchmark(const char* file, StringPiece name, Lambda&& lambda) {
-  addBenchmark(file, name, [=](UserCounters& counters, unsigned int times) {
-    unsigned int niter = 0;
-    while (times-- > 0) {
-      niter += lambda(counters);
-    }
-    return niter;
-  });
+void addBenchmark(const char* file, StringPiece name, Lambda&& lambda) {
+  detail::globalBenchmarkState().addBenchmark(file, name, lambda);
 }
 
 struct dynamic;
@@ -289,34 +359,34 @@ void printResultComparison(
  * friends below.
  */
 
-#define BENCHMARK_IMPL(funName, stringName, rv, paramType, paramName)          \
-  static void funName(paramType);                                              \
-  FOLLY_MAYBE_UNUSED static bool FB_ANONYMOUS_VARIABLE(follyBenchmarkUnused) = \
-      (::folly::addBenchmark(                                                  \
-           __FILE__,                                                           \
-           stringName,                                                         \
-           [](paramType paramName) -> unsigned {                               \
-             funName(paramName);                                               \
-             return rv;                                                        \
-           }),                                                                 \
-       true);                                                                  \
+#define BENCHMARK_IMPL(funName, stringName, rv, paramType, paramName)        \
+  static void funName(paramType);                                            \
+  [[maybe_unused]] static bool FB_ANONYMOUS_VARIABLE(follyBenchmarkUnused) = \
+      (::folly::addBenchmark(                                                \
+           __FILE__,                                                         \
+           stringName,                                                       \
+           [](paramType paramName) -> unsigned {                             \
+             funName(paramName);                                             \
+             return rv;                                                      \
+           }),                                                               \
+       true);                                                                \
   static void funName(paramType paramName)
 
-#define BENCHMARK_IMPL_COUNTERS(                                               \
-    funName, stringName, counters, rv, paramType, paramName)                   \
-  static void funName(                                                         \
-      ::folly::UserCounters& FOLLY_PP_DETAIL_APPEND_VA_ARG(paramType));        \
-  FOLLY_MAYBE_UNUSED static bool FB_ANONYMOUS_VARIABLE(follyBenchmarkUnused) = \
-      (::folly::addBenchmark(                                                  \
-           __FILE__,                                                           \
-           stringName,                                                         \
-           [](::folly::UserCounters& counters FOLLY_PP_DETAIL_APPEND_VA_ARG(   \
-               paramType paramName)) -> unsigned {                             \
-             funName(counters FOLLY_PP_DETAIL_APPEND_VA_ARG(paramName));       \
-             return rv;                                                        \
-           }),                                                                 \
-       true);                                                                  \
-  static void funName(::folly::UserCounters& counters                          \
+#define BENCHMARK_IMPL_COUNTERS(                                             \
+    funName, stringName, counters, rv, paramType, paramName)                 \
+  static void funName(                                                       \
+      ::folly::UserCounters& FOLLY_PP_DETAIL_APPEND_VA_ARG(paramType));      \
+  [[maybe_unused]] static bool FB_ANONYMOUS_VARIABLE(follyBenchmarkUnused) = \
+      (::folly::addBenchmark(                                                \
+           __FILE__,                                                         \
+           stringName,                                                       \
+           [](::folly::UserCounters& counters FOLLY_PP_DETAIL_APPEND_VA_ARG( \
+               paramType paramName)) -> unsigned {                           \
+             funName(counters FOLLY_PP_DETAIL_APPEND_VA_ARG(paramName));     \
+             return rv;                                                      \
+           }),                                                               \
+       true);                                                                \
+  static void funName([[maybe_unused]] ::folly::UserCounters& counters       \
                           FOLLY_PP_DETAIL_APPEND_VA_ARG(paramType paramName))
 
 /**
@@ -324,14 +394,14 @@ void printResultComparison(
  * number of iterations. Used internally, see BENCHMARK_MULTI and friends
  * below.
  */
-#define BENCHMARK_MULTI_IMPL(funName, stringName, paramType, paramName)        \
-  static unsigned funName(paramType);                                          \
-  FOLLY_MAYBE_UNUSED static bool FB_ANONYMOUS_VARIABLE(follyBenchmarkUnused) = \
-      (::folly::addBenchmark(                                                  \
-           __FILE__,                                                           \
-           stringName,                                                         \
-           [](paramType paramName) { return funName(paramName); }),            \
-       true);                                                                  \
+#define BENCHMARK_MULTI_IMPL(funName, stringName, paramType, paramName)      \
+  static unsigned funName(paramType);                                        \
+  [[maybe_unused]] static bool FB_ANONYMOUS_VARIABLE(follyBenchmarkUnused) = \
+      (::folly::addBenchmark(                                                \
+           __FILE__,                                                         \
+           stringName,                                                       \
+           [](paramType paramName) { return funName(paramName); }),          \
+       true);                                                                \
   static unsigned funName(paramType paramName)
 
 /**
@@ -575,9 +645,9 @@ void printResultComparison(
 /**
  * Draws a line of dashes.
  */
-#define BENCHMARK_DRAW_LINE()                                                  \
-  FOLLY_MAYBE_UNUSED static bool FB_ANONYMOUS_VARIABLE(follyBenchmarkUnused) = \
-      (::folly::addBenchmark(__FILE__, "-", []() -> unsigned { return 0; }),   \
+#define BENCHMARK_DRAW_LINE()                                                \
+  [[maybe_unused]] static bool FB_ANONYMOUS_VARIABLE(follyBenchmarkUnused) = \
+      (::folly::addBenchmark(__FILE__, "-", []() -> unsigned { return 0; }), \
        true)
 
 /**
