@@ -19,6 +19,8 @@
 #include <algorithm>
 #include <cstring>
 
+#include <folly/CppAttributes.h>
+#include <folly/functional/Invoke.h>
 #include <folly/lang/CString.h>
 
 #if __has_include(<cxxabi.h>)
@@ -57,10 +59,44 @@ static constexpr auto cxxabi_demangle = static_cast<char* (*)(...)>(nullptr);
 //  liberty
 //
 //  in contrast with cxxabi, where there are certainly other referenced symbols
+//
+//  for rust_demangle_callback, detect its declaration in the header
 
 #if __has_include(<demangle.h>)
 
-static constexpr auto liberty_demangle = cplus_demangle_v3_callback;
+namespace {
+struct poison {};
+
+[[maybe_unused]] FOLLY_ERASE void rust_demangle_callback(poison);
+
+[[maybe_unused]] FOLLY_ERASE int rust_demangle_callback_fallback(
+    const char*, int, demangle_callbackref, void*) {
+  return 0;
+}
+
+FOLLY_CREATE_QUAL_INVOKER(
+    invoke_rust_demangle_primary, ::rust_demangle_callback);
+FOLLY_CREATE_QUAL_INVOKER(
+    invoke_rust_demangle_fallback, rust_demangle_callback_fallback);
+
+using invoke_rust_demangle_fn = folly::invoke_first_match<
+    invoke_rust_demangle_primary,
+    invoke_rust_demangle_fallback>;
+constexpr invoke_rust_demangle_fn invoke_rust_demangle;
+
+int call_rust_demangle_callback(
+    const char* mangled, int options, demangle_callbackref cb, void* opaque) {
+  return invoke_rust_demangle(mangled, options, cb, opaque);
+}
+
+} // namespace
+
+using liberty_demangle_t = int(const char*, int, demangle_callbackref, void*);
+
+static constexpr liberty_demangle_t* liberty_cplus_demangle =
+    cplus_demangle_v3_callback;
+static constexpr liberty_demangle_t* liberty_rust_demangle =
+    call_rust_demangle_callback;
 
 #if defined(DMGL_NO_RECURSE_LIMIT)
 static constexpr auto liberty_demangle_options_no_recurse_limit =
@@ -75,7 +111,10 @@ static constexpr auto liberty_demangle_options = //
 
 #else // __has_include(<demangle.h>)
 
-static constexpr auto liberty_demangle = static_cast<int (*)(...)>(nullptr);
+using liberty_demangle_t = int(...);
+
+static constexpr liberty_demangle_t* liberty_cplus_demangle = nullptr;
+static constexpr liberty_demangle_t* liberty_rust_demangle = nullptr;
 static constexpr auto liberty_demangle_options = 0;
 
 #endif // __has_include(<demangle.h>)
@@ -84,10 +123,19 @@ static constexpr auto liberty_demangle_options = 0;
 
 namespace folly {
 
-//  reinterpret-cast currently evades -Waddress
 bool const demangle_build_has_cxxabi = cxxabi_demangle;
+//  reinterpret-cast currently evades -Waddress
 bool const demangle_build_has_liberty =
-    reinterpret_cast<void*>(liberty_demangle);
+    reinterpret_cast<void*>(liberty_cplus_demangle) &&
+    reinterpret_cast<void*>(liberty_rust_demangle);
+
+namespace {
+void demangleStringCallback(const char* str, size_t size, void* p) {
+  fbstring* demangle = static_cast<fbstring*>(p);
+
+  demangle->append(str, size);
+}
+} // namespace
 
 fbstring demangle(const char* name) {
   if (!name) {
@@ -106,6 +154,25 @@ fbstring demangle(const char* name) {
     }
   }
 
+  if (folly::demangle_build_has_liberty) {
+    liberty_demangle_t* funcs[] = {
+        liberty_rust_demangle,
+        liberty_cplus_demangle,
+    };
+
+    for (auto func : funcs) {
+      fbstring demangled;
+
+      // Unlike most library functions, this returns 1 on success and 0 on
+      // failure
+      int success = func(
+          name, liberty_demangle_options, demangleStringCallback, &demangled);
+      if (success && !demangled.empty()) {
+        return demangled;
+      }
+    }
+  }
+
   if (cxxabi_demangle) {
     int status;
     size_t len = 0;
@@ -131,7 +198,7 @@ struct DemangleBuf {
   size_t total;
 };
 
-void demangleCallback(const char* str, size_t size, void* p) {
+void demangleBufCallback(const char* str, size_t size, void* p) {
   DemangleBuf* buf = static_cast<DemangleBuf*>(p);
   size_t n = std::min(buf->remaining, size);
   memcpy(buf->dest, str, n);
@@ -156,24 +223,32 @@ size_t demangle(const char* name, char* out, size_t outSize) {
   }
 
   if (folly::demangle_build_has_liberty) {
-    DemangleBuf dbuf;
-    dbuf.dest = out;
-    dbuf.remaining = outSize ? outSize - 1 : 0; // leave room for null term
-    dbuf.total = 0;
+    liberty_demangle_t* funcs[] = {
+        liberty_rust_demangle,
+        liberty_cplus_demangle,
+    };
 
-    // Unlike most library functions, this returns 1 on success and 0 on failure
-    int status = liberty_demangle(
-        name, liberty_demangle_options, demangleCallback, &dbuf);
-    if (status == 0) { // failed, return original
-      return folly::strlcpy(out, name, outSize);
+    for (auto func : funcs) {
+      DemangleBuf dbuf;
+      dbuf.dest = out;
+      dbuf.remaining = outSize ? outSize - 1 : 0; // leave room for null term
+      dbuf.total = 0;
+
+      // Unlike most library functions, this returns 1 on success and 0 on
+      // failure
+      int success =
+          func(name, liberty_demangle_options, demangleBufCallback, &dbuf);
+      if (success) {
+        if (outSize != 0) {
+          *dbuf.dest = '\0';
+        }
+        return dbuf.total;
+      }
     }
-    if (outSize != 0) {
-      *dbuf.dest = '\0';
-    }
-    return dbuf.total;
-  } else {
-    return folly::strlcpy(out, name, outSize);
   }
+
+  // fallback - just return original
+  return folly::strlcpy(out, name, outSize);
 }
 
 } // namespace folly
