@@ -18,11 +18,6 @@
 // Docs: https://fburl.com/fbcref_function
 //
 
-/*
- * @author Eric Niebler (eniebler@fb.com), Sven Over (over@fb.com)
- * Acknowledgements: Giuseppe Ottaviano (ott@fb.com)
- */
-
 /**
  * @class folly::Function
  * @refcode folly/docs/examples/folly/Function.cpp
@@ -218,6 +213,7 @@
 #include <folly/functional/Invoke.h>
 #include <folly/lang/Align.h>
 #include <folly/lang/Exception.h>
+#include <folly/lang/New.h>
 
 namespace folly {
 
@@ -238,7 +234,14 @@ namespace function {
 enum class Op { MOVE, NUKE, HEAP };
 
 union Data {
+  struct BigTrivialLayout {
+    void* big;
+    std::size_t size;
+    std::size_t align;
+  };
+
   void* big;
+  BigTrivialLayout bigt;
   std::aligned_storage<6 * sizeof(void*)>::type tiny;
 };
 
@@ -261,7 +264,7 @@ constexpr bool isEmptyFunction(T const& t) {
 }
 
 template <typename F, typename... Args>
-using CallableResult = decltype(FOLLY_DECLVAL(F &&)(FOLLY_DECLVAL(Args &&)...));
+using CallableResult = decltype(FOLLY_DECLVAL(F&&)(FOLLY_DECLVAL(Args&&)...));
 
 template <typename F, typename... Args>
 constexpr bool CallableNoexcept =
@@ -326,6 +329,21 @@ class FunctionTraitsSharedProxy {
   }
 };
 
+template <
+    typename Fun,
+    bool Small,
+    bool Nx,
+    typename ReturnType,
+    typename... Args>
+ReturnType call_(Args... args, Data& p) noexcept(Nx) {
+  auto& fn = *static_cast<Fun*>(Small ? &p.tiny : p.big);
+  if constexpr (std::is_void<ReturnType>::value) {
+    fn(static_cast<Args&&>(args)...);
+  } else {
+    return fn(static_cast<Args&&>(args)...);
+  }
+}
+
 template <typename FunctionType>
 struct FunctionTraits;
 
@@ -339,25 +357,9 @@ struct FunctionTraits<ReturnType(Args...)> {
   template <typename F, typename R = CallableResult<F&, Args...>>
   using IfSafeResult = IfSafeResultImpl<R, ReturnType>;
 
-  template <typename Fun>
-  static ReturnType callSmall(CallArg<Args>... args, Data& p) {
-    auto& fn = *static_cast<Fun*>(static_cast<void*>(&p.tiny));
-    if constexpr (std::is_void<ReturnType>::value) {
-      fn(static_cast<Args&&>(args)...);
-    } else {
-      return fn(static_cast<Args&&>(args)...);
-    }
-  }
-
-  template <typename Fun>
-  static ReturnType callBig(CallArg<Args>... args, Data& p) {
-    auto& fn = *static_cast<Fun*>(p.big);
-    if constexpr (std::is_void<ReturnType>::value) {
-      fn(static_cast<Args&&>(args)...);
-    } else {
-      return fn(static_cast<Args&&>(args)...);
-    }
-  }
+  template <typename Fun, bool Small>
+  static constexpr Call call =
+      call_<Fun, Small, false, ReturnType, CallArg<Args>...>;
 
   static ReturnType uninitCall(CallArg<Args>..., Data&) {
     throw_exception<std::bad_function_call>();
@@ -382,25 +384,9 @@ struct FunctionTraits<ReturnType(Args...) const> {
   template <typename F, typename R = CallableResult<const F&, Args...>>
   using IfSafeResult = IfSafeResultImpl<R, ReturnType>;
 
-  template <typename Fun>
-  static ReturnType callSmall(CallArg<Args>... args, Data& p) {
-    auto& fn = *static_cast<const Fun*>(static_cast<void*>(&p.tiny));
-    if constexpr (std::is_void<ReturnType>::value) {
-      fn(static_cast<Args&&>(args)...);
-    } else {
-      return fn(static_cast<Args&&>(args)...);
-    }
-  }
-
-  template <typename Fun>
-  static ReturnType callBig(CallArg<Args>... args, Data& p) {
-    auto& fn = *static_cast<const Fun*>(p.big);
-    if constexpr (std::is_void<ReturnType>::value) {
-      fn(static_cast<Args&&>(args)...);
-    } else {
-      return fn(static_cast<Args&&>(args)...);
-    }
-  }
+  template <typename Fun, bool Small>
+  static constexpr Call call =
+      call_<const Fun, Small, false, ReturnType, CallArg<Args>...>;
 
   static ReturnType uninitCall(CallArg<Args>..., Data&) {
     throw_exception<std::bad_function_call>();
@@ -428,25 +414,9 @@ struct FunctionTraits<ReturnType(Args...) noexcept> {
       std::enable_if_t<CallableNoexcept<F&, Args...>, int> = 0>
   using IfSafeResult = IfSafeResultImpl<R, ReturnType>;
 
-  template <typename Fun>
-  static ReturnType callSmall(CallArg<Args>... args, Data& p) noexcept {
-    auto& fn = *static_cast<Fun*>(static_cast<void*>(&p.tiny));
-    if constexpr (std::is_void<ReturnType>::value) {
-      fn(static_cast<Args&&>(args)...);
-    } else {
-      return fn(static_cast<Args&&>(args)...);
-    }
-  }
-
-  template <typename Fun>
-  static ReturnType callBig(CallArg<Args>... args, Data& p) noexcept {
-    auto& fn = *static_cast<Fun*>(p.big);
-    if constexpr (std::is_void<ReturnType>::value) {
-      fn(static_cast<Args&&>(args)...);
-    } else {
-      return fn(static_cast<Args&&>(args)...);
-    }
-  }
+  template <typename Fun, bool Small>
+  static constexpr Call call =
+      call_<Fun, Small, true, ReturnType, CallArg<Args>...>;
 
   static ReturnType uninitCall(CallArg<Args>..., Data&) noexcept {
     terminate_with<std::bad_function_call>();
@@ -474,25 +444,9 @@ struct FunctionTraits<ReturnType(Args...) const noexcept> {
       std::enable_if_t<CallableNoexcept<const F&, Args...>, int> = 0>
   using IfSafeResult = IfSafeResultImpl<R, ReturnType>;
 
-  template <typename Fun>
-  static ReturnType callSmall(CallArg<Args>... args, Data& p) noexcept {
-    auto& fn = *static_cast<const Fun*>(static_cast<void*>(&p.tiny));
-    if constexpr (std::is_void<ReturnType>::value) {
-      fn(static_cast<Args&&>(args)...);
-    } else {
-      return fn(static_cast<Args&&>(args)...);
-    }
-  }
-
-  template <typename Fun>
-  static ReturnType callBig(CallArg<Args>... args, Data& p) noexcept {
-    auto& fn = *static_cast<const Fun*>(p.big);
-    if constexpr (std::is_void<ReturnType>::value) {
-      fn(static_cast<Args&&>(args)...);
-    } else {
-      return fn(static_cast<Args&&>(args)...);
-    }
-  }
+  template <typename Fun, bool Small>
+  static constexpr Call call =
+      call_<const Fun, Small, true, ReturnType, CallArg<Args>...>;
 
   static ReturnType uninitCall(CallArg<Args>..., Data&) noexcept {
     terminate_with<std::bad_function_call>();
@@ -522,8 +476,8 @@ struct FunctionTraits<ReturnType(Args...) const noexcept> {
 // need. But it is only necessary to handle those sizes which are multiples of
 // the alignof(Data), and to round up other sizes.
 struct DispatchSmallTrivial {
-  template <typename Fun, typename Base>
-  static constexpr auto call = Base::template callSmall<Fun>;
+  static constexpr bool is_in_situ = true;
+  static constexpr bool is_trivial = true;
 
   template <std::size_t Size>
   static std::size_t exec_(Op o, Data* src, Data* dst) noexcept {
@@ -544,9 +498,57 @@ struct DispatchSmallTrivial {
   static constexpr auto exec = exec_<size_<sizeof(Fun)>>;
 };
 
-struct DispatchSmall {
+struct DispatchBigTrivial {
+  static constexpr bool is_in_situ = false;
+  static constexpr bool is_trivial = true;
+
   template <typename Fun, typename Base>
-  static constexpr auto call = Base::template callSmall<Fun>;
+  static constexpr auto call = Base::template callBig<Fun>;
+
+  static constexpr bool is_align_large(size_t align) {
+    return align > __STDCPP_DEFAULT_NEW_ALIGNMENT__;
+  }
+
+  template <bool IsAlignLarge>
+  static std::size_t exec_(Op o, Data* src, Data* dst) noexcept {
+    switch (o) {
+      case Op::MOVE:
+        dst->bigt = src->bigt;
+        src->bigt = {};
+        break;
+      case Op::NUKE:
+        IsAlignLarge
+            ? operator_delete(
+                  src->big, src->bigt.size, std::align_val_t(src->bigt.align))
+            : operator_delete(src->big, src->bigt.size);
+        break;
+      case Op::HEAP:
+        break;
+    }
+    return src->bigt.size;
+  }
+  template <typename T>
+  static constexpr auto exec = exec_<is_align_large(alignof(T))>;
+
+  FOLLY_ALWAYS_INLINE static void ctor(
+      Data& data,
+      void const* fun,
+      std::size_t size,
+      std::size_t align) noexcept {
+    // cannot use type-specific new since type-specific new is overrideable
+    // in concert with type-specific delete
+    data.bigt.big = is_align_large(align)
+        ? operator_new(size, std::align_val_t(align))
+        : operator_new(size);
+    data.bigt.size = size;
+    data.bigt.align = align;
+    std::memcpy(data.bigt.big, fun, size);
+  }
+};
+
+struct DispatchSmall {
+  static constexpr bool is_in_situ = true;
+  static constexpr bool is_trivial = false;
 
   template <typename Fun>
   static std::size_t exec(Op o, Data* src, Data* dst) noexcept {
@@ -554,7 +556,7 @@ struct DispatchSmall {
       case Op::MOVE:
         ::new (static_cast<void*>(&dst->tiny)) Fun(static_cast<Fun&&>(
             *static_cast<Fun*>(static_cast<void*>(&src->tiny))));
-        FOLLY_FALLTHROUGH;
+        [[fallthrough]];
       case Op::NUKE:
         static_cast<Fun*>(static_cast<void*>(&src->tiny))->~Fun();
         break;
@@ -566,8 +568,8 @@ struct DispatchSmall {
 };
 
 struct DispatchBig {
-  template <typename Fun, typename Base>
-  static constexpr auto call = Base::template callBig<Fun>;
+  static constexpr bool is_in_situ = false;
+  static constexpr bool is_trivial = false;
 
   template <typename Fun>
   static std::size_t exec(Op o, Data* src, Data* dst) noexcept {
@@ -586,6 +588,26 @@ struct DispatchBig {
   }
 };
 
+template <bool InSitu, bool IsTriv>
+struct Dispatch;
+template <>
+struct Dispatch<true, true> : DispatchSmallTrivial {};
+template <>
+struct Dispatch<true, false> : DispatchSmall {};
+template <>
+struct Dispatch<false, true> : DispatchBigTrivial {};
+template <>
+struct Dispatch<false, false> : DispatchBig {};
+
+template <
+    typename Fun,
+    bool InSituSize = sizeof(Fun) <= sizeof(Data),
+    bool InSituAlign = alignof(Fun) <= alignof(Data),
+    bool InSituNoexcept = noexcept(Fun(FOLLY_DECLVAL(Fun)))>
+using DispatchOf = Dispatch<
+    InSituSize && InSituAlign && InSituNoexcept,
+    std::is_trivially_copyable_v<Fun>>;
+
 } // namespace function
 } // namespace detail
 
@@ -627,7 +649,7 @@ class Function final : private detail::function::FunctionTraits<FunctionType> {
     using Fun = Function<Signature>;
     if (fun) {
       data_.big = new Fun(static_cast<Fun&&>(fun));
-      call_ = Traits::template callBig<Fun>;
+      call_ = Traits::template call<Fun, false>;
       exec_ = Exec(detail::function::DispatchBig::exec<Fun>);
     }
   }
@@ -679,11 +701,11 @@ class Function final : private detail::function::FunctionTraits<FunctionType> {
    * from being selected by overload resolution when `fun` is not a compatible
    * function.
    *
-   * \note The noexcept requires some explanation. `IsSmall` is true when the
+   * \note The noexcept requires some explanation. `is_in_situ` is true when the
    * decayed type fits within the internal buffer and is noexcept-movable. But
    * this ctor might copy, not move. What we need here, if this ctor does a
    * copy, is that this ctor be noexcept when the copy is noexcept. That is not
-   * checked in `IsSmall`, and shouldn't be, because once the `Function` is
+   * checked in `is_in_situ`, and shouldn't be, because once the `Function` is
    * constructed, the contained object is never copied. This check is for this
    * ctor only, in the case that this ctor does a copy.
    *
@@ -697,33 +719,28 @@ class Function final : private detail::function::FunctionTraits<FunctionType> {
       typename Fun,
       typename =
           std::enable_if_t<!detail::is_similar_instantiation_v<Function, Fun>>,
-      typename = typename Traits::template IfSafeResult<Fun>,
-      bool IsSmall = ( //
-          sizeof(Fun) <= sizeof(Data) && //
-          alignof(Fun) <= alignof(Data) && //
-              noexcept(Fun(FOLLY_DECLVAL(Fun))))>
-  /* implicit */ constexpr Function(Fun fun) noexcept(IsSmall) {
-    using Dispatch = conditional_t<
-        IsSmall && is_trivially_copyable_v<Fun>,
-        detail::function::DispatchSmallTrivial,
-        conditional_t<
-            IsSmall,
-            detail::function::DispatchSmall,
-            detail::function::DispatchBig>>;
+      typename = typename Traits::template IfSafeResult<Fun>>
+  /* implicit */ constexpr Function(Fun fun) noexcept(
+      detail::function::DispatchOf<Fun>::is_in_situ) {
+    using Dispatch = detail::function::DispatchOf<Fun>;
     if constexpr (detail::function::IsNullptrCompatible<Fun>) {
       if (detail::function::isEmptyFunction(fun)) {
         return;
       }
     }
-    if constexpr (IsSmall) {
+    if constexpr (Dispatch::is_in_situ) {
       if constexpr (
-          !std::is_empty<Fun>::value || !is_trivially_copyable_v<Fun>) {
+          !std::is_empty<Fun>::value || !std::is_trivially_copyable_v<Fun>) {
         ::new (&data_.tiny) Fun(static_cast<Fun&&>(fun));
       }
     } else {
-      data_.big = new Fun(static_cast<Fun&&>(fun));
+      if constexpr (Dispatch::is_trivial) {
+        Dispatch::ctor(data_, &fun, sizeof(Fun), alignof(Fun));
+      } else {
+        data_.big = new Fun(static_cast<Fun&&>(fun));
+      }
     }
-    call_ = Dispatch::template call<Fun, Traits>;
+    call_ = Traits::template call<Fun, Dispatch::is_in_situ>;
     exec_ = Exec(Dispatch::template exec<Fun>);
   }
 
@@ -874,7 +891,7 @@ class Function final : private detail::function::FunctionTraits<FunctionType> {
    * allocation because the callable is stored within the `Function` object.
    */
   std::size_t heapAllocatedMemory() const noexcept {
-    return exec(Op::HEAP, nullptr, nullptr);
+    return exec(Op::HEAP, &data_, nullptr);
   }
 
   using typename Traits::SharedProxy;
