@@ -229,41 +229,6 @@ size_t allocationBytesForOverAligned(size_t n) {
   }
 }
 
-/**
- * For exception safety and consistency with make_shared. Erase me when
- * we have std::make_unique().
- *
- * @author Louis Brandy (ldbrandy@fb.com)
- * @author Xu Ning (xning@fb.com)
- */
-
-#if __cplusplus >= 201402L || __cpp_lib_make_unique >= 201304L || \
-    (__ANDROID__ && __cplusplus >= 201300L) || _MSC_VER >= 1900
-
-/* using override */ using std::make_unique;
-
-#else
-
-template <typename T, typename... Args>
-typename std::enable_if<!std::is_array<T>::value, std::unique_ptr<T>>::type
-make_unique(Args&&... args) {
-  return std::unique_ptr<T>(new T(std::forward<Args>(args)...));
-}
-
-// Allows 'make_unique<T[]>(10)'. (N3690 s20.9.1.4 p3-4)
-template <typename T>
-typename std::enable_if<std::is_array<T>::value, std::unique_ptr<T>>::type
-make_unique(const size_t n) {
-  return std::unique_ptr<T>(new typename std::remove_extent<T>::type[n]());
-}
-
-// Disallows 'make_unique<T[10]>()'. (N3690 s20.9.1.4 p5)
-template <typename T, typename... Args>
-typename std::enable_if<std::extent<T>::value != 0, std::unique_ptr<T>>::type
-make_unique(Args&&...) = delete;
-
-#endif
-
 /**
  * static_function_deleter
  *
@@ -399,11 +364,11 @@ std::weak_ptr<U> to_weak_ptr_aliasing(const std::shared_ptr<T>& r, U* ptr) {
  *
  *  Move or copy the argument to the heap and return it owned by a unique_ptr.
  *
- *  Like make_unique, but deduces the type of the owned object.
+ *  Like std::make_unique, but deduces the type of the owned object.
  */
 template <typename T>
 std::unique_ptr<remove_cvref_t<T>> copy_to_unique_ptr(T&& t) {
-  return make_unique<remove_cvref_t<T>>(static_cast<T&&>(t));
+  return std::make_unique<remove_cvref_t<T>>(static_cast<T&&>(t));
 }
 
 /**
@@ -484,21 +449,6 @@ inline erased_unique_ptr empty_erased_unique_ptr() {
   return {nullptr, nullptr};
 }
 
-//  reinterpret_pointer_cast
-//
-//  import or backport
-#if FOLLY_CPLUSPLUS >= 201703L && __cpp_lib_shared_ptr_arrays >= 201611
-using std::reinterpret_pointer_cast;
-#else
-template <typename T, typename U>
-std::shared_ptr<T> reinterpret_pointer_cast(
-    const std::shared_ptr<U>& r) noexcept {
-  auto p =
-      reinterpret_cast<typename std::shared_ptr<T>::element_type*>(r.get());
-  return std::shared_ptr<T>{r, p};
-}
-#endif
-
 /**
  * SysAllocator
  *
