@@ -14,6 +14,10 @@
  * limitations under the License.
  */
 
+//
+// Docs: https://fburl.com/fbcref_optional
+//
+
 #pragma once
 
 /*
@@ -54,16 +58,14 @@
  *  }
  */
 
+#include <cassert>
 #include <cstddef>
 #include <functional>
 #include <new>
+#include <optional>
 #include <stdexcept>
 #include <type_traits>
 #include <utility>
-#if __cplusplus >= 201703L && __has_include(<optional>)
-#define FOLLY_HAS_STD_OPTIONAL
-#include <optional>
-#endif
 
 #include <folly/Portability.h>
 #include <folly/Traits.h>
@@ -76,6 +78,7 @@ template <class Value>
 class Optional;
 
 namespace detail {
+struct OptionalEmptyTag {};
 template <class Value>
 struct OptionalPromise;
 template <class Value>
@@ -150,18 +153,18 @@ class Optional {
   /**
    * Creates an Optional with a value, where that value is constructed in-place.
    *
-   * The in_place_t argument exists so that values can be default constructed
+   * The std::in_place argument exists so that values can be default constructed
    * (i.e. have no arguments), since this would otherwise be confused with
    * default-constructing an Optional, which in turn results in None.
    */
   template <typename... Args>
-  constexpr explicit Optional(in_place_t, Args&&... args) noexcept(
+  constexpr explicit Optional(std::in_place_t, Args&&... args) noexcept(
       std::is_nothrow_constructible<Value, Args...>::value)
       : Optional{PrivateConstructor{}, std::forward<Args>(args)...} {}
 
   template <typename U, typename... Args>
   constexpr explicit Optional(
-      in_place_t,
+      std::in_place_t,
       std::initializer_list<U> il,
       Args&&... args) noexcept(std::
                                    is_nothrow_constructible<
@@ -176,8 +179,8 @@ class Optional {
     p.promise_->value_ = this;
   }
 
-// Conversions to ease migration to std::optional
-#ifdef FOLLY_HAS_STD_OPTIONAL
+  // Conversions to ease migration to std::optional
+
   /// Allow construction of Optional from std::optional.
   template <
       typename U,
@@ -198,7 +201,7 @@ class Optional {
       construct(*newValue);
     }
   }
-  /// Allow implict cast to std::optional
+  /// Allow explicit cast to std::optional
   /// @methodset Migration
   explicit operator std::optional<Value>() && noexcept(
       std::is_nothrow_move_constructible<Value>::value) {
@@ -213,7 +216,14 @@ class Optional {
     return storage_.hasValue ? std::optional<Value>(storage_.value)
                              : std::nullopt;
   }
-#endif
+
+  std::optional<Value> toStdOptional() && noexcept {
+    return static_cast<std::optional<Value>>(std::move(*this));
+  }
+
+  std::optional<Value> toStdOptional() const& noexcept {
+    return static_cast<std::optional<Value>>(*this);
+  }
 
   /// Set the Optional
   /// @methodset Modifiers
@@ -306,7 +316,7 @@ class Optional {
   void clear() noexcept { reset(); }
 
   /// @methodset Modifiers
-  void swap(Optional& that) noexcept(IsNothrowSwappable<Value>::value) {
+  void swap(Optional& that) noexcept(std::is_nothrow_swappable_v<Value>) {
     if (hasValue() && that.hasValue()) {
       using std::swap;
       swap(value(), that.value());
@@ -396,6 +406,8 @@ class Optional {
   }
 
  private:
+  friend struct detail::OptionalPromiseReturn<Value>;
+
   template <class T>
   friend constexpr Optional<std::decay_t<T>> make_optional(T&&);
   template <class T, class... Args>
@@ -419,6 +431,10 @@ class Optional {
       std::is_nothrow_constructible<Value, Args&&...>::value) {
     construct(std::forward<Args>(args)...);
   }
+  // for when coroutine promise return-object conversion is eager
+  explicit Optional(detail::OptionalEmptyTag, Optional*& pointer) noexcept {
+    pointer = this;
+  }
 
   void require_value() const {
     if (!storage_.hasValue) {
@@ -639,12 +655,10 @@ constexpr bool operator>=(None, const Optional<V>& a) noexcept {
 // Allow usage of Optional<T> in std::unordered_map and std::unordered_set
 FOLLY_NAMESPACE_STD_BEGIN
 template <class T>
-struct hash<folly::Optional<T>> {
+struct hash<
+    folly::enable_std_hash_helper<folly::Optional<T>, remove_const_t<T>>> {
   size_t operator()(folly::Optional<T> const& obj) const {
-    if (!obj.hasValue()) {
-      return 0;
-    }
-    return hash<typename remove_const<T>::type>()(*obj);
+    return static_cast<bool>(obj) ? hash<remove_const_t<T>>()(*obj) : 0;
   }
 };
 FOLLY_NAMESPACE_STD_END
@@ -662,15 +676,26 @@ struct OptionalPromise;
 template <typename Value>
 struct OptionalPromiseReturn {
   Optional<Value> storage_;
-  OptionalPromise<Value>* promise_;
-  /* implicit */ OptionalPromiseReturn(OptionalPromise<Value>& promise) noexcept
-      : promise_(&promise) {
-    promise.value_ = &storage_;
+  Optional<Value>*& pointer_;
+
+  /* implicit */ OptionalPromiseReturn(OptionalPromise<Value>& p) noexcept
+      : pointer_{p.value_} {
+    pointer_ = &storage_;
   }
-  OptionalPromiseReturn(OptionalPromiseReturn&& that) noexcept
-      : OptionalPromiseReturn{*that.promise_} {}
+  OptionalPromiseReturn(OptionalPromiseReturn const&) = delete;
+  // letting dtor be trivial makes the coroutine crash
+  // TODO: fix clang/llvm codegen
   ~OptionalPromiseReturn() {}
-  /* implicit */ operator Optional<Value>() & { return std::move(storage_); }
+  /* implicit */ operator Optional<Value>() {
+    // handle both deferred and eager return-object conversion behaviors
+    // see docs for detect_promise_return_object_eager_conversion
+    if (folly::coro::detect_promise_return_object_eager_conversion()) {
+      assert(!storage_.has_value());
+      return Optional{OptionalEmptyTag{}, pointer_}; // eager
+    } else {
+      return std::move(storage_); // deferred
+    }
+  }
 };
 
 template <typename Value>
@@ -678,10 +703,6 @@ struct OptionalPromise {
   Optional<Value>* value_ = nullptr;
   OptionalPromise() = default;
   OptionalPromise(OptionalPromise const&) = delete;
-  // This should work regardless of whether the compiler generates:
-  //    folly::Optional<Value> retobj{ p.get_return_object(); } // MSVC
-  // or:
-  //    auto retobj = p.get_return_object(); // clang
   OptionalPromiseReturn<Value> get_return_object() noexcept { return *this; }
   coro::suspend_never initial_suspend() const noexcept { return {}; }
   coro::suspend_never final_suspend() const noexcept { return {}; }
