@@ -14,8 +14,6 @@
  * limitations under the License.
  */
 
-// @author Nathan Bronson (ngbronson@fb.com)
-
 #pragma once
 
 #include <stdint.h>
@@ -30,6 +28,7 @@
 #include <utility>
 
 #include <folly/CPortability.h>
+#include <folly/CppAttributes.h>
 #include <folly/Likely.h>
 #include <folly/chrono/Hardware.h>
 #include <folly/concurrency/CacheLocality.h>
@@ -278,7 +277,8 @@ std::unique_lock<std::mutex> annotationGuard(void* ptr);
 
 constexpr uint32_t kMaxDeferredReadersAllocated = 256 * 2;
 
-FOLLY_COLD uint32_t getMaxDeferredReadersSlow(relaxed_atomic<uint32_t>& cache);
+[[FOLLY_ATTR_GNU_COLD]] uint32_t getMaxDeferredReadersSlow(
+    relaxed_atomic<uint32_t>& cache);
 
 long getCurrentThreadInvoluntaryContextSwitchCount();
 
@@ -359,10 +359,6 @@ class SharedMutexImpl : std::conditional_t<
 
   typedef SharedMutexToken Token;
 
-  class FOLLY_NODISCARD ReadHolder;
-  class FOLLY_NODISCARD UpgradeHolder;
-  class FOLLY_NODISCARD WriteHolder;
-
   constexpr SharedMutexImpl() noexcept : state_(0) {}
 
   SharedMutexImpl(const SharedMutexImpl&) = delete;
@@ -1210,6 +1206,7 @@ class SharedMutexImpl : std::conditional_t<
     }
   }
 
+  [[FOLLY_ATTR_GNU_USED]]
   void wakeRegisteredWaitersImpl(uint32_t& state, uint32_t wakeMask) {
     // If there are multiple lock() pending only one of them will actually
     // get to wake up, so issuing futexWakeAll will make a thundering herd.
@@ -1417,200 +1414,6 @@ class SharedMutexImpl : std::conditional_t<
     } while (!state_.compare_exchange_strong(state, state | kHasU));
     return true;
   }
-
- public:
-  FOLLY_PUSH_WARNING
-
-  FOLLY_GNU_DISABLE_WARNING("-Wdeprecated-declarations")
-
-  class FOLLY_NODISCARD FOLLY_DEPRECATED("use std::shared_lock") ReadHolder {
-    using folly_is_unsafe_for_async_usage = std::true_type;
-
-    ReadHolder() : lock_(nullptr) {}
-
-   public:
-    explicit ReadHolder(const SharedMutexImpl* lock)
-        : lock_(const_cast<SharedMutexImpl*>(lock)) {
-      if (lock_) {
-        lock_->lock_shared(token_);
-      }
-    }
-
-    explicit ReadHolder(const SharedMutexImpl& lock)
-        : lock_(const_cast<SharedMutexImpl*>(&lock)) {
-      lock_->lock_shared(token_);
-    }
-
-    ReadHolder(ReadHolder&& rhs) noexcept
-        : lock_(rhs.lock_), token_(rhs.token_) {
-      rhs.lock_ = nullptr;
-    }
-
-    // Downgrade from upgrade mode
-    explicit ReadHolder(UpgradeHolder&& upgraded) : lock_(upgraded.lock_) {
-      assert(upgraded.lock_ != nullptr);
-      upgraded.lock_ = nullptr;
-      lock_->unlock_upgrade_and_lock_shared(token_);
-    }
-
-    // Downgrade from exclusive mode
-    explicit ReadHolder(WriteHolder&& writer) : lock_(writer.lock_) {
-      assert(writer.lock_ != nullptr);
-      writer.lock_ = nullptr;
-      lock_->unlock_and_lock_shared(token_);
-    }
-
-    ReadHolder& operator=(ReadHolder&& rhs) noexcept {
-      std::swap(lock_, rhs.lock_);
-      std::swap(token_, rhs.token_);
-      return *this;
-    }
-
-    ReadHolder(const ReadHolder& rhs) = delete;
-    ReadHolder& operator=(const ReadHolder& rhs) = delete;
-
-    ~ReadHolder() { unlock(); }
-
-    void unlock() {
-      if (lock_) {
-        lock_->unlock_shared(token_);
-        lock_ = nullptr;
-      }
-    }
-
-   private:
-    friend class UpgradeHolder;
-    friend class WriteHolder;
-    SharedMutexImpl* lock_;
-    SharedMutexToken token_;
-  };
-
-  class FOLLY_NODISCARD FOLLY_DEPRECATED("use folly::upgrade_lock")
-      UpgradeHolder {
-    using folly_is_unsafe_for_async_usage = std::true_type;
-
-    UpgradeHolder() : lock_(nullptr) {}
-
-   public:
-    explicit UpgradeHolder(SharedMutexImpl* lock) : lock_(lock) {
-      if (lock_) {
-        lock_->lock_upgrade();
-      }
-    }
-
-    explicit UpgradeHolder(SharedMutexImpl& lock) : lock_(&lock) {
-      lock_->lock_upgrade();
-    }
-
-    // Downgrade from exclusive mode
-    explicit UpgradeHolder(WriteHolder&& writer) : lock_(writer.lock_) {
-      assert(writer.lock_ != nullptr);
-      writer.lock_ = nullptr;
-      lock_->unlock_and_lock_upgrade();
-    }
-
-    UpgradeHolder(UpgradeHolder&& rhs) noexcept : lock_(rhs.lock_) {
-      rhs.lock_ = nullptr;
-    }
-
-    UpgradeHolder& operator=(UpgradeHolder&& rhs) noexcept {
-      std::swap(lock_, rhs.lock_);
-      return *this;
-    }
-
-    UpgradeHolder(const UpgradeHolder& rhs) = delete;
-    UpgradeHolder& operator=(const UpgradeHolder& rhs) = delete;
-
-    ~UpgradeHolder() { unlock(); }
-
-    void unlock() {
-      if (lock_) {
-        lock_->unlock_upgrade();
-        lock_ = nullptr;
-      }
-    }
-
-   private:
-    friend class WriteHolder;
-    friend class ReadHolder;
-    SharedMutexImpl* lock_;
-  };
-
-  class FOLLY_NODISCARD FOLLY_DEPRECATED("use std::unique_lock") WriteHolder {
-    using folly_is_unsafe_for_async_usage = std::true_type;
-
-    WriteHolder() : lock_(nullptr) {}
-
-   public:
-    explicit WriteHolder(SharedMutexImpl* lock) : lock_(lock) {
-      if (lock_) {
-        lock_->lock();
-      }
-    }
-
-    explicit WriteHolder(SharedMutexImpl& lock) : lock_(&lock) {
-      lock_->lock();
-    }
-
-    // Promotion from upgrade mode
-    explicit WriteHolder(UpgradeHolder&& upgrade) : lock_(upgrade.lock_) {
-      assert(upgrade.lock_ != nullptr);
-      upgrade.lock_ = nullptr;
-      lock_->unlock_upgrade_and_lock();
-    }
-
-    // README:
-    //
-    // It is intended that WriteHolder(ReadHolder&& rhs) do not exist.
-    //
-    // Shared locks (read) can not safely upgrade to unique locks (write).
-    // That upgrade path is a well-known recipe for deadlock, so we explicitly
-    // disallow it.
-    //
-    // If you need to do a conditional mutation, you have a few options:
-    // 1. Check the condition under a shared lock and release it.
-    //    Then maybe check the condition again under a unique lock and maybe do
-    //    the mutation.
-    // 2. Check the condition once under an upgradeable lock.
-    //    Then maybe upgrade the lock to a unique lock and do the mutation.
-    // 3. Check the condition and maybe perform the mutation under a unique
-    //    lock.
-    //
-    // Relevant upgradeable lock notes:
-    // * At most one upgradeable lock can be held at a time for a given shared
-    //   mutex, just like a unique lock.
-    // * An upgradeable lock may be held concurrently with any number of shared
-    //   locks.
-    // * An upgradeable lock may be upgraded atomically to a unique lock.
-
-    WriteHolder(WriteHolder&& rhs) noexcept : lock_(rhs.lock_) {
-      rhs.lock_ = nullptr;
-    }
-
-    WriteHolder& operator=(WriteHolder&& rhs) noexcept {
-      std::swap(lock_, rhs.lock_);
-      return *this;
-    }
-
-    WriteHolder(const WriteHolder& rhs) = delete;
-    WriteHolder& operator=(const WriteHolder& rhs) = delete;
-
-    ~WriteHolder() { unlock(); }
-
-    void unlock() {
-      if (lock_) {
-        lock_->unlock();
-        lock_ = nullptr;
-      }
-    }
-
-    FOLLY_POP_WARNING
-
-   private:
-    friend class ReadHolder;
-    friend class UpgradeHolder;
-    SharedMutexImpl* lock_;
-  };
 };
 
 using SharedMutexReadPriority = SharedMutexImpl<true>;
