@@ -14,6 +14,10 @@
  * limitations under the License.
  */
 
+//
+// Docs: https://fburl.com/fbcref_singleton
+//
+
 // SingletonVault - a library to manage the creation and destruction
 // of interdependent singletons.
 //
@@ -153,11 +157,6 @@
 
 #include <glog/logging.h>
 
-// use this guard to handleSingleton breaking change in 3rd party code
-#ifndef FOLLY_SINGLETON_TRY_GET
-#define FOLLY_SINGLETON_TRY_GET
-#endif
-
 namespace folly {
 
 // For actual usage, please see the Singleton<T> class at the bottom
@@ -274,6 +273,8 @@ struct SingletonVaultState {
       throw_exception<std::logic_error>(msg);
     }
   }
+
+  bool isDisabled() const { return state == Type::Quiescing; }
 };
 
 // This interface is used by SingletonVault to interact with SingletonHolders.
@@ -546,6 +547,8 @@ class SingletonVault {
     failOnUseAfterFork_ = failOnUseAfterFork;
   }
 
+  bool isDisabled() const { return state_.rlock()->isDisabled(); }
+
  private:
   template <typename T>
   friend struct detail::SingletonHolder;
@@ -859,7 +862,7 @@ class LeakySingleton {
 
   static T& instance() {
     auto& entry = entryInstance();
-    if (UNLIKELY(entry.state != State::Living)) {
+    if (FOLLY_UNLIKELY(entry.state != State::Living)) {
       createInstance();
     }
 
