@@ -52,23 +52,23 @@ namespace folly {
 namespace detail {
 
 template <typename, typename Mutex>
-FOLLY_INLINE_VARIABLE constexpr bool kSynchronizedMutexIsUnique = false;
+inline constexpr bool kSynchronizedMutexIsUnique = false;
 template <typename Mutex>
-FOLLY_INLINE_VARIABLE constexpr bool kSynchronizedMutexIsUnique<
+inline constexpr bool kSynchronizedMutexIsUnique<
     decltype(void(std::declval<Mutex&>().lock())),
     Mutex> = true;
 
 template <typename, typename Mutex>
-FOLLY_INLINE_VARIABLE constexpr bool kSynchronizedMutexIsShared = false;
+inline constexpr bool kSynchronizedMutexIsShared = false;
 template <typename Mutex>
-FOLLY_INLINE_VARIABLE constexpr bool kSynchronizedMutexIsShared<
+inline constexpr bool kSynchronizedMutexIsShared<
     decltype(void(std::declval<Mutex&>().lock_shared())),
     Mutex> = true;
 
 template <typename, typename Mutex>
-FOLLY_INLINE_VARIABLE constexpr bool kSynchronizedMutexIsUpgrade = false;
+inline constexpr bool kSynchronizedMutexIsUpgrade = false;
 template <typename Mutex>
-FOLLY_INLINE_VARIABLE constexpr bool kSynchronizedMutexIsUpgrade<
+inline constexpr bool kSynchronizedMutexIsUpgrade<
     decltype(void(std::declval<Mutex&>().lock_upgrade())),
     Mutex> = true;
 
@@ -82,7 +82,7 @@ FOLLY_INLINE_VARIABLE constexpr bool kSynchronizedMutexIsUpgrade<
 enum class SynchronizedMutexLevel { Unknown, Unique, Shared, Upgrade };
 
 template <typename Mutex>
-FOLLY_INLINE_VARIABLE constexpr SynchronizedMutexLevel kSynchronizedMutexLevel =
+inline constexpr SynchronizedMutexLevel kSynchronizedMutexLevel =
     kSynchronizedMutexIsUpgrade<void, Mutex>  ? SynchronizedMutexLevel::Upgrade
     : kSynchronizedMutexIsShared<void, Mutex> ? SynchronizedMutexLevel::Shared
     : kSynchronizedMutexIsUnique<void, Mutex> ? SynchronizedMutexLevel::Unique
@@ -708,7 +708,7 @@ struct Synchronized : public SynchronizedBase<
    * instance `in_place` as the first argument.
    */
   template <typename... Args>
-  explicit constexpr Synchronized(in_place_t, Args&&... args)
+  explicit constexpr Synchronized(std::in_place_t, Args&&... args)
       : datum_(std::forward<Args>(args)...) {}
 
   /**
@@ -1512,7 +1512,7 @@ class LockedPtr {
     using simulacrum = typename SynchronizedType::Simulacrum;
     static_assert(sizeof(simulacrum) == sizeof(SynchronizedType), "mismatch");
     static_assert(alignof(simulacrum) == alignof(SynchronizedType), "mismatch");
-    constexpr auto off = offsetof(simulacrum, mutex_);
+    auto off = offsetof(simulacrum, mutex_);
     const auto raw = reinterpret_cast<char*>(lock_.mutex());
     return reinterpret_cast<SynchronizedType*>(raw - (raw ? off : 0));
   }
@@ -1738,8 +1738,9 @@ void swap(Synchronized<T, M>& lhs, Synchronized<T, M>& rhs) {
 
 namespace detail {
 struct [[deprecated(
-    "use explicit lock(), wlock(), or rlock() instead")]] SYNCHRONIZED_macro_is_deprecated{};
-}
+    "use explicit lock(), wlock(), or rlock() instead")]] SYNCHRONIZED_macro_is_deprecated {
+};
+} // namespace detail
 
 /**
  * NOTE: This API is deprecated.  Use lock(), wlock(), rlock() or the withLock
@@ -1794,7 +1795,7 @@ struct [[deprecated(
 #define SYNCHRONIZED_CONST(...)            \
   SYNCHRONIZED(                            \
       FB_VA_GLUE(FB_ARG_1, (__VA_ARGS__)), \
-      as_const(FB_VA_GLUE(FB_ARG_2_OR_1, (__VA_ARGS__))))
+      std::as_const(FB_VA_GLUE(FB_ARG_2_OR_1, (__VA_ARGS__))))
 
 /**
  * NOTE: This API is deprecated.  Use lock(), wlock(), rlock() or the withLock
