@@ -34,8 +34,6 @@
  * There are two classes here - ThreadLocal and ThreadLocalPtr.  ThreadLocalPtr
  * has semantics similar to boost::thread_specific_ptr. ThreadLocal is a thin
  * wrapper around ThreadLocalPtr that manages allocation automatically.
- *
- * @author Spencer Ahrens (sahrens)
  */
 
 #pragma once
@@ -59,9 +57,9 @@ class ThreadLocalPtr;
 template <class T, class Tag = void, class AccessMode = void>
 class ThreadLocal {
  public:
-  constexpr ThreadLocal() : constructor_([]() { return new T(); }) {}
+  constexpr ThreadLocal() : constructor_([]() { return T(); }) {}
 
-  template <typename F, std::enable_if_t<is_invocable_r_v<T*, F>, int> = 0>
+  template <typename F, std::enable_if_t<is_invocable_r_v<T, F>, int> = 0>
   explicit ThreadLocal(F&& constructor)
       : constructor_(std::forward<F>(constructor)) {}
 
@@ -92,13 +90,13 @@ class ThreadLocal {
   ThreadLocal& operator=(const ThreadLocal&) = delete;
 
   FOLLY_NOINLINE T* makeTlp() const {
-    auto const ptr = static_cast<T*>(constructor_());
+    auto const ptr = new T(constructor_());
     tlp_.reset(ptr);
     return ptr;
   }
 
   mutable ThreadLocalPtr<T, Tag, AccessMode> tlp_;
-  std::function<void*()> constructor_;
+  std::function<T()> constructor_;
 };
 
 /*
@@ -159,25 +157,22 @@ class ThreadLocalPtr {
 
   T* release() {
     auto rlock = getAccessAllThreadsLockReadHolderIfEnabled();
-
-    threadlocal_detail::ElementWrapper& w = StaticMeta::get(&id_);
-
-    return static_cast<T*>(w.release());
+    threadlocal_detail::ThreadEntry* te = StaticMeta::getThreadEntry(&id_);
+    auto id = id_.getOrInvalid();
+    // Only valid index into the the elements array
+    DCHECK_NE(id, threadlocal_detail::kEntryIDInvalid);
+    return static_cast<T*>(te->releaseElement(id));
   }
 
   void reset(T* newPtr = nullptr) {
     auto rlock = getAccessAllThreadsLockReadHolderIfEnabled();
-
     auto guard = makeGuard([&] { delete newPtr; });
-    threadlocal_detail::ElementWrapper* w = &StaticMeta::get(&id_);
-
-    w->dispose(TLPDestructionMode::THIS_THREAD);
-    // need to get a new ptr since the
-    // ThreadEntry::elements array can be reallocated
-    w = &StaticMeta::get(&id_);
-    w->cleanup();
+    threadlocal_detail::ThreadEntry* te = StaticMeta::getThreadEntry(&id_);
+    uint32_t id = id_.getOrInvalid();
+    // Only valid index into the the elements array
+    DCHECK_NE(id, threadlocal_detail::kEntryIDInvalid);
+    te->resetElement(newPtr, id);
     guard.dismiss();
-    w->set(newPtr);
   }
 
   explicit operator bool() const { return get() != nullptr; }
@@ -219,20 +214,18 @@ class ThreadLocalPtr {
   template <class Deleter>
   void reset(T* newPtr, const Deleter& deleter) {
     auto rlock = getAccessAllThreadsLockReadHolderIfEnabled();
-
     auto guard = makeGuard([&] {
       if (newPtr) {
         deleter(newPtr, TLPDestructionMode::THIS_THREAD);
       }
     });
-    threadlocal_detail::ElementWrapper* w = &StaticMeta::get(&id_);
-    w->dispose(TLPDestructionMode::THIS_THREAD);
-    // need to get a new ptr since the
-    // ThreadEntry::elements array can be reallocated
-    w = &StaticMeta::get(&id_);
-    w->cleanup();
+
+    threadlocal_detail::ThreadEntry* te = StaticMeta::getThreadEntry(&id_);
+    uint32_t id = id_.getOrInvalid();
+    // Only valid index into the the elements array
+    DCHECK_NE(id, threadlocal_detail::kEntryIDInvalid);
+    te->resetElement(newPtr, deleter, id);
     guard.dismiss();
-    w->set(newPtr, deleter);
   }
 
   // Holds a global lock for iteration through all thread local child objects.
@@ -250,57 +243,67 @@ class ThreadLocalPtr {
     class Iterator;
     friend class Iterator;
 
-    // The iterators obtained from Accessor are bidirectional iterators.
+    // The iterators obtained from Accessor are forward iterators.
     class Iterator {
       friend class Accessor;
       const Accessor* accessor_{nullptr};
-      threadlocal_detail::ThreadEntryNode* e_{nullptr};
+      using InnerVector = threadlocal_detail::ThreadEntrySet::EntryVector;
+      using InnerIterator = InnerVector::iterator;
+
+      InnerVector& vec_;
+      InnerIterator iter_;
 
       void increment() {
-        e_ = e_->getNext();
-        incrementToValid();
+        if (iter_ != vec_.end()) {
+          ++iter_;
+          incrementToValid();
+        }
       }
 
       void decrement() {
-        e_ = e_->getPrev();
-        decrementToValid();
+        if (iter_ != vec_.begin()) {
+          --iter_;
+          decrementToValid();
+        }
       }
 
       const T& dereference() const {
-        return *static_cast<T*>(
-            e_->getThreadEntry()->elements[accessor_->id_].ptr);
+        return *static_cast<T*>((*iter_)->elements[accessor_->id_].ptr);
       }
 
       T& dereference() {
-        return *static_cast<T*>(
-            e_->getThreadEntry()->elements[accessor_->id_].ptr);
+        return *static_cast<T*>((*iter_)->elements[accessor_->id_].ptr);
       }
 
       bool equal(const Iterator& other) const {
-        return (accessor_->id_ == other.accessor_->id_ && e_ == other.e_);
+        return (accessor_->id_ == other.accessor_->id_ && iter_ == other.iter_);
       }
 
-      explicit Iterator(const Accessor* accessor)
+      void setToEnd() { iter_ = vec_.end(); }
+
+      explicit Iterator(const Accessor* accessor, bool toEnd = false)
           : accessor_(accessor),
-            e_(&accessor_->meta_.head_.elements[accessor_->id_].node) {}
+            vec_(accessor_->meta_.allThreadEntryMap_[accessor_->id_]
+                     .threadEntries),
+            iter_(vec_.begin()) {
+        if (toEnd) {
+          setToEnd();
+        }
+      }
 
       // we just need to check the ptr since it can be set to nullptr
       // even if the entry is part of the list
       bool valid() const {
-        return (e_->getThreadEntry()->elements[accessor_->id_].ptr);
+        return (iter_ != vec_.end() && (*iter_)->elements[accessor_->id_].ptr);
       }
 
       void incrementToValid() {
-        for (; e_ != &accessor_->meta_.head_.elements[accessor_->id_].node &&
-             !valid();
-             e_ = e_->getNext()) {
+        for (; iter_ != vec_.end() && !valid(); ++iter_) {
         }
       }
 
       void decrementToValid() {
-        for (; e_ != &accessor_->meta_.head_.elements[accessor_->id_].node &&
-             !valid();
-             e_ = e_->getPrev()) {
+        for (; iter_ != vec_.begin() && !valid(); --iter_) {
         }
       }
 
@@ -347,18 +350,16 @@ class ThreadLocalPtr {
 
       bool operator!=(Iterator const& rhs) const { return !equal(rhs); }
 
-      std::thread::id getThreadId() const {
-        return e_->getThreadEntry()->tid();
-      }
+      std::thread::id getThreadId() const { return (*iter_)->tid(); }
 
-      uint64_t getOSThreadId() const { return e_->getThreadEntry()->tid_os; }
+      uint64_t getOSThreadId() const { return (*iter_)->tid_os; }
     };
 
     ~Accessor() { release(); }
 
-    Iterator begin() const { return ++Iterator(this); }
+    Iterator begin() const { return Iterator(this); }
 
-    Iterator end() const { return Iterator(this); }
+    Iterator end() const { return Iterator(this, true); }
 
     Accessor(const Accessor&) = delete;
     Accessor& operator=(const Accessor&) = delete;
