@@ -33,7 +33,7 @@ inline std::uint64_t hardware_timestamp() {
   return __rdtsc();
 #elif defined(__GNUC__) && (defined(__i386__) || FOLLY_X64)
   return __builtin_ia32_rdtsc();
-#elif defined(__aarch64__)
+#elif FOLLY_AARCH64 && !FOLLY_MOBILE
   uint64_t cval;
   asm volatile("mrs %0, cntvct_el0" : "=r"(cval));
   return cval;
