@@ -22,9 +22,6 @@
  * F14FastMap conditionally works like F14ValueMap or F14VectorMap
  *
  * See F14.md
- *
- * @author Nathan Bronson <ngbronson@fb.com>
- * @author Xiao Shi       <xshi@fb.com>
  */
 
 #include <cstddef>
@@ -433,6 +430,13 @@ class F14BasicMap {
     return rv;
   }
 
+  template <typename M>
+  std::pair<iterator, bool> insert_or_assign(
+      const F14HashedKey<key_type, hasher>& hashedKey, M&& obj) {
+    return insert_or_assign(
+        hashedKey.getHashToken(), hashedKey.getKey(), std::forward<M>(obj));
+  }
+
   template <typename M>
   std::pair<iterator, bool> insert_or_assign(
       F14HashToken const& token, key_type&& key, M&& obj) {
@@ -443,6 +447,15 @@ class F14BasicMap {
     return rv;
   }
 
+  template <typename M>
+  std::pair<iterator, bool> insert_or_assign(
+      F14HashedKey<key_type, hasher>&& hashedKey, M&& obj) {
+    return insert_or_assign(
+        hashedKey.getHashToken(),
+        std::move(hashedKey.getKey()),
+        std::forward<M>(obj));
+  }
+
   template <typename M>
   iterator insert_or_assign(
       const_iterator /*hint*/, key_type const& key, M&& obj) {
@@ -537,6 +550,15 @@ class F14BasicMap {
     return std::make_pair(table_.makeIter(rv.first), rv.second);
   }
 
+  template <typename... Args>
+  std::pair<iterator, bool> try_emplace_token(
+      const F14HashedKey<key_type, hasher>& hashedKey, Args&&... args) {
+    return try_emplace_token(
+        hashedKey.getHashToken(),
+        hashedKey.getKey(),
+        std::forward<Args>(args)...);
+  }
+
   template <typename... Args>
   std::pair<iterator, bool> try_emplace_token(
       F14HashToken const& token, key_type&& key, Args&&... args) {
@@ -549,6 +571,15 @@ class F14BasicMap {
     return std::make_pair(table_.makeIter(rv.first), rv.second);
   }
 
+  template <typename... Args>
+  std::pair<iterator, bool> try_emplace_token(
+      F14HashedKey<key_type, hasher>&& hashedKey, Args&&... args) {
+    return try_emplace_token(
+        hashedKey.getHashToken(),
+        std::move(hashedKey.getKey()),
+        std::forward<Args>(args)...);
+  }
+
   template <typename... Args>
   iterator try_emplace(
       const_iterator /*hint*/, key_type const& key, Args&&... args) {
@@ -833,11 +864,23 @@ class F14BasicMap {
     return table_.makeIter(table_.find(token, key));
   }
 
+  FOLLY_ALWAYS_INLINE iterator
+  find(const F14HashedKey<key_type, hasher>& hashedKey) {
+    return table_.makeIter(
+        table_.find(hashedKey.getHashToken(), hashedKey.getKey()));
+  }
+
   FOLLY_ALWAYS_INLINE const_iterator
   find(F14HashToken const& token, key_type const& key) const {
     return table_.makeConstIter(table_.find(token, key));
   }
 
+  FOLLY_ALWAYS_INLINE const_iterator
+  find(F14HashedKey<key_type, hasher> const& hashedKey) const {
+    return table_.makeIter(
+        table_.find(hashedKey.getHashToken(), hashedKey.getKey()));
+  }
+
   template <typename K>
   FOLLY_ALWAYS_INLINE EnableHeterogeneousFind<K, iterator> find(K const& key) {
     return table_.makeIter(table_.find(key));
@@ -881,6 +924,11 @@ class F14BasicMap {
     return !table_.find(token, key).atEnd();
   }
 
+  FOLLY_ALWAYS_INLINE bool contains(
+      const F14HashedKey<key_type, hasher>& hashedKey) const {
+    return !table_.find(hashedKey.getHashToken(), hashedKey.getKey()).atEnd();
+  }
+
   template <typename K>
   FOLLY_ALWAYS_INLINE EnableHeterogeneousFind<K, bool> contains(
       F14HashToken const& token, K const& key) const {
@@ -1142,12 +1190,13 @@ template <
     typename Alloc,
     typename = detail::RequireInputIterator<InputIt>,
     typename = detail::RequireAllocator<Alloc>>
-F14ValueMap(InputIt, InputIt, std::size_t, Alloc) -> F14ValueMap<
-    iterator_key_type_t<InputIt>,
-    iterator_mapped_type_t<InputIt>,
-    f14::DefaultHasher<iterator_key_type_t<InputIt>>,
-    f14::DefaultKeyEqual<iterator_key_type_t<InputIt>>,
-    Alloc>;
+F14ValueMap(InputIt, InputIt, std::size_t, Alloc)
+    -> F14ValueMap<
+        iterator_key_type_t<InputIt>,
+        iterator_mapped_type_t<InputIt>,
+        f14::DefaultHasher<iterator_key_type_t<InputIt>>,
+        f14::DefaultKeyEqual<iterator_key_type_t<InputIt>>,
+        Alloc>;
 
 template <
     typename InputIt,
@@ -1156,12 +1205,13 @@ template <
     typename = detail::RequireInputIterator<InputIt>,
     typename = detail::RequireNotAllocator<Hasher>,
     typename = detail::RequireAllocator<Alloc>>
-F14ValueMap(InputIt, InputIt, std::size_t, Hasher, Alloc) -> F14ValueMap<
-    iterator_key_type_t<InputIt>,
-    iterator_mapped_type_t<InputIt>,
-    Hasher,
-    f14::DefaultKeyEqual<iterator_key_type_t<InputIt>>,
-    Alloc>;
+F14ValueMap(InputIt, InputIt, std::size_t, Hasher, Alloc)
+    -> F14ValueMap<
+        iterator_key_type_t<InputIt>,
+        iterator_mapped_type_t<InputIt>,
+        Hasher,
+        f14::DefaultKeyEqual<iterator_key_type_t<InputIt>>,
+        Alloc>;
 
 template <
     typename Key,
@@ -1290,12 +1340,13 @@ template <
     typename Alloc,
     typename = detail::RequireInputIterator<InputIt>,
     typename = detail::RequireAllocator<Alloc>>
-F14NodeMap(InputIt, InputIt, std::size_t, Alloc) -> F14NodeMap<
-    iterator_key_type_t<InputIt>,
-    iterator_mapped_type_t<InputIt>,
-    f14::DefaultHasher<iterator_key_type_t<InputIt>>,
-    f14::DefaultKeyEqual<iterator_key_type_t<InputIt>>,
-    Alloc>;
+F14NodeMap(InputIt, InputIt, std::size_t, Alloc)
+    -> F14NodeMap<
+        iterator_key_type_t<InputIt>,
+        iterator_mapped_type_t<InputIt>,
+        f14::DefaultHasher<iterator_key_type_t<InputIt>>,
+        f14::DefaultKeyEqual<iterator_key_type_t<InputIt>>,
+        Alloc>;
 
 template <
     typename InputIt,
@@ -1304,12 +1355,13 @@ template <
     typename = detail::RequireInputIterator<InputIt>,
     typename = detail::RequireNotAllocator<Hasher>,
     typename = detail::RequireAllocator<Alloc>>
-F14NodeMap(InputIt, InputIt, std::size_t, Hasher, Alloc) -> F14NodeMap<
-    iterator_key_type_t<InputIt>,
-    iterator_mapped_type_t<InputIt>,
-    Hasher,
-    f14::DefaultKeyEqual<iterator_key_type_t<InputIt>>,
-    Alloc>;
+F14NodeMap(InputIt, InputIt, std::size_t, Hasher, Alloc)
+    -> F14NodeMap<
+        iterator_key_type_t<InputIt>,
+        iterator_mapped_type_t<InputIt>,
+        Hasher,
+        f14::DefaultKeyEqual<iterator_key_type_t<InputIt>>,
+        Alloc>;
 
 template <
     typename Key,
@@ -1697,12 +1749,13 @@ template <
     typename Alloc,
     typename = detail::RequireInputIterator<InputIt>,
     typename = detail::RequireAllocator<Alloc>>
-F14VectorMap(InputIt, InputIt, std::size_t, Alloc) -> F14VectorMap<
-    iterator_key_type_t<InputIt>,
-    iterator_mapped_type_t<InputIt>,
-    f14::DefaultHasher<iterator_key_type_t<InputIt>>,
-    f14::DefaultKeyEqual<iterator_key_type_t<InputIt>>,
-    Alloc>;
+F14VectorMap(InputIt, InputIt, std::size_t, Alloc)
+    -> F14VectorMap<
+        iterator_key_type_t<InputIt>,
+        iterator_mapped_type_t<InputIt>,
+        f14::DefaultHasher<iterator_key_type_t<InputIt>>,
+        f14::DefaultKeyEqual<iterator_key_type_t<InputIt>>,
+        Alloc>;
 
 template <
     typename InputIt,
@@ -1711,12 +1764,13 @@ template <
     typename = detail::RequireInputIterator<InputIt>,
     typename = detail::RequireNotAllocator<Hasher>,
     typename = detail::RequireAllocator<Alloc>>
-F14VectorMap(InputIt, InputIt, std::size_t, Hasher, Alloc) -> F14VectorMap<
-    iterator_key_type_t<InputIt>,
-    iterator_mapped_type_t<InputIt>,
-    Hasher,
-    f14::DefaultKeyEqual<iterator_key_type_t<InputIt>>,
-    Alloc>;
+F14VectorMap(InputIt, InputIt, std::size_t, Hasher, Alloc)
+    -> F14VectorMap<
+        iterator_key_type_t<InputIt>,
+        iterator_mapped_type_t<InputIt>,
+        Hasher,
+        f14::DefaultKeyEqual<iterator_key_type_t<InputIt>>,
+        Alloc>;
 
 template <
     typename Key,
@@ -1832,12 +1886,13 @@ template <
     typename Alloc,
     typename = detail::RequireInputIterator<InputIt>,
     typename = detail::RequireAllocator<Alloc>>
-F14FastMap(InputIt, InputIt, std::size_t, Alloc) -> F14FastMap<
-    iterator_key_type_t<InputIt>,
-    iterator_mapped_type_t<InputIt>,
-    f14::DefaultHasher<iterator_key_type_t<InputIt>>,
-    f14::DefaultKeyEqual<iterator_key_type_t<InputIt>>,
-    Alloc>;
+F14FastMap(InputIt, InputIt, std::size_t, Alloc)
+    -> F14FastMap<
+        iterator_key_type_t<InputIt>,
+        iterator_mapped_type_t<InputIt>,
+        f14::DefaultHasher<iterator_key_type_t<InputIt>>,
+        f14::DefaultKeyEqual<iterator_key_type_t<InputIt>>,
+        Alloc>;
 
 template <
     typename InputIt,
@@ -1846,12 +1901,13 @@ template <
     typename = detail::RequireInputIterator<InputIt>,
     typename = detail::RequireNotAllocator<Hasher>,
     typename = detail::RequireAllocator<Alloc>>
-F14FastMap(InputIt, InputIt, std::size_t, Hasher, Alloc) -> F14FastMap<
-    iterator_key_type_t<InputIt>,
-    iterator_mapped_type_t<InputIt>,
-    Hasher,
-    f14::DefaultKeyEqual<iterator_key_type_t<InputIt>>,
-    Alloc>;
+F14FastMap(InputIt, InputIt, std::size_t, Hasher, Alloc)
+    -> F14FastMap<
+        iterator_key_type_t<InputIt>,
+        iterator_mapped_type_t<InputIt>,
+        Hasher,
+        f14::DefaultKeyEqual<iterator_key_type_t<InputIt>>,
+        Alloc>;
 
 template <
     typename Key,
