@@ -26,9 +26,6 @@
  * F14FastSet conditionally works like F14ValueSet or F14VectorSet
  *
  * See F14.md
- *
- * @author Nathan Bronson <ngbronson@fb.com>
- * @author Xiao Shi       <xshi@fb.com>
  */
 
 #include <cstddef>
@@ -565,11 +562,23 @@ class F14BasicSet {
     return const_cast<F14BasicSet const*>(this)->find(token, key);
   }
 
+  FOLLY_ALWAYS_INLINE iterator
+  find(const F14HashedKey<key_type, hasher>& hashedKey) {
+    return const_cast<F14BasicSet const*>(this)->find(
+        hashedKey.getHashToken(), hashedKey.getKey());
+  }
+
   FOLLY_ALWAYS_INLINE const_iterator
   find(F14HashToken const& token, key_type const& key) const {
     return table_.makeIter(table_.find(token, key));
   }
 
+  FOLLY_ALWAYS_INLINE const_iterator
+  find(const F14HashedKey<key_type, hasher>& hashedKey) const {
+    return table_.makeIter(
+        table_.find(hashedKey.getHashToken(), hashedKey.getKey()));
+  }
+
   template <typename K>
   FOLLY_ALWAYS_INLINE EnableHeterogeneousFind<K, iterator> find(K const& key) {
     return const_cast<F14BasicSet const*>(this)->find(key);
@@ -613,6 +622,11 @@ class F14BasicSet {
     return !table_.find(token, key).atEnd();
   }
 
+  FOLLY_ALWAYS_INLINE bool contains(
+      const F14HashedKey<key_type, hasher>& hashedKey) const {
+    return !table_.find(hashedKey.getHashToken(), hashedKey.getKey()).atEnd();
+  }
+
   template <typename K>
   FOLLY_ALWAYS_INLINE EnableHeterogeneousFind<K, bool> contains(
       F14HashToken const& token, K const& key) const {
@@ -865,11 +879,12 @@ template <
     typename Alloc,
     typename = detail::RequireInputIterator<InputIt>,
     typename = detail::RequireAllocator<Alloc>>
-F14ValueSet(InputIt, InputIt, std::size_t, Alloc) -> F14ValueSet<
-    iterator_value_type_t<InputIt>,
-    f14::DefaultHasher<iterator_value_type_t<InputIt>>,
-    f14::DefaultKeyEqual<iterator_value_type_t<InputIt>>,
-    Alloc>;
+F14ValueSet(InputIt, InputIt, std::size_t, Alloc)
+    -> F14ValueSet<
+        iterator_value_type_t<InputIt>,
+        f14::DefaultHasher<iterator_value_type_t<InputIt>>,
+        f14::DefaultKeyEqual<iterator_value_type_t<InputIt>>,
+        Alloc>;
 
 template <
     typename InputIt,
@@ -878,11 +893,12 @@ template <
     typename = detail::RequireInputIterator<InputIt>,
     typename = detail::RequireNotAllocator<Hasher>,
     typename = detail::RequireAllocator<Alloc>>
-F14ValueSet(InputIt, InputIt, std::size_t, Hasher, Alloc) -> F14ValueSet<
-    iterator_value_type_t<InputIt>,
-    Hasher,
-    f14::DefaultKeyEqual<iterator_value_type_t<InputIt>>,
-    Alloc>;
+F14ValueSet(InputIt, InputIt, std::size_t, Hasher, Alloc)
+    -> F14ValueSet<
+        iterator_value_type_t<InputIt>,
+        Hasher,
+        f14::DefaultKeyEqual<iterator_value_type_t<InputIt>>,
+        Alloc>;
 
 template <
     typename Key,
@@ -903,11 +919,12 @@ template <
     typename Key,
     typename Alloc,
     typename = detail::RequireAllocator<Alloc>>
-F14ValueSet(std::initializer_list<Key>, std::size_t, Alloc) -> F14ValueSet<
-    Key,
-    f14::DefaultHasher<Key>,
-    f14::DefaultKeyEqual<Key>,
-    Alloc>;
+F14ValueSet(std::initializer_list<Key>, std::size_t, Alloc)
+    -> F14ValueSet<
+        Key,
+        f14::DefaultHasher<Key>,
+        f14::DefaultKeyEqual<Key>,
+        Alloc>;
 
 template <
     typename Key,
@@ -981,11 +998,12 @@ template <
     typename Alloc,
     typename = detail::RequireInputIterator<InputIt>,
     typename = detail::RequireAllocator<Alloc>>
-F14NodeSet(InputIt, InputIt, std::size_t, Alloc) -> F14NodeSet<
-    iterator_value_type_t<InputIt>,
-    f14::DefaultHasher<iterator_value_type_t<InputIt>>,
-    f14::DefaultKeyEqual<iterator_value_type_t<InputIt>>,
-    Alloc>;
+F14NodeSet(InputIt, InputIt, std::size_t, Alloc)
+    -> F14NodeSet<
+        iterator_value_type_t<InputIt>,
+        f14::DefaultHasher<iterator_value_type_t<InputIt>>,
+        f14::DefaultKeyEqual<iterator_value_type_t<InputIt>>,
+        Alloc>;
 
 template <
     typename InputIt,
@@ -994,11 +1012,12 @@ template <
     typename = detail::RequireInputIterator<InputIt>,
     typename = detail::RequireNotAllocator<Hasher>,
     typename = detail::RequireAllocator<Alloc>>
-F14NodeSet(InputIt, InputIt, std::size_t, Hasher, Alloc) -> F14NodeSet<
-    iterator_value_type_t<InputIt>,
-    Hasher,
-    f14::DefaultKeyEqual<iterator_value_type_t<InputIt>>,
-    Alloc>;
+F14NodeSet(InputIt, InputIt, std::size_t, Hasher, Alloc)
+    -> F14NodeSet<
+        iterator_value_type_t<InputIt>,
+        Hasher,
+        f14::DefaultKeyEqual<iterator_value_type_t<InputIt>>,
+        Alloc>;
 
 template <
     typename Key,
@@ -1019,11 +1038,12 @@ template <
     typename Key,
     typename Alloc,
     typename = detail::RequireAllocator<Alloc>>
-F14NodeSet(std::initializer_list<Key>, std::size_t, Alloc) -> F14NodeSet<
-    Key,
-    f14::DefaultHasher<Key>,
-    f14::DefaultKeyEqual<Key>,
-    Alloc>;
+F14NodeSet(std::initializer_list<Key>, std::size_t, Alloc)
+    -> F14NodeSet<
+        Key,
+        f14::DefaultHasher<Key>,
+        f14::DefaultKeyEqual<Key>,
+        Alloc>;
 
 template <
     typename Key,
@@ -1297,11 +1317,12 @@ template <
     typename Alloc,
     typename = detail::RequireInputIterator<InputIt>,
     typename = detail::RequireAllocator<Alloc>>
-F14VectorSet(InputIt, InputIt, std::size_t, Alloc) -> F14VectorSet<
-    iterator_value_type_t<InputIt>,
-    f14::DefaultHasher<iterator_value_type_t<InputIt>>,
-    f14::DefaultKeyEqual<iterator_value_type_t<InputIt>>,
-    Alloc>;
+F14VectorSet(InputIt, InputIt, std::size_t, Alloc)
+    -> F14VectorSet<
+        iterator_value_type_t<InputIt>,
+        f14::DefaultHasher<iterator_value_type_t<InputIt>>,
+        f14::DefaultKeyEqual<iterator_value_type_t<InputIt>>,
+        Alloc>;
 
 template <
     typename InputIt,
@@ -1310,11 +1331,12 @@ template <
     typename = detail::RequireInputIterator<InputIt>,
     typename = detail::RequireNotAllocator<Hasher>,
     typename = detail::RequireAllocator<Alloc>>
-F14VectorSet(InputIt, InputIt, std::size_t, Hasher, Alloc) -> F14VectorSet<
-    iterator_value_type_t<InputIt>,
-    Hasher,
-    f14::DefaultKeyEqual<iterator_value_type_t<InputIt>>,
-    Alloc>;
+F14VectorSet(InputIt, InputIt, std::size_t, Hasher, Alloc)
+    -> F14VectorSet<
+        iterator_value_type_t<InputIt>,
+        Hasher,
+        f14::DefaultKeyEqual<iterator_value_type_t<InputIt>>,
+        Alloc>;
 
 template <
     typename Key,
@@ -1335,11 +1357,12 @@ template <
     typename Key,
     typename Alloc,
     typename = detail::RequireAllocator<Alloc>>
-F14VectorSet(std::initializer_list<Key>, std::size_t, Alloc) -> F14VectorSet<
-    Key,
-    f14::DefaultHasher<Key>,
-    f14::DefaultKeyEqual<Key>,
-    Alloc>;
+F14VectorSet(std::initializer_list<Key>, std::size_t, Alloc)
+    -> F14VectorSet<
+        Key,
+        f14::DefaultHasher<Key>,
+        f14::DefaultKeyEqual<Key>,
+        Alloc>;
 
 template <
     typename Key,
@@ -1399,11 +1422,12 @@ template <
     typename Alloc,
     typename = detail::RequireInputIterator<InputIt>,
     typename = detail::RequireAllocator<Alloc>>
-F14FastSet(InputIt, InputIt, std::size_t, Alloc) -> F14FastSet<
-    iterator_value_type_t<InputIt>,
-    f14::DefaultHasher<iterator_value_type_t<InputIt>>,
-    f14::DefaultKeyEqual<iterator_value_type_t<InputIt>>,
-    Alloc>;
+F14FastSet(InputIt, InputIt, std::size_t, Alloc)
+    -> F14FastSet<
+        iterator_value_type_t<InputIt>,
+        f14::DefaultHasher<iterator_value_type_t<InputIt>>,
+        f14::DefaultKeyEqual<iterator_value_type_t<InputIt>>,
+        Alloc>;
 
 template <
     typename InputIt,
@@ -1412,11 +1436,12 @@ template <
     typename = detail::RequireInputIterator<InputIt>,
     typename = detail::RequireNotAllocator<Hasher>,
     typename = detail::RequireAllocator<Alloc>>
-F14FastSet(InputIt, InputIt, std::size_t, Hasher, Alloc) -> F14FastSet<
-    iterator_value_type_t<InputIt>,
-    Hasher,
-    f14::DefaultKeyEqual<iterator_value_type_t<InputIt>>,
-    Alloc>;
+F14FastSet(InputIt, InputIt, std::size_t, Hasher, Alloc)
+    -> F14FastSet<
+        iterator_value_type_t<InputIt>,
+        Hasher,
+        f14::DefaultKeyEqual<iterator_value_type_t<InputIt>>,
+        Alloc>;
 
 template <
     typename Key,
@@ -1437,11 +1462,12 @@ template <
     typename Key,
     typename Alloc,
     typename = detail::RequireAllocator<Alloc>>
-F14FastSet(std::initializer_list<Key>, std::size_t, Alloc) -> F14FastSet<
-    Key,
-    f14::DefaultHasher<Key>,
-    f14::DefaultKeyEqual<Key>,
-    Alloc>;
+F14FastSet(std::initializer_list<Key>, std::size_t, Alloc)
+    -> F14FastSet<
+        Key,
+        f14::DefaultHasher<Key>,
+        f14::DefaultKeyEqual<Key>,
+        Alloc>;
 
 template <
     typename Key,
