@@ -25,6 +25,7 @@
 #include <limits>
 #include <memory>
 #include <new>
+#include <string_view>
 #include <type_traits>
 #include <utility>
 #include <vector>
@@ -39,7 +40,6 @@
 #include <folly/lang/Align.h>
 #include <folly/lang/Assume.h>
 #include <folly/lang/Exception.h>
-#include <folly/lang/Launder.h>
 #include <folly/lang/Pretty.h>
 #include <folly/lang/SafeAssert.h>
 #include <folly/portability/Builtins.h>
@@ -49,10 +49,6 @@
 #include <folly/container/detail/F14IntrinsicsAvailability.h>
 #include <folly/container/detail/F14Mask.h>
 
-#if FOLLY_HAS_STRING_VIEW
-#include <string_view> // @manual
-#endif
-
 #if FOLLY_F14_VECTOR_INTRINSICS_AVAILABLE
 
 #if FOLLY_F14_CRC_INTRINSIC_AVAILABLE
@@ -160,11 +156,9 @@ struct StdIsFastHash<std::hash<long double>> : std::false_type {};
 template <typename... Args>
 struct StdIsFastHash<std::hash<std::basic_string<Args...>>> : std::false_type {
 };
-#if FOLLY_HAS_STRING_VIEW
 template <typename... Args>
 struct StdIsFastHash<std::hash<std::basic_string_view<Args...>>>
     : std::false_type {};
-#endif
 
 // mimic internal node of unordered containers in STL to estimate the size
 template <typename K, typename V, typename H, typename Enable = void>
@@ -224,11 +218,194 @@ class F14HashToken final {
 
   template <typename Policy>
   friend class f14::detail::F14Table;
+
+  template <typename Key, typename Hasher>
+  friend class F14HashedKey;
 };
+
 #else
 class F14HashToken final {};
 #endif
 
+namespace f14 {
+namespace detail {
+
+// Detection for folly_assume_32bit_hash
+
+template <typename Hasher, typename Void = void>
+struct ShouldAssume32BitHash : std::bool_constant<!require_sizeof<Hasher>> {};
+
+template <typename Hasher>
+struct ShouldAssume32BitHash<
+    Hasher,
+    void_t<typename Hasher::folly_assume_32bit_hash>>
+    : std::bool_constant<Hasher::folly_assume_32bit_hash::value> {};
+
+//////// hash helpers
+
+// Hash values are used to compute the desired position, which is the
+// chunk index at which we would like to place a value (if there is no
+// overflow), and the tag, which is an additional 7 bits of entropy.
+//
+// The standard's definition of hash function quality only refers to
+// the probability of collisions of the entire hash value, not to the
+// probability of collisions of the results of shifting or masking the
+// hash value.  Some hash functions, however, provide this stronger
+// guarantee (not quite the same as the definition of avalanching,
+// but similar).
+//
+// If the user-supplied hasher is an avalanching one (each bit of the
+// hash value has a 50% chance of being the same for differing hash
+// inputs), then we can just take 7 bits of the hash value for the tag
+// and the rest for the desired position.  Avalanching hashers also
+// let us map hash value to array index position with just a bitmask
+// without risking clumping.  (Many hash tables just accept the risk
+// and do it regardless.)
+//
+// std::hash<std::string> avalanches in all implementations we've
+// examined: libstdc++-v3 uses MurmurHash2, and libc++ uses CityHash
+// or MurmurHash2.  The other std::hash specializations, however, do not
+// have this property.  std::hash for integral and pointer values is the
+// identity function on libstdc++-v3 and libc++, in particular.  In our
+// experience it is also fairly common for user-defined specializations
+// of std::hash to combine fields in an ad-hoc way that does not evenly
+// distribute entropy among the bits of the result (a + 37 * b, for
+// example, where a and b are integer fields).
+//
+// For hash functions we don't trust to avalanche, we repair things by
+// applying a bit mixer to the user-supplied hash.
+
+#if FOLLY_F14_VECTOR_INTRINSICS_AVAILABLE
+#if FOLLY_X64 || FOLLY_AARCH64 || FOLLY_RISCV64
+// 64-bit
+template <typename Hasher, typename Key>
+std::pair<std::size_t, std::size_t> splitHashImpl(std::size_t hash) {
+  static_assert(sizeof(std::size_t) == sizeof(uint64_t), "");
+  std::size_t tag;
+  if (!IsAvalanchingHasher<Hasher, Key>::value) {
+#if FOLLY_F14_CRC_INTRINSIC_AVAILABLE
+#if FOLLY_SSE_PREREQ(4, 2)
+    // SSE4.2 CRC
+    std::size_t c = _mm_crc32_u64(0, hash);
+    tag = (c >> 24) | 0x80;
+    hash += c;
+#else
+    // CRC is optional on armv8 (-march=armv8-a+crc), standard on armv8.1
+    std::size_t c = __crc32cd(0, hash);
+    tag = (c >> 24) | 0x80;
+    hash += c;
+#endif
+#else
+    // The mixer below is not fully avalanching for all 64 bits of
+    // output, but looks quite good for bits 18..63 and puts plenty
+    // of entropy even lower when considering multiple bits together
+    // (like the tag).  Importantly, when under register pressure it
+    // uses fewer registers, instructions, and immediate constants
+    // than the alternatives, resulting in compact code that is more
+    // easily inlinable.  In one instantiation a modified Murmur mixer
+    // was 48 bytes of assembly (even after using the same multiplicand
+    // for both steps) and this one was 27 bytes, for example.
+    auto const kMul = 0xc4ceb9fe1a85ec53ULL;
+#ifdef _WIN32
+    __int64 signedHi;
+    __int64 signedLo = _mul128(
+        static_cast<__int64>(hash), static_cast<__int64>(kMul), &signedHi);
+    auto hi = static_cast<uint64_t>(signedHi);
+    auto lo = static_cast<uint64_t>(signedLo);
+#else
+    auto hi = static_cast<uint64_t>(
+        (static_cast<unsigned __int128>(hash) * kMul) >> 64);
+    auto lo = hash * kMul;
+#endif
+    hash = hi ^ lo;
+    hash *= kMul;
+    tag = ((hash >> 15) & 0x7f) | 0x80;
+    hash >>= 22;
+#endif
+  } else {
+    // F14 uses the bottom bits of the hash to form the index, so for maps
+    // with less than 16.7 million entries, it's safe to have a 32-bit hash,
+    // and use the bottom 24 bits for the index and leave the top 8 for the
+    // tag.
+    if (ShouldAssume32BitHash<Hasher>::value) {
+      // we don't trust the top bit
+      tag = ((hash >> 24) | 0x80) & 0xFF;
+      // Explicitly mask off the top 32-bits so that the compiler can
+      // optimize away whatever is populating the top 32-bits, which is likely
+      // just the lower 32-bits duplicated.
+      hash = hash & 0xFFFF'FFFF;
+    } else {
+      // we don't trust the top bit
+      tag = (hash >> 56) | 0x80;
+    }
+  }
+  return std::make_pair(hash, tag);
+}
+#else
+// 32-bit
+template <typename Hasher, typename Key>
+std::pair<std::size_t, std::size_t> splitHashImpl(std::size_t hash) {
+  static_assert(sizeof(std::size_t) == sizeof(uint32_t), "");
+  uint8_t tag;
+  if (!IsAvalanchingHasher<Hasher, Key>::value) {
+#if FOLLY_F14_CRC_INTRINSIC_AVAILABLE
+#if FOLLY_SSE_PREREQ(4, 2)
+    // SSE4.2 CRC
+    auto c = _mm_crc32_u32(0, hash);
+    tag = static_cast<uint8_t>(~(c >> 25));
+    hash += c;
+#else
+    auto c = __crc32cw(0, hash);
+    tag = static_cast<uint8_t>(~(c >> 25));
+    hash += c;
+#endif
+#else
+    // finalizer for 32-bit murmur2
+    hash ^= hash >> 13;
+    hash *= 0x5bd1e995;
+    hash ^= hash >> 15;
+    tag = static_cast<uint8_t>(~(hash >> 25));
+#endif
+  } else {
+    // we don't trust the top bit
+    tag = (hash >> 24) | 0x80;
+  }
+  return std::make_pair(hash, tag);
+}
+#endif
+#endif
+} // namespace detail
+} // namespace f14
+
+template <typename TKeyType, typename Hasher = f14::DefaultHasher<TKeyType>>
+class F14HashedKey final {
+ public:
+#if FOLLY_F14_VECTOR_INTRINSICS_AVAILABLE
+  template <typename... Args>
+  explicit F14HashedKey(Args&&... args)
+      : key_(std::forward<Args>(args)...),
+        hash_(f14::detail::splitHashImpl<Hasher, TKeyType>(Hasher{}(key_))) {}
+#else
+  F14HashedKey() = delete;
+#endif
+
+  const TKeyType& getKey() const { return key_; }
+  const F14HashToken& getHashToken() const { return hash_; }
+  // We want the conversion to the key to be implicit - the hashed key should
+  // seamlessly behave as the key itself.
+  /* implicit */ operator const TKeyType&() const { return key_; }
+  explicit operator const F14HashToken&() const { return hash_; }
+
+  bool operator==(const F14HashedKey& other) const {
+    return key_ == other.key_;
+  }
+  bool operator==(const TKeyType& other) const { return key_ == other; }
+
+ private:
+  TKeyType key_;
+  F14HashToken hash_;
+};
+
 #if FOLLY_F14_VECTOR_INTRINSICS_AVAILABLE
 namespace f14 {
 namespace detail {
@@ -323,7 +500,7 @@ struct alignas(kRequiredVectorAlignment) F14Chunk {
 
   static F14Chunk* emptyInstance() {
     auto raw = reinterpret_cast<char*>(&kEmptyTagVector);
-    if (kRequiredVectorAlignment > alignof(max_align_t)) {
+    if constexpr (kRequiredVectorAlignment > alignof(max_align_t)) {
       auto delta = kRequiredVectorAlignment -
           (reinterpret_cast<uintptr_t>(raw) % kRequiredVectorAlignment);
       raw += delta;
@@ -363,7 +540,7 @@ struct alignas(kRequiredVectorAlignment) F14Chunk {
   bool eof() const { return capacityScale() != 0; }
 
   std::size_t capacityScale() const {
-    if (kCapacityScaleBits == 4) {
+    if constexpr (kCapacityScaleBits == 4) {
       return control_ & 0xf;
     } else {
       uint16_t v;
@@ -377,7 +554,7 @@ struct alignas(kRequiredVectorAlignment) F14Chunk {
         this != emptyInstance() && scale > 0 &&
             scale < (std::size_t{1} << kCapacityScaleBits),
         "");
-    if (kCapacityScaleBits == 4) {
+    if constexpr (kCapacityScaleBits == 4) {
       control_ = static_cast<uint8_t>((control_ & ~0xf) | scale);
     } else {
       uint16_t v = static_cast<uint16_t>(scale);
@@ -542,12 +719,12 @@ struct alignas(kRequiredVectorAlignment) F14Chunk {
 
   Item& item(std::size_t i) {
     FOLLY_SAFE_DCHECK(this->occupied(i), "");
-    return *launder(itemAddr(i));
+    return *std::launder(itemAddr(i));
   }
 
   Item const& citem(std::size_t i) const {
     FOLLY_SAFE_DCHECK(this->occupied(i), "");
-    return *launder(itemAddr(i));
+    return *std::launder(itemAddr(i));
   }
 
   static F14Chunk& owner(Item& item, std::size_t index) {
@@ -1001,6 +1178,7 @@ class F14Table : public Policy {
   using Alloc = typename Policy::Alloc;
   using AllocTraits = typename Policy::AllocTraits;
   using Hasher = typename Policy::Hasher;
+  using InternalSizeType = typename Policy::InternalSizeType;
   using KeyEqual = typename Policy::KeyEqual;
 
   using Policy::kAllocIsAlwaysEqual;
@@ -1086,9 +1264,9 @@ class F14Table : public Policy {
   }
 
   F14Table(F14Table&& rhs) noexcept(
-      std::is_nothrow_move_constructible<Hasher>::value&&
-          std::is_nothrow_move_constructible<KeyEqual>::value&&
-              std::is_nothrow_move_constructible<Alloc>::value)
+      std::is_nothrow_move_constructible<Hasher>::value &&
+      std::is_nothrow_move_constructible<KeyEqual>::value &&
+      std::is_nothrow_move_constructible<Alloc>::value)
       : Policy{std::move(rhs)} {
     swapContents(rhs);
   }
@@ -1118,8 +1296,8 @@ class F14Table : public Policy {
   }
 
   F14Table& operator=(F14Table&& rhs) noexcept(
-      std::is_nothrow_move_assignable<Hasher>::value&&
-          std::is_nothrow_move_assignable<KeyEqual>::value &&
+      std::is_nothrow_move_assignable<Hasher>::value &&
+      std::is_nothrow_move_assignable<KeyEqual>::value &&
       (kAllocIsAlwaysEqual ||
        (AllocTraits::propagate_on_container_move_assignment::value &&
         std::is_nothrow_move_assignable<Alloc>::value))) {
@@ -1160,136 +1338,9 @@ class F14Table : public Policy {
   }
 
  private:
-  //////// hash helpers
-
-  // Hash values are used to compute the desired position, which is the
-  // chunk index at which we would like to place a value (if there is no
-  // overflow), and the tag, which is an additional 7 bits of entropy.
-  //
-  // The standard's definition of hash function quality only refers to
-  // the probability of collisions of the entire hash value, not to the
-  // probability of collisions of the results of shifting or masking the
-  // hash value.  Some hash functions, however, provide this stronger
-  // guarantee (not quite the same as the definition of avalanching,
-  // but similar).
-  //
-  // If the user-supplied hasher is an avalanching one (each bit of the
-  // hash value has a 50% chance of being the same for differing hash
-  // inputs), then we can just take 7 bits of the hash value for the tag
-  // and the rest for the desired position.  Avalanching hashers also
-  // let us map hash value to array index position with just a bitmask
-  // without risking clumping.  (Many hash tables just accept the risk
-  // and do it regardless.)
-  //
-  // std::hash<std::string> avalanches in all implementations we've
-  // examined: libstdc++-v3 uses MurmurHash2, and libc++ uses CityHash
-  // or MurmurHash2.  The other std::hash specializations, however, do not
-  // have this property.  std::hash for integral and pointer values is the
-  // identity function on libstdc++-v3 and libc++, in particular.  In our
-  // experience it is also fairly common for user-defined specializations
-  // of std::hash to combine fields in an ad-hoc way that does not evenly
-  // distribute entropy among the bits of the result (a + 37 * b, for
-  // example, where a and b are integer fields).
-  //
-  // For hash functions we don't trust to avalanche, we repair things by
-  // applying a bit mixer to the user-supplied hash.
-
-#if FOLLY_X64 || FOLLY_AARCH64 || FOLLY_RISCV64
-  // 64-bit
   static HashPair splitHash(std::size_t hash) {
-    static_assert(sizeof(std::size_t) == sizeof(uint64_t), "");
-    std::size_t tag;
-    if (!isAvalanchingHasher()) {
-#if FOLLY_F14_CRC_INTRINSIC_AVAILABLE
-#if FOLLY_SSE_PREREQ(4, 2)
-      // SSE4.2 CRC
-      std::size_t c = _mm_crc32_u64(0, hash);
-      tag = (c >> 24) | 0x80;
-      hash += c;
-#else
-      // CRC is optional on armv8 (-march=armv8-a+crc), standard on armv8.1
-      std::size_t c = __crc32cd(0, hash);
-      tag = (c >> 24) | 0x80;
-      hash += c;
-#endif
-#else
-      // The mixer below is not fully avalanching for all 64 bits of
-      // output, but looks quite good for bits 18..63 and puts plenty
-      // of entropy even lower when considering multiple bits together
-      // (like the tag).  Importantly, when under register pressure it
-      // uses fewer registers, instructions, and immediate constants
-      // than the alternatives, resulting in compact code that is more
-      // easily inlinable.  In one instantiation a modified Murmur mixer
-      // was 48 bytes of assembly (even after using the same multiplicand
-      // for both steps) and this one was 27 bytes, for example.
-      auto const kMul = 0xc4ceb9fe1a85ec53ULL;
-#ifdef _WIN32
-      __int64 signedHi;
-      __int64 signedLo = _mul128(
-          static_cast<__int64>(hash), static_cast<__int64>(kMul), &signedHi);
-      auto hi = static_cast<uint64_t>(signedHi);
-      auto lo = static_cast<uint64_t>(signedLo);
-#else
-      auto hi = static_cast<uint64_t>(
-          (static_cast<unsigned __int128>(hash) * kMul) >> 64);
-      auto lo = hash * kMul;
-#endif
-      hash = hi ^ lo;
-      hash *= kMul;
-      tag = ((hash >> 15) & 0x7f) | 0x80;
-      hash >>= 22;
-#endif
-    } else {
-      // F14 uses the bottom bits of the hash to form the index, so for maps
-      // with less than 16.7 million entries, it's safe to have a 32-bit hash,
-      // and use the bottom 24 bits for the index and leave the top 8 for the
-      // tag.
-      if (shouldAssume32BitHash()) {
-        // we don't trust the top bit
-        tag = ((hash >> 24) | 0x80) & 0xFF;
-        // Explicitly mask off the top 32-bits so that the compiler can
-        // optimize away whatever is populating the top 32-bits, which is likely
-        // just the lower 32-bits duplicated.
-        hash = hash & 0xFFFF'FFFF;
-      } else {
-        // we don't trust the top bit
-        tag = (hash >> 56) | 0x80;
-      }
-    }
-    return std::make_pair(hash, tag);
+    return f14::detail::splitHashImpl<Hasher, value_type>(hash);
   }
-#else
-  // 32-bit
-  static HashPair splitHash(std::size_t hash) {
-    static_assert(sizeof(std::size_t) == sizeof(uint32_t), "");
-    uint8_t tag;
-    if (!isAvalanchingHasher()) {
-#if FOLLY_F14_CRC_INTRINSIC_AVAILABLE
-#if FOLLY_SSE_PREREQ(4, 2)
-      // SSE4.2 CRC
-      auto c = _mm_crc32_u32(0, hash);
-      tag = static_cast<uint8_t>(~(c >> 25));
-      hash += c;
-#else
-      auto c = __crc32cw(0, hash);
-      tag = static_cast<uint8_t>(~(c >> 25));
-      hash += c;
-#endif
-#else
-      // finalizer for 32-bit murmur2
-      hash ^= hash >> 13;
-      hash *= 0x5bd1e995;
-      hash ^= hash >> 15;
-      tag = static_cast<uint8_t>(~(hash >> 25));
-#endif
-    } else {
-      // we don't trust the top bit
-      tag = (hash >> 24) | 0x80;
-    }
-    return std::make_pair(hash, tag);
-  }
-#endif
-
   //////// memory management helpers
 
   static std::size_t computeCapacity(
@@ -1404,7 +1455,9 @@ class F14Table : public Policy {
   std::size_t max_size() const noexcept {
     auto& a = this->alloc();
     return std::min<std::size_t>(
-        SizeAndChunkShift::kMaxSize, AllocTraits::max_size(a));
+        {SizeAndChunkShift::kMaxSize,
+         std::numeric_limits<InternalSizeType>::max(),
+         AllocTraits::max_size(a)});
   }
 
   std::size_t bucket_count() const noexcept {
@@ -1574,7 +1627,7 @@ class F14Table : public Policy {
 
  private:
   void adjustSizeAndBeginAfterInsert(ItemIter iter) {
-    if (kEnableItemIteration) {
+    if constexpr (kEnableItemIteration) {
       // packedBegin is the max of all valid ItemIter::pack()
       auto packed = iter.pack();
       if (sizeAndChunkShiftAndPackedBegin_.packedBegin() < packed) {
@@ -1609,7 +1662,7 @@ class F14Table : public Policy {
 
   void adjustSizeAndBeginBeforeErase(ItemIter iter) {
     sizeAndChunkShiftAndPackedBegin_.decrementSize();
-    if (kEnableItemIteration) {
+    if constexpr (kEnableItemIteration) {
       if (iter.pack() == sizeAndChunkShiftAndPackedBegin_.packedBegin()) {
         if (size() == 0) {
           iter = ItemIter{};
@@ -1657,7 +1710,7 @@ class F14Table : public Policy {
 
   ChunkPtr lastOccupiedChunk() const {
     FOLLY_SAFE_DCHECK(size() > 0, "");
-    if (kEnableItemIteration) {
+    if constexpr (kEnableItemIteration) {
       return begin().chunk();
     } else {
       return chunks_ + chunkCount() - 1;
@@ -1691,8 +1744,8 @@ class F14Table : public Policy {
     // partial failure should not occur.  Sorry for the subtle invariants
     // in the Policy API.
 
-    if (is_trivially_copyable<Item>::value && !this->destroyItemOnClear() &&
-        itemCount() == src.itemCount()) {
+    if (std::is_trivially_copyable<Item>::value &&
+        !this->destroyItemOnClear() && itemCount() == src.itemCount()) {
       FOLLY_SAFE_DCHECK(chunkShift() == src.chunkShift(), "");
 
       auto scale = chunks_->capacityScale();
@@ -1701,14 +1754,14 @@ class F14Table : public Policy {
       auto n = chunkAllocSize(chunkCount(), scale);
       std::memcpy(&chunks_[0], &src.chunks_[0], n);
       sizeAndChunkShiftAndPackedBegin_.setSize(src.size());
-      if (kEnableItemIteration) {
+      if constexpr (kEnableItemIteration) {
         auto srcBegin = src.begin();
         sizeAndChunkShiftAndPackedBegin_.packedBegin() =
             ItemIter{
                 chunks_ + (srcBegin.chunk() - src.chunks_), srcBegin.index()}
                 .pack();
       }
-      if (kContinuousCapacity) {
+      if constexpr (kContinuousCapacity) {
         // capacityScale might not match even if itemCount matches
         chunks_->setCapacityScale(scale);
       }
@@ -1746,7 +1799,7 @@ class F14Table : public Policy {
       } while (size() != src.size());
 
       // reset doesn't care about packedBegin, so we don't fix it until the end
-      if (kEnableItemIteration) {
+      if constexpr (kEnableItemIteration) {
         std::size_t maxChunkIndex = src.lastOccupiedChunk() - src.chunks_;
         sizeAndChunkShiftAndPackedBegin_.packedBegin() =
             ItemIter{
@@ -2056,7 +2109,7 @@ class F14Table : public Policy {
         }
         ++srcI;
       }
-      if (kEnableItemIteration) {
+      if constexpr (kEnableItemIteration) {
         sizeAndChunkShiftAndPackedBegin_.packedBegin() =
             ItemIter{dstChunk, dstI - 1}.pack();
       }
@@ -2107,7 +2160,7 @@ class F14Table : public Policy {
         --srcChunk;
       }
 
-      if (kEnableItemIteration) {
+      if constexpr (kEnableItemIteration) {
         // this code replaces size invocations of adjustSizeAndBeginAfterInsert
         std::size_t i = chunkCount() - 1;
         while (fullness[i] == 0) {
@@ -2125,7 +2178,7 @@ class F14Table : public Policy {
   // sanitizer builds
 
   FOLLY_ALWAYS_INLINE void debugModeOnReserve(std::size_t capacity) {
-    if (kIsLibrarySanitizeAddress || kIsDebug) {
+    if constexpr (kIsLibrarySanitizeAddress || kIsDebug) {
       if (capacity > size()) {
         tlsPendingSafeInserts(static_cast<std::ptrdiff_t>(capacity - size()));
       }
@@ -2149,14 +2202,16 @@ class F14Table : public Policy {
     // One way to fix this is to call map.reserve(N) before such a
     // sequence, where N is the number of keys that might be inserted
     // within the section that retains references plus the existing size.
-    if (kIsLibrarySanitizeAddress && !tlsPendingSafeInserts() && size() > 0 &&
-        tlsMinstdRand(size()) == 0) {
-      debugModeSpuriousRehash();
+    if constexpr (kIsLibrarySanitizeAddress) {
+      if (!tlsPendingSafeInserts() && size() > 0 &&
+          tlsMinstdRand(size()) == 0) {
+        debugModeSpuriousRehash();
+      }
     }
   }
 
   FOLLY_ALWAYS_INLINE void debugModeAfterInsert() {
-    if (kIsLibrarySanitizeAddress || kIsDebug) {
+    if constexpr (kIsLibrarySanitizeAddress || kIsDebug) {
       tlsPendingSafeInserts(-1);
     }
   }
@@ -2309,7 +2364,7 @@ class F14Table : public Policy {
         }
         chunks_[0].markEof(scale);
       }
-      if (kEnableItemIteration) {
+      if constexpr (kEnableItemIteration) {
         sizeAndChunkShiftAndPackedBegin_.packedBegin() = ItemIter{}.pack();
       }
       sizeAndChunkShiftAndPackedBegin_.setSize(0);
@@ -2367,7 +2422,7 @@ class F14Table : public Policy {
   }
 
   void clear() noexcept {
-    if (kIsLibrarySanitizeAddress) {
+    if constexpr (kIsLibrarySanitizeAddress) {
       // force recycling of heap memory
       auto bc = bucket_count();
       reset();
@@ -2466,7 +2521,7 @@ class F14Table : public Policy {
   F14TableStats computeStats() const {
     F14TableStats stats;
 
-    if (kIsDebug && kEnableItemIteration) {
+    if constexpr (kIsDebug && kEnableItemIteration) {
       // validate iteration
       std::size_t n = 0;
       ItemIter prev;
@@ -2565,7 +2620,7 @@ class F14Table : public Policy {
 namespace f14 {
 namespace test {
 inline void disableInsertOrderRandomization() {
-  if (kIsLibrarySanitizeAddress || kIsDebug) {
+  if constexpr (kIsLibrarySanitizeAddress || kIsDebug) {
     detail::tlsPendingSafeInserts(static_cast<std::ptrdiff_t>(
         (std::numeric_limits<std::size_t>::max)() / 2));
   }
