@@ -77,3 +77,29 @@ TEST_F(SparseByteSetTest, clear) {
     EXPECT_TRUE(s.add(c));
   }
 }
+
+TEST_F(SparseByteSetTest, remove) {
+  for (auto c = lims::min(); c < lims::max(); ++c) {
+    EXPECT_TRUE(s.add(c));
+  }
+  for (auto c = lims::min(); c < lims::max() / 2; ++c) {
+    EXPECT_TRUE(s.remove(c));
+    EXPECT_FALSE(s.contains(c));
+  }
+
+  // did not corrupt rest data
+  for (auto c = lims::max() / 2; c < lims::max(); ++c) {
+    EXPECT_TRUE(s.contains(c));
+  }
+
+  // check deleting last elements
+  for (auto c = lims::max() - 1; c >= lims::max() / 2; --c) {
+    EXPECT_TRUE(s.remove(c));
+    EXPECT_FALSE(s.contains(c));
+  }
+}
+
+TEST_F(SparseByteSetTest, remove_nop) {
+  bool r = s.remove(12);
+  EXPECT_FALSE(r);
+}
