@@ -20,6 +20,7 @@
 #include <mutex>
 
 #include <folly/lang/Hint.h>
+#include <folly/memory/SanitizeLeak.h>
 #include <folly/synchronization/CallOnce.h>
 
 constexpr auto kSmallGrowthFactor = 1.1;
@@ -28,47 +29,6 @@ constexpr auto kBigGrowthFactor = 1.7;
 namespace folly {
 namespace threadlocal_detail {
 
-void ThreadEntryNode::initIfZero(bool locked) {
-  if (FOLLY_UNLIKELY(isZero)) {
-    if (FOLLY_LIKELY(locked)) {
-      parent->meta->pushBackLocked(parent, id);
-    } else {
-      parent->meta->pushBackUnlocked(parent, id);
-    }
-  }
-}
-
-void ThreadEntryNode::push_back(ThreadEntry* head) {
-  // get the head prev and next nodes
-  ThreadEntryNode* hnode = &head->elements[id].node;
-
-  // update current
-  next = head;
-  prev = hnode->prev;
-  isZero = false;
-
-  // hprev
-  ThreadEntryNode* hprev = &hnode->prev->elements[id].node;
-  hprev->next = parent;
-  hnode->prev = parent;
-}
-
-void ThreadEntryNode::eraseZero() {
-  if (FOLLY_LIKELY(prev != nullptr)) {
-    // get the prev and next nodes
-    ThreadEntryNode* nprev = &prev->elements[id].node;
-    ThreadEntryNode* nnext = &next->elements[id].node;
-
-    // update the prev and next
-    nnext->prev = prev;
-    nprev->next = next;
-
-    // set the prev and next to nullptr
-    next = prev = nullptr;
-    isZero = true;
-  }
-}
-
 StaticMetaBase::StaticMetaBase(ThreadEntry* (*threadEntry)(), bool strict)
     : nextId_(1), threadEntry_(threadEntry), strict_(strict) {
   int ret = pthread_key_create(&pthreadKey_, &onThreadExit);
@@ -77,37 +37,45 @@ StaticMetaBase::StaticMetaBase(ThreadEntry* (*threadEntry)(), bool strict)
 }
 
 ThreadEntryList* StaticMetaBase::getThreadEntryList() {
-  if (kUseThreadLocal) {
-    static thread_local ThreadEntryList threadEntryListSingleton;
-    return &threadEntryListSingleton;
-  } else {
-    class PthreadKey {
-     public:
-      PthreadKey() {
-        int ret = pthread_key_create(&pthreadKey_, nullptr);
-        checkPosixError(ret, "pthread_key_create failed");
-        PthreadKeyUnregister::registerKey(pthreadKey_);
-      }
-
-      FOLLY_ALWAYS_INLINE pthread_key_t get() const { return pthreadKey_; }
-
-     private:
-      pthread_key_t pthreadKey_;
-    };
-
-    auto& instance = detail::createGlobal<PthreadKey, void>();
-
-    ThreadEntryList* threadEntryList =
-        static_cast<ThreadEntryList*>(pthread_getspecific(instance.get()));
-
-    if (FOLLY_UNLIKELY(!threadEntryList)) {
-      threadEntryList = new ThreadEntryList();
-      int ret = pthread_setspecific(instance.get(), threadEntryList);
-      checkPosixError(ret, "pthread_setspecific failed");
+  class PthreadKey {
+   public:
+    static void onThreadExit(void* ptr) {
+      ThreadEntryList* list = static_cast<ThreadEntryList*>(ptr);
+      StaticMetaBase::cleanupThreadEntriesAndList(list);
     }
 
-    return threadEntryList;
+    PthreadKey() {
+      int ret = pthread_key_create(&pthreadKey_, &onThreadExit);
+      checkPosixError(ret, "pthread_key_create failed");
+      PthreadKeyUnregister::registerKey(pthreadKey_);
+    }
+
+    FOLLY_ALWAYS_INLINE pthread_key_t get() const { return pthreadKey_; }
+
+   private:
+    pthread_key_t pthreadKey_;
+  };
+
+  static thread_local ThreadEntryList* threadEntryListTL{};
+  if (kUseThreadLocal && threadEntryListTL) {
+    return threadEntryListTL;
   }
+  auto& instance = detail::createGlobal<PthreadKey, void>();
+
+  ThreadEntryList* threadEntryList =
+      static_cast<ThreadEntryList*>(pthread_getspecific(instance.get()));
+
+  if (FOLLY_UNLIKELY(!threadEntryList)) {
+    threadEntryList = new ThreadEntryList();
+    int ret = pthread_setspecific(instance.get(), threadEntryList);
+    checkPosixError(ret, "pthread_setspecific failed");
+    threadEntryList->count = 1; // Pin once for own onThreadExit callback.
+  }
+
+  if (kUseThreadLocal) {
+    threadEntryListTL = threadEntryList;
+  }
+  return threadEntryList;
 }
 
 bool StaticMetaBase::dying() {
@@ -137,9 +105,7 @@ void StaticMetaBase::onThreadExit(void* ptr) {
       // mark it as removed
       threadEntry->removed_ = true;
       auto elementsCapacity = threadEntry->getElementsCapacity();
-      for (size_t i = 0u; i < elementsCapacity; ++i) {
-        threadEntry->elements[i].node.eraseZero();
-      }
+      meta.removeThreadEntryFromAllInMapLocked(threadEntry);
       auto beforeCount = meta.totalElementWrappers_.fetch_sub(elementsCapacity);
       DCHECK_GE(beforeCount, elementsCapacity);
       // No need to hold the lock any longer; the ThreadEntry is private to this
@@ -158,6 +124,7 @@ void StaticMetaBase::onThreadExit(void* ptr) {
           shouldRun = true;
         }
       }
+      DCHECK(threadEntry->meta->isThreadEntryRemovedFromAllInMap(threadEntry));
     }
     pthread_setspecific(meta.pthreadKey_, nullptr);
   }
@@ -165,8 +132,13 @@ void StaticMetaBase::onThreadExit(void* ptr) {
   auto threadEntryList = threadEntry->list;
   DCHECK_GT(threadEntryList->count, 0u);
 
-  --threadEntryList->count;
+  cleanupThreadEntriesAndList(threadEntryList);
+}
 
+/* static */
+void StaticMetaBase::cleanupThreadEntriesAndList(
+    ThreadEntryList* threadEntryList) {
+  --threadEntryList->count;
   if (threadEntryList->count) {
     return;
   }
@@ -182,6 +154,7 @@ void StaticMetaBase::onThreadExit(void* ptr) {
       if (meta.strict_) {
         rlock.lock();
       }
+
       for (bool shouldRunInner = true; shouldRunInner;) {
         shouldRunInner = false;
         auto elementsCapacity = tmp->getElementsCapacity();
@@ -210,14 +183,14 @@ void StaticMetaBase::onThreadExit(void* ptr) {
       tmp->setElementsCapacity(0);
     }
 
-    if (!kUseThreadLocal) {
-      delete tmp;
-    }
+    // Fail safe check to make sure that the ThreadEntry is not present
+    // before issuing a delete.
+    DCHECK(tmp->meta->isThreadEntryRemovedFromAllInMap(tmp));
+
+    delete tmp;
   }
 
-  if (!kUseThreadLocal) {
-    delete threadEntryList;
-  }
+  delete threadEntryList;
 }
 
 uint32_t StaticMetaBase::elementsCapacity() const {
@@ -246,8 +219,6 @@ uint32_t StaticMetaBase::allocate(EntryID* ent) {
   uint32_t old_id = ent->value.exchange(id, std::memory_order_release);
   DCHECK_EQ(old_id, kEntryIDInvalid);
 
-  reserveHeadUnlocked(id);
-
   return id;
 }
 
@@ -279,32 +250,34 @@ void StaticMetaBase::destroy(EntryID* ent) {
           return;
         }
 
-        auto& node = meta.head_.elements[id].node;
-        while (!node.empty()) {
-          auto* next = node.getNext();
-          next->eraseZero();
-
-          ThreadEntry* e = next->parent;
+        auto threadEntrySet = get_ptr(meta.allThreadEntryMap_, id);
+        if (!threadEntrySet) {
+          return;
+        }
+        for (auto& e : threadEntrySet->threadEntries) {
           auto elementsCapacity = e->getElementsCapacity();
-          if (id < elementsCapacity && e->elements[id].ptr) {
-            elements.push_back(e->elements[id]);
-
-            /*
-             * Writing another thread's ThreadEntry from here is fine;
-             * the only other potential reader is the owning thread --
-             * from onThreadExit (which grabs the lock, so is properly
-             * synchronized with us) or from get(), which also grabs
-             * the lock if it needs to resize the elements vector.
-             *
-             * We can't conflict with reads for a get(id), because
-             * it's illegal to call get on a thread local that's
-             * destructing.
-             */
-            e->elements[id].ptr = nullptr;
-            e->elements[id].deleter1 = nullptr;
-            e->elements[id].ownsDeleter = false;
+          if (id < elementsCapacity) {
+            if (e->elements[id].ptr) {
+              elements.push_back(e->elements[id]);
+              /*
+               * Writing another thread's ThreadEntry from here is fine;
+               * the only other potential reader is the owning thread --
+               * from onThreadExit (which grabs the lock, so is properly
+               * synchronized with us) or from get(), which also grabs
+               * the lock if it needs to resize the elements vector.
+               *
+               * We can't conflict with reads for a get(id), because
+               * it's illegal to call get on a thread local that's
+               * destructing.
+               */
+              e->elements[id].ptr = nullptr;
+              e->elements[id].deleter1 = nullptr;
+              e->elements[id].ownsDeleter = false;
+            }
+            e->elements[id].isLinked = false;
           }
         }
+        meta.clearSetforIdInMapLocked(id);
         meta.freeIds_.push_back(id);
       }
     }
@@ -329,8 +302,7 @@ ElementWrapper* StaticMetaBase::reallocate(
   auto bigCapacity = static_cast<size_t>((idval + 5) * kBigGrowthFactor);
 
   newCapacity =
-      (threadEntry->meta &&
-       (bigCapacity <= threadEntry->meta->head_.getElementsCapacity()))
+      (threadEntry->meta && (bigCapacity <= threadEntry->meta->nextId_))
       ? bigCapacity
       : smallCapacity;
 
@@ -380,6 +352,13 @@ ElementWrapper* StaticMetaBase::reallocate(
     if (!reallocated) {
       throw_exception<std::bad_alloc>();
     }
+
+    // When the main thread exits, it will call functions registered with
+    // 'atexit' and then call 'exit()'. However, It will NOT call any functions
+    // registered via the 'TLS' feature of pthread_key_create.
+    // Reference:
+    // https://pubs.opengroup.org/onlinepubs/9699919799/functions/pthread_create.html
+    folly::lsan_ignore_object(reallocated);
   }
   return reallocated;
 }
@@ -422,60 +401,42 @@ void StaticMetaBase::reserve(EntryID* id) {
       std::swap(reallocated, threadEntry->elements);
     }
 
-    for (size_t i = prevCapacity; i < newCapacity; i++) {
-      threadEntry->elements[i].node.initZero(threadEntry, i);
-    }
-
     threadEntry->setElementsCapacity(newCapacity);
   }
+  for (size_t i = prevCapacity; i < newCapacity; ++i) {
+    threadEntry->elements[i].isLinked = false;
+  }
 
   meta.totalElementWrappers_ += (newCapacity - prevCapacity);
   free(reallocated);
 }
 
-void StaticMetaBase::reserveHeadUnlocked(uint32_t id) {
-  if (head_.getElementsCapacity() <= id) {
-    size_t prevCapacity = head_.getElementsCapacity();
-    size_t newCapacity;
-    ElementWrapper* reallocated = reallocate(&head_, id, newCapacity);
-
-    if (reallocated) {
-      if (prevCapacity != 0) {
-        memcpy(
-            reallocated, head_.elements, sizeof(*reallocated) * prevCapacity);
-      }
-      std::swap(reallocated, head_.elements);
-    }
-
-    for (size_t i = prevCapacity; i < newCapacity; i++) {
-      head_.elements[i].node.init(&head_, i);
-    }
-
-    head_.setElementsCapacity(newCapacity);
-    totalElementWrappers_ += (newCapacity - prevCapacity);
-    free(reallocated);
-  }
+/*
+ * Evict threadEntry for @id from allThreadEntryMap_
+ * ThreadEntry* set and release the element @id.
+ */
+void* ThreadEntry::releaseElement(uint32_t id) {
+  return elements[id].release();
 }
 
-void StaticMetaBase::pushBackLocked(ThreadEntry* t, uint32_t id) noexcept {
-  if (FOLLY_LIKELY(!t->removed_)) {
-    std::lock_guard<std::mutex> g(lock_);
-    auto* node = &t->elements[id].node;
-    node->push_back(&head_);
-  }
-}
-
-void StaticMetaBase::pushBackUnlocked(ThreadEntry* t, uint32_t id) noexcept {
-  if (FOLLY_LIKELY(!t->removed_)) {
-    auto* node = &t->elements[id].node;
-    node->push_back(&head_);
+/*
+ * Cleanup the element and set the thread entry in the map, if valid.
+ */
+void ThreadEntry::cleanupElementAndSetThreadEntry(
+    uint32_t id, bool validThreadEntry) {
+  elements[id].dispose(TLPDestructionMode::THIS_THREAD);
+  // Cleanup
+  elements[id].cleanup();
+  // Add the allThreadEntryMap_ only iff newPtr is not nullptr and threadEntry
+  // is not marked as removed
+  if (validThreadEntry) {
+    meta->addThreadEntryToMap(this, id);
   }
 }
 
 FOLLY_STATIC_CTOR_PRIORITY_MAX
 PthreadKeyUnregister PthreadKeyUnregister::instance_;
-#if defined(__GLIBC__) && \
-    (__GLIBC__ < 2 || (__GLIBC__ == 2 && __GLIBC_MINOR__ < 18))
+#if defined(__GLIBC__)
 // Invoking thread_local dtor register early to fix issue
 // https://github.com/facebook/folly/issues/1252
 struct GlibcThreadLocalInit {
@@ -489,8 +450,8 @@ struct GlibcThreadLocalInit {
     compiler_must_not_elide(glibcThreadLocalInit);
   }
 };
-__attribute__((__init_priority__(101)))
-GlibcThreadLocalInit glibcThreadLocalInit;
+__attribute__((
+    __init_priority__(101))) GlibcThreadLocalInit glibcThreadLocalInit;
 #endif
 } // namespace threadlocal_detail
 } // namespace folly
