@@ -17,7 +17,9 @@
 #include <folly/executors/SerialExecutor.h>
 
 #include <chrono>
+#include <optional>
 
+#include <folly/Random.h>
 #include <folly/ScopeGuard.h>
 #include <folly/executors/CPUThreadPoolExecutor.h>
 #include <folly/executors/InlineExecutor.h>
@@ -25,16 +27,26 @@
 #include <folly/portability/GTest.h>
 #include <folly/synchronization/Baton.h>
 
-using namespace std::chrono;
-using folly::SerialExecutor;
-
 namespace {
-void burnMs(uint64_t ms) {
-  /* sleep override */ std::this_thread::sleep_for(milliseconds(ms));
+
+void sleepMs(uint64_t ms) {
+  /* sleep override */ std::this_thread::sleep_for(
+      std::chrono::milliseconds(ms));
 }
+
 } // namespace
 
-void simpleTest(std::shared_ptr<folly::Executor> const& parent) {
+template <typename T>
+class SerialExecutorTest : public testing::Test {};
+
+using SerialExecutorTypes = ::testing::Types<
+    folly::SerialExecutor,
+    folly::SerialExecutorWithLgSegmentSize<5>,
+    folly::SmallSerialExecutor>;
+TYPED_TEST_SUITE(SerialExecutorTest, SerialExecutorTypes);
+
+template <class SerialExecutorType>
+void simpleTest(folly::Executor& parent) {
   class SerialExecutorContextData : public folly::RequestData {
    public:
     static std::string kCtxKey() {
@@ -48,18 +60,16 @@ void simpleTest(std::shared_ptr<folly::Executor> const& parent) {
     const int id_;
   };
 
-  auto executor =
-      SerialExecutor::create(folly::getKeepAliveToken(parent.get()));
+  auto executor = SerialExecutorType::create(&parent);
 
   std::vector<int> values;
   std::vector<int> expected;
 
   for (int i = 0; i < 20; ++i) {
-    auto ctx = std::make_shared<folly::RequestContext>();
-    ctx->setContextData(
+    folly::RequestContextScopeGuard ctxGuard;
+    folly::RequestContext::try_get()->setContextData(
         SerialExecutorContextData::kCtxKey(),
         std::make_unique<SerialExecutorContextData>(i));
-    folly::RequestContextScopeGuard ctxGuard(ctx);
     auto checkReqCtx = [i] {
       EXPECT_EQ(
           i,
@@ -72,7 +82,7 @@ void simpleTest(std::shared_ptr<folly::Executor> const& parent) {
       checkReqCtx();
       // make this extra vulnerable to concurrent execution
       values.push_back(0);
-      burnMs(10);
+      sleepMs(10);
       values.back() = i;
     });
     expected.push_back(i);
@@ -86,20 +96,20 @@ void simpleTest(std::shared_ptr<folly::Executor> const& parent) {
   EXPECT_EQ(expected, values);
 }
 
-TEST(SerialExecutor, Simple) {
-  simpleTest(std::make_shared<folly::CPUThreadPoolExecutor>(4));
+TYPED_TEST(SerialExecutorTest, Simple) {
+  folly::CPUThreadPoolExecutor parent{4};
+  simpleTest<TypeParam>(parent);
 }
-TEST(SerialExecutor, SimpleInline) {
-  simpleTest(std::make_shared<folly::InlineExecutor>());
+TYPED_TEST(SerialExecutorTest, SimpleInline) {
+  simpleTest<TypeParam>(folly::InlineExecutor::instance());
 }
 
 // The Afterlife test only works with an asynchronous executor (not the
 // InlineExecutor), because we want execution of tasks to happen after we
 // destroy the SerialExecutor
-TEST(SerialExecutor, Afterlife) {
-  auto cpu_executor = std::make_shared<folly::CPUThreadPoolExecutor>(4);
-  auto executor =
-      SerialExecutor::create(folly::getKeepAliveToken(cpu_executor.get()));
+TYPED_TEST(SerialExecutorTest, Afterlife) {
+  folly::CPUThreadPoolExecutor parent{4};
+  auto executor = TypeParam::create(&parent);
 
   // block executor until we call start_baton.post()
   folly::Baton<> start_baton;
@@ -112,7 +122,7 @@ TEST(SerialExecutor, Afterlife) {
     executor->add([i, &values] {
       // make this extra vulnerable to concurrent execution
       values.push_back(0);
-      burnMs(10);
+      sleepMs(10);
       values.back() = i;
     });
     expected.push_back(i);
@@ -133,9 +143,9 @@ TEST(SerialExecutor, Afterlife) {
   EXPECT_EQ(expected, values);
 }
 
-void RecursiveAddTest(std::shared_ptr<folly::Executor> const& parent) {
-  auto executor =
-      SerialExecutor::create(folly::getKeepAliveToken(parent.get()));
+template <class SerialExecutorType>
+void recursiveAddTest(folly::Executor& parent) {
+  auto executor = SerialExecutorType::create(&parent);
 
   folly::Baton<> finished_baton;
 
@@ -147,7 +157,7 @@ void RecursiveAddTest(std::shared_ptr<folly::Executor> const& parent) {
     if (i < 10) {
       // make this extra vulnerable to concurrent execution
       values.push_back(0);
-      burnMs(10);
+      sleepMs(10);
       values.back() = i;
       executor->add(lambda);
     } else if (i < 12) {
@@ -171,30 +181,112 @@ void RecursiveAddTest(std::shared_ptr<folly::Executor> const& parent) {
   EXPECT_EQ(expected, values);
 }
 
-TEST(SerialExecutor, RecursiveAdd) {
-  RecursiveAddTest(std::make_shared<folly::CPUThreadPoolExecutor>(4));
+TYPED_TEST(SerialExecutorTest, RecursiveAdd) {
+  folly::CPUThreadPoolExecutor parent{4};
+  recursiveAddTest<TypeParam>(parent);
 }
-TEST(SerialExecutor, RecursiveAddInline) {
-  RecursiveAddTest(std::make_shared<folly::InlineExecutor>());
+TYPED_TEST(SerialExecutorTest, RecursiveAddInline) {
+  recursiveAddTest<TypeParam>(folly::InlineExecutor::instance());
 }
 
-TEST(SerialExecutor, ExecutionThrows) {
-  auto executor = SerialExecutor::create();
+TYPED_TEST(SerialExecutorTest, ExecutionThrows) {
+  auto executor = TypeParam::create();
 
   // an empty Func will throw std::bad_function_call when invoked,
   // but SerialExecutor should catch that exception
   executor->add(folly::Func{});
 }
 
-TEST(SerialExecutor, ParentExecutorDiscardsFunc) {
+TYPED_TEST(SerialExecutorTest, ParentExecutorDiscardsFunc) {
   struct FakeExecutor : folly::Executor {
-    void add(folly::Func) override {}
+    void add(folly::Func f) override { queue.push_back(std::move(f)); }
+
+    std::vector<folly::Func> queue;
   };
 
-  FakeExecutor ex;
-  auto se = folly::SerialExecutor::create(&ex);
+  std::optional<FakeExecutor> ex{std::in_place};
+  auto se = TypeParam::create(&*ex);
   bool ran = false;
-  se->add([&, ka = folly::getKeepAliveToken(se.get())] { ran = true; });
+  bool destructorRan = false;
+  {
+    folly::RequestContextScopeGuard ctxGuard;
+    auto destructionGuard = folly::makeGuard(
+        [&destructorRan, ctx = folly::RequestContext::try_get()] {
+          destructorRan = true;
+          EXPECT_EQ(ctx, folly::RequestContext::try_get());
+        });
+    se->add([&,
+             ka = folly::getKeepAliveToken(se.get()),
+             dg = std::move(destructionGuard)] { ran = true; });
+    EXPECT_FALSE(destructorRan); // Task is still stuck in the queue.
+  }
   se.reset();
+  ex.reset();
+  EXPECT_TRUE(destructorRan);
   ASSERT_FALSE(ran);
 }
+
+TYPED_TEST(SerialExecutorTest, Stress) {
+  folly::CPUThreadPoolExecutor parent{4};
+  auto se = TypeParam::create(&parent);
+
+  size_t tasksRan = 0;
+  constexpr size_t kNumProducers = 16;
+  static constexpr size_t kNumIterations = 4096;
+  folly::CPUThreadPoolExecutor producers{kNumProducers};
+  for (size_t i = 0; i < kNumProducers; ++i) {
+    producers.add([i, se, &tasksRan] {
+      for (size_t j = 0; j < kNumIterations; ++j) {
+        se->add([i, j, se, &tasksRan] {
+          if (i == j) {
+            // Do a few recursive adds just in case.
+            se->add([&tasksRan] { ++tasksRan; });
+          }
+          ++tasksRan;
+        });
+      }
+    });
+  }
+
+  producers.join();
+  se = {};
+  parent.join();
+  EXPECT_EQ(tasksRan, kNumProducers * (kNumIterations + 1));
+}
+
+// Basic test for SerialExecutorMPSCQueue, does not exercise concurrent access
+// but just ensure that the state stays consistent under different
+// enqueue/dequeue patterns.
+TEST(SerialExecutorTest2, SerialExecutorMPSCQueue) {
+  folly::detail::SerialExecutorMPSCQueue<std::unique_ptr<size_t>> q;
+  size_t size = 0;
+
+  auto produce = [&q, &size, nextWrite = 0](size_t n) mutable {
+    for (size_t i = 0; i < n; ++i) {
+      q.enqueue(std::make_unique<size_t>(nextWrite++));
+    }
+    size += n;
+  };
+
+  auto consume = [&q, &size, nextRead = 0](size_t n) mutable {
+    for (size_t j = 0; j < n; ++j) {
+      std::unique_ptr<size_t> entry;
+      q.dequeue(entry);
+      EXPECT_EQ(*entry, nextRead++);
+    }
+    size -= n;
+  };
+
+  for (size_t round = 0; round < 1024; ++round) {
+    size_t toEnqueue = folly::Random::rand64(128) + 1;
+    produce(toEnqueue);
+
+    // Consume completely with 20% probability, otherwise leave several entries
+    // (possibly multiple segments).
+    size_t toDequeue = folly::Random::oneIn(5)
+        ? size
+        : size - folly::Random::rand64(std::min<size_t>(size, 128));
+    consume(toDequeue);
+  }
+  consume(size);
+}
