@@ -14,6 +14,7 @@
  * limitations under the License.
  */
 
+#include <atomic>
 #include <utility>
 
 #include <folly/CancellationToken.h>
@@ -43,6 +44,32 @@ inline Unit getValueOrUnit([[maybe_unused]] Try<void>&& value) {
   return Unit{};
 }
 
+template <
+    typename InputRange,
+    typename Make,
+    typename Iter = invoke_result_t<access::begin_fn, InputRange&>,
+    typename Elem = remove_cvref_t<decltype(*std::declval<Iter&>())>,
+    typename RTask = invoke_result_t<Make&, Elem, std::size_t>>
+std::vector<RTask> collectMakeInnerTaskVec(InputRange& awaitables, Make& make) {
+  std::vector<RTask> tasks;
+
+  auto abegin = access::begin(awaitables);
+  auto aend = access::end(awaitables);
+
+  if constexpr (is_invocable_v<folly::access::size_fn, InputRange&>) {
+    tasks.reserve(static_cast<std::size_t>(folly::access::size(awaitables)));
+  } else if constexpr (range_has_known_distance_v<InputRange&>) {
+    tasks.reserve(static_cast<std::size_t>(std::distance(abegin, aend)));
+  }
+
+  std::size_t index = 0;
+  for (auto aiter = abegin; aiter != aend; ++aiter) {
+    tasks.push_back(make(std::move(*aiter), index++));
+  }
+
+  return tasks;
+}
+
 template <typename SemiAwaitable, typename Result>
 BarrierTask makeCollectAllTryTask(
     Executor::KeepAlive<> executor,
@@ -242,7 +269,7 @@ auto makeUnorderedAsyncGeneratorImpl(
       RequestContext::setContext(context);
     }
 
-    while (true) {
+    while (expected > 0) {
       CancellationCallback cancelCallback(
           co_await co_current_cancellation_token,
           [&]() noexcept { cancelSource.requestCancellation(); });
@@ -260,7 +287,7 @@ auto makeUnorderedAsyncGeneratorImpl(
         co_yield co_result(std::move(result));
       } else {
         // Prevent AsyncPipe from receiving cancellation so we get the right
-        // number of OperationCancelleds.
+        // number of OperationCancelled.
         auto result = co_await co_withCancellation({}, results.next());
         co_yield std::move(*result);
         if (--expected == 0) {
@@ -304,6 +331,38 @@ auto collectAnyImpl(
   co_return firstCompletion;
 }
 
+template <typename... SemiAwaitables, size_t... Indices>
+auto collectAnyWithoutExceptionImpl(
+    std::index_sequence<Indices...>, SemiAwaitables&&... awaitables)
+    -> folly::coro::Task<std::pair<
+        std::size_t,
+        folly::Try<detail::collect_any_component_t<SemiAwaitables...>>>> {
+  const CancellationToken& parentCancelToken =
+      co_await co_current_cancellation_token;
+  const CancellationSource cancelSource;
+  const CancellationToken cancelToken =
+      CancellationToken::merge(parentCancelToken, cancelSource.getToken());
+
+  constexpr std::size_t nAwaitables = sizeof...(SemiAwaitables);
+  std::atomic<std::size_t> nAwaited = 1;
+  std::pair<std::size_t, folly::Try<collect_any_component_t<SemiAwaitables...>>>
+      firstValueOrLastException;
+  firstValueOrLastException.first = std::numeric_limits<size_t>::max();
+  co_await folly::coro::collectAll(folly::coro::co_withCancellation(
+      cancelToken, [&]() -> folly::coro::Task<void> {
+        auto result = co_await folly::coro::co_awaitTry(
+            std::forward<SemiAwaitables>(awaitables));
+        if ((result.hasValue() ||
+             nAwaited.fetch_add(1, std::memory_order_relaxed) == nAwaitables) &&
+            !cancelSource.requestCancellation()) {
+          firstValueOrLastException.first = Indices;
+          firstValueOrLastException.second = std::move(result);
+        }
+      }())...);
+
+  co_return firstValueOrLastException;
+}
+
 template <typename... SemiAwaitables, size_t... Indices>
 auto collectAnyNoDiscardImpl(
     std::index_sequence<Indices...>, SemiAwaitables&&... awaitables)
@@ -328,8 +387,9 @@ auto collectAnyNoDiscardImpl(
 } // namespace detail
 
 template <typename... SemiAwaitables>
-auto collectAll(SemiAwaitables&&... awaitables) -> folly::coro::Task<std::tuple<
-    detail::collect_all_component_t<remove_cvref_t<SemiAwaitables>>...>> {
+auto collectAll(SemiAwaitables&&... awaitables)
+    -> folly::coro::Task<std::tuple<
+        detail::collect_all_component_t<remove_cvref_t<SemiAwaitables>>...>> {
   return detail::collectAllImpl(
       std::make_index_sequence<sizeof...(SemiAwaitables)>{},
       static_cast<SemiAwaitables&&>(awaitables)...);
@@ -380,19 +440,9 @@ auto collectAllRange(InputRange awaitables)
     }
   };
 
-  // Create a task to await each input awaitable.
-  std::vector<detail::BarrierTask> tasks;
+  auto tasks = detail::collectMakeInnerTaskVec(awaitables, makeTask);
 
-  // TODO: Detect when the input range supports constant-time
-  // .size() and pre-reserve storage for that many elements in 'tasks'.
-
-  std::size_t taskCount = 0;
-  for (auto&& semiAwaitable : static_cast<InputRange&&>(awaitables)) {
-    tasks.push_back(makeTask(
-        static_cast<decltype(semiAwaitable)&&>(semiAwaitable), taskCount++));
-  }
-
-  tryResults.resize(taskCount);
+  tryResults.resize(tasks.size());
 
   // Save the initial context and restore it after starting each task
   // as the task may have modified the context before suspending and we
@@ -443,7 +493,8 @@ auto collectAllRange(InputRange awaitables) -> folly::coro::Task<void> {
   exception_wrapper firstException;
 
   using awaitable_type = remove_cvref_t<detail::range_reference_t<InputRange>>;
-  auto makeTask = [&](awaitable_type semiAwaitable) -> detail::BarrierTask {
+  auto makeTask = [&](awaitable_type semiAwaitable,
+                      std::size_t) -> detail::BarrierTask {
     try {
       co_await co_viaIfAsync(
           executor.get_alias(),
@@ -455,16 +506,7 @@ auto collectAllRange(InputRange awaitables) -> folly::coro::Task<void> {
     }
   };
 
-  // Create a task to await each input awaitable.
-  std::vector<detail::BarrierTask> tasks;
-
-  // TODO: Detect when the input range supports constant-time
-  // .size() and pre-reserve storage for that many elements in 'tasks'.
-
-  for (auto&& semiAwaitable : static_cast<InputRange&&>(awaitables)) {
-    tasks.push_back(
-        makeTask(static_cast<decltype(semiAwaitable)&&>(semiAwaitable)));
-  }
+  auto tasks = detail::collectMakeInnerTaskVec(awaitables, makeTask);
 
   // Save the initial context and restore it after starting each task
   // as the task may have modified the context before suspending and we
@@ -504,8 +546,8 @@ auto collectAllTryRange(InputRange awaitables)
   const CancellationToken& cancelToken = co_await co_current_cancellation_token;
 
   using awaitable_type = remove_cvref_t<detail::range_reference_t<InputRange>>;
-  auto makeTask = [&](std::size_t index,
-                      awaitable_type semiAwaitable) -> detail::BarrierTask {
+  auto makeTask = [&](awaitable_type semiAwaitable,
+                      std::size_t index) -> detail::BarrierTask {
     assert(index < results.size());
     auto& result = results[index];
     try {
@@ -525,19 +567,7 @@ auto collectAllTryRange(InputRange awaitables)
     }
   };
 
-  // Create a task to await each input awaitable.
-  std::vector<detail::BarrierTask> tasks;
-
-  // TODO: Detect when the input range supports constant-time
-  // .size() and pre-reserve storage for that many elements in 'tasks'.
-
-  {
-    std::size_t index = 0;
-    for (auto&& semiAwaitable : awaitables) {
-      tasks.push_back(makeTask(
-          index++, static_cast<decltype(semiAwaitable)&&>(semiAwaitable)));
-    }
-  }
+  auto tasks = detail::collectMakeInnerTaskVec(awaitables, makeTask);
 
   // Now that we know how many tasks there are, allocate that
   // many Try objects to store the results before we start
@@ -957,7 +987,7 @@ auto makeUnorderedAsyncGenerator(AsyncScope& scope, InputRange awaitables)
         InputRange,
         false>&&> {
   return detail::makeUnorderedAsyncGeneratorImpl(
-      scope, std::move(awaitables), bool_constant<false>{});
+      scope, std::move(awaitables), std::bool_constant<false>{});
 }
 
 template <typename InputRange>
@@ -966,7 +996,7 @@ auto makeUnorderedTryAsyncGenerator(AsyncScope& scope, InputRange awaitables)
         InputRange,
         true>&&> {
   return detail::makeUnorderedAsyncGeneratorImpl(
-      scope, std::move(awaitables), bool_constant<true>{});
+      scope, std::move(awaitables), std::bool_constant<true>{});
 }
 
 template <typename InputRange>
@@ -976,7 +1006,7 @@ auto makeUnorderedAsyncGenerator(
         InputRange,
         false>&&> {
   return detail::makeUnorderedAsyncGeneratorImpl(
-      scope, std::move(awaitables), bool_constant<false>{});
+      scope, std::move(awaitables), std::bool_constant<false>{});
 }
 
 template <typename InputRange>
@@ -986,7 +1016,7 @@ auto makeUnorderedTryAsyncGenerator(
         InputRange,
         true>&&> {
   return detail::makeUnorderedAsyncGeneratorImpl(
-      scope, std::move(awaitables), bool_constant<true>{});
+      scope, std::move(awaitables), std::bool_constant<true>{});
 }
 
 template <typename SemiAwaitable, typename... SemiAwaitables>
@@ -1002,6 +1032,16 @@ auto collectAny(SemiAwaitable&& awaitable, SemiAwaitables&&... awaitables)
       static_cast<SemiAwaitables&&>(awaitables)...);
 }
 
+template <typename... SemiAwaitables>
+auto collectAnyWithoutException(SemiAwaitables&&... awaitables)
+    -> folly::coro::Task<std::pair<
+        std::size_t,
+        folly::Try<detail::collect_any_component_t<SemiAwaitables...>>>> {
+  return detail::collectAnyWithoutExceptionImpl(
+      std::make_index_sequence<sizeof...(SemiAwaitables)>{},
+      static_cast<SemiAwaitables&&>(awaitables)...);
+}
+
 template <typename... SemiAwaitables>
 auto collectAnyNoDiscard(SemiAwaitables&&... awaitables)
     -> folly::coro::Task<std::tuple<detail::collect_all_try_component_t<
@@ -1012,10 +1052,11 @@ auto collectAnyNoDiscard(SemiAwaitables&&... awaitables)
 }
 
 template <typename InputRange>
-auto collectAnyRange(InputRange awaitables) -> folly::coro::Task<std::pair<
-    size_t,
-    folly::Try<detail::collect_all_range_component_t<
-        detail::range_reference_t<InputRange>>>>> {
+auto collectAnyRange(InputRange awaitables)
+    -> folly::coro::Task<std::pair<
+        size_t,
+        folly::Try<detail::collect_all_range_component_t<
+            detail::range_reference_t<InputRange>>>>> {
   const CancellationToken& parentCancelToken =
       co_await co_current_cancellation_token;
   const CancellationSource cancelSource;
@@ -1040,17 +1081,7 @@ auto collectAnyRange(InputRange awaitables) -> folly::coro::Task<std::pair<
     }
   };
 
-  // Create a task to await each input awaitable.
-  std::vector<folly::coro::Task<void>> tasks;
-
-  // TODO: Detect when the input range supports constant-time
-  // .size() and pre-reserve storage for that many elements in 'tasks'.
-
-  size_t taskCount = 0;
-  for (auto&& semiAwaitable : static_cast<InputRange&&>(awaitables)) {
-    tasks.push_back(makeTask(
-        static_cast<decltype(semiAwaitable)&&>(semiAwaitable), taskCount++));
-  }
+  auto tasks = detail::collectMakeInnerTaskVec(awaitables, makeTask);
 
   co_await folly::coro::co_withCancellation(
       cancelToken, folly::coro::collectAllRange(detail::MoveRange(tasks)));
@@ -1058,6 +1089,47 @@ auto collectAnyRange(InputRange awaitables) -> folly::coro::Task<std::pair<
   co_return firstCompletion;
 }
 
+template <typename InputRange>
+auto collectAnyWithoutExceptionRange(InputRange awaitables)
+    -> folly::coro::Task<std::pair<
+        size_t,
+        folly::Try<detail::collect_all_range_component_t<
+            detail::range_reference_t<InputRange>>>>> {
+  const CancellationToken& parentCancelToken =
+      co_await co_current_cancellation_token;
+  const CancellationSource cancelSource;
+  const CancellationToken cancelToken =
+      CancellationToken::merge(parentCancelToken, cancelSource.getToken());
+
+  size_t nAwaitables;
+  std::atomic<std::size_t> nAwaited = 1;
+  std::pair<
+      size_t,
+      folly::Try<detail::collect_all_range_component_t<
+          detail::range_reference_t<InputRange>>>>
+      firstValueOrLastException;
+  firstValueOrLastException.first = std::numeric_limits<size_t>::max();
+
+  using awaitable_type = remove_cvref_t<detail::range_reference_t<InputRange>>;
+  auto makeTask = [&](awaitable_type semiAwaitable,
+                      size_t index) -> folly::coro::Task<void> {
+    auto result = co_await folly::coro::co_awaitTry(std::move(semiAwaitable));
+    if ((result.hasValue() ||
+         nAwaited.fetch_add(1, std::memory_order_relaxed) == nAwaitables) &&
+        !cancelSource.requestCancellation()) {
+      firstValueOrLastException.first = index;
+      firstValueOrLastException.second = std::move(result);
+    }
+  };
+
+  auto tasks = detail::collectMakeInnerTaskVec(awaitables, makeTask);
+  nAwaitables = tasks.size();
+  co_await folly::coro::co_withCancellation(
+      cancelToken, folly::coro::collectAllRange(detail::MoveRange(tasks)));
+
+  co_return firstValueOrLastException;
+}
+
 template <typename InputRange>
 auto collectAnyNoDiscardRange(InputRange awaitables)
     -> folly::coro::Task<std::vector<detail::collect_all_try_range_component_t<
@@ -1080,19 +1152,9 @@ auto collectAnyNoDiscardRange(InputRange awaitables)
     results[index] = std::move(result);
   };
 
-  // Create a task to await each input awaitable.
-  std::vector<folly::coro::Task<void>> tasks;
+  auto tasks = detail::collectMakeInnerTaskVec(awaitables, makeTask);
 
-  // TODO: Detect when the input range supports constant-time
-  // .size() and pre-reserve storage for that many elements in 'tasks'.
-
-  size_t taskCount = 0;
-  for (auto&& semiAwaitable : static_cast<InputRange&&>(awaitables)) {
-    tasks.push_back(makeTask(
-        static_cast<decltype(semiAwaitable)&&>(semiAwaitable), taskCount++));
-  }
-
-  results.resize(taskCount);
+  results.resize(tasks.size());
   co_await folly::coro::co_withCancellation(
       cancelToken, folly::coro::collectAllRange(detail::MoveRange(tasks)));
 
