@@ -122,8 +122,9 @@ class MoveRange {
 //       co_await folly::coro::collectAll(doSomething(), doSomethingElse());
 //
 template <typename... SemiAwaitables>
-auto collectAll(SemiAwaitables&&... awaitables) -> folly::coro::Task<std::tuple<
-    detail::collect_all_component_t<remove_cvref_t<SemiAwaitables>>...>>;
+auto collectAll(SemiAwaitables&&... awaitables)
+    -> folly::coro::Task<std::tuple<
+        detail::collect_all_component_t<remove_cvref_t<SemiAwaitables>>...>>;
 
 ///////////////////////////////////////////////////////////////////////////
 // collectAllTry(SemiAwaitable<Ts>...)
@@ -455,6 +456,37 @@ auto collectAny(SemiAwaitable&& awaitable, SemiAwaitables&&... awaitables)
             SemiAwaitable,
             SemiAwaitables...>>>>;
 
+///////////////////////////////////////////////////////////////////////////
+// collectAnyWithoutException(SemiAwaitable<Ts>...)
+//    -> SemiAwaitable<std::pair<std::size_t, folly::Try<T>>>
+//
+// The collectAnyWithoutException() function is similar to collectAny() in that
+// it co_awaits multiple SemiAwaitables and cancels any outstanding operations
+// when complete. Unlike collectAny(), it returns the first success, or the last
+// exception if all of the SemiAwaitables fail.
+//
+// collectAnyWithoutException() is built on top of collectAll(), be aware of the
+// coroutine starting behavior described in collectAll() documentation.
+//
+// The result of the first successful SemiAwaitable, or the the exception from
+// the last SemiAwaitable is returned if none are successful. Any result of the
+// remaining SemiAwaitables will be discarded, independently of whether it's a
+// value or an exception.
+//
+// Example:
+//   folly::coro::Task<Foo> getDataOneWay();
+//   folly::coro::Task<Foo> getDataAnotherWay();
+//
+//   std::pair<std::size_t, Try<Foo>> result =
+//       co_await folly::coro::collectAnyWithoutException(
+//           getDataOneWay(), getDataAnotherWay());
+//
+template <typename... SemiAwaitables>
+auto collectAnyWithoutException(SemiAwaitables&&... awaitables)
+    -> folly::coro::Task<std::pair<
+        std::size_t,
+        folly::Try<detail::collect_any_component_t<SemiAwaitables...>>>>;
+
 ///////////////////////////////////////////////////////////////////////////
 // collectAnyNoDiscard(SemiAwaitable<Ts>...) ->
 //   SemiAwaitable<std::tuple<folly::Try<Ts>...>>
@@ -508,10 +540,42 @@ auto collectAnyNoDiscard(SemiAwaitables&&... awaitables)
 //       ranges::views::move);
 //
 template <typename InputRange>
-auto collectAnyRange(InputRange awaitables) -> folly::coro::Task<std::pair<
-    size_t,
-    folly::Try<detail::collect_all_range_component_t<
-        detail::range_reference_t<InputRange>>>>>;
+auto collectAnyRange(InputRange awaitables)
+    -> folly::coro::Task<std::pair<
+        size_t,
+        folly::Try<detail::collect_all_range_component_t<
+            detail::range_reference_t<InputRange>>>>>;
+
+///////////////////////////////////////////////////////////////////////////
+// collectAnyWithoutExceptionRange(RangeOf<SemiAwaitable<T>>&&)
+//   -> SemiAwaitable<std::pair<std::size_t, folly::Try<T>>>
+//
+// The collectAnyWithoutExceptionRange() function is similar to
+// collectAnyRange() in that it co_awaits multiple SemiAwaitables and cancels
+// any outstanding operations when complete. Unlike collectAnyRange(), it
+// returns the first success, or the last exception if all of the SemiAwaitables
+// fail.
+//
+// collectAnyWithoutExceptionRange() is built on top of collectAllRange(), be
+// aware of the coroutine starting behavior described in collectAll()
+// documentation.
+//
+// The result of the first successful SemiAwaitable, or the the exception from
+// the last SemiAwaitable is returned if none are successful. Any result of the
+// remaining SemiAwaitables will be discarded, independently of whether it's a
+// value or an exception.
+//
+// Example:
+//   std::vector<Task<T>> tasks = ...;
+//   std::pair<size_t, Try<T>> result =
+//       co_await collectAnyWithoutExceptionRange(tasks | ranges::views::move);
+//
+template <typename InputRange>
+auto collectAnyWithoutExceptionRange(InputRange awaitables)
+    -> folly::coro::Task<std::pair<
+        size_t,
+        folly::Try<detail::collect_all_range_component_t<
+            detail::range_reference_t<InputRange>>>>>;
 
 ///////////////////////////////////////////////////////////////////////////
 // collectAnyNoDiscardRange(RangeOf<SemiAwaitable<T>>&&)
@@ -541,8 +605,9 @@ auto collectAnyNoDiscardRange(InputRange awaitables)
     -> folly::coro::Task<std::vector<detail::collect_all_try_range_component_t<
         detail::range_reference_t<InputRange>>>>;
 
-// collectAnyRange()/collectAnyNoDiscardRange() overloads that simplifies the
-// common-case where an rvalue std::vector<SemiAwaitable> is passed.
+// collectAnyRange()/collectAnyWithoutExceptionRange()/collectAnyNoDiscardRange()
+// overloads that simplifies the common-case where an rvalue
+// std::vector<SemiAwaitable> is passed.
 //
 // This avoids the caller needing to pipe the input through ranges::views::move
 // transform to force the elements to be rvalue-references since the
@@ -554,6 +619,13 @@ auto collectAnyRange(std::vector<SemiAwaitable> awaitables)
   co_return co_await collectAnyRange(detail::MoveRange(awaitables));
 }
 template <typename SemiAwaitable>
+auto collectAnyWithoutExceptionRange(std::vector<SemiAwaitable> awaitables)
+    -> decltype(collectAnyWithoutExceptionRange(
+        detail::MoveRange(awaitables))) {
+  co_return co_await collectAnyWithoutExceptionRange(
+      detail::MoveRange(awaitables));
+}
+template <typename SemiAwaitable>
 auto collectAnyNoDiscardRange(std::vector<SemiAwaitable> awaitables)
     -> decltype(collectAnyNoDiscardRange(detail::MoveRange(awaitables))) {
   co_return co_await collectAnyNoDiscardRange(detail::MoveRange(awaitables));
