@@ -76,10 +76,6 @@ class ViaCoroutinePromiseBase {
 
  protected:
   void scheduleContinuation() noexcept {
-    // Pass the coroutine's RequestContext to Executor::add(), in case the
-    // Executor implementation wants to know what runs on it (e.g. for stats).
-    RequestContextScopeGuard contextScope{context_};
-
     executor_->add([this]() noexcept { this->executeContinuation(); });
   }
 
@@ -438,7 +434,8 @@ class ViaIfAsyncAwaitable {
 namespace detail {
 
 template <typename SemiAwaitable, typename = void>
-struct HasViaIfAsyncMethod : bool_constant<!require_sizeof<SemiAwaitable>> {};
+struct HasViaIfAsyncMethod
+    : std::bool_constant<!require_sizeof<SemiAwaitable>> {};
 
 template <typename SemiAwaitable>
 struct HasViaIfAsyncMethod<
@@ -501,7 +498,7 @@ struct ViaIfAsyncFunction {
 FOLLY_DEFINE_CPO(detail::adl::ViaIfAsyncFunction, co_viaIfAsync)
 
 template <typename T, typename = void>
-struct is_semi_awaitable : bool_constant<!require_sizeof<T>> {};
+struct is_semi_awaitable : std::bool_constant<!require_sizeof<T>> {};
 
 template <typename T>
 struct is_semi_awaitable<T, std::enable_if_t<std::is_void_v<T>>>
@@ -632,7 +629,7 @@ class TryAwaitable : public CommutativeWrapperAwaitable<TryAwaitable, T> {
           int> = 0,
       typename T2 = like_t<Self, T>,
       std::enable_if_t<is_awaitable_v<T2>, int> = 0>
-  friend TryAwaiter<T2> operator co_await(Self&& self) {
+  friend TryAwaiter<T2> operator co_await(Self && self) {
     return TryAwaiter<T2>{static_cast<Self&&>(self).inner_};
   }
 };
