@@ -66,7 +66,9 @@ TEST_F(AsyncGeneratorTest, PartiallyConsumingSequenceDestroysObjectsInScope) {
   bool started = false;
   bool destroyed = false;
   auto makeGenerator = [&]() -> folly::coro::AsyncGenerator<int> {
-    SCOPE_EXIT { destroyed = true; };
+    SCOPE_EXIT {
+      destroyed = true;
+    };
     started = true;
     co_yield 1;
     co_yield 2;
