@@ -17,13 +17,14 @@
 #pragma once
 
 #include <folly/experimental/io/AsyncIoUringSocket.h>
+#include <folly/experimental/io/Liburing.h>
 
 namespace folly {
 
 class AsyncIoUringSocketFactory {
  public:
-  static bool supports(FOLLY_MAYBE_UNUSED folly::EventBase* eb) {
-#if defined(__linux__) && __has_include(<liburing.h>)
+  static bool supports([[maybe_unused]] folly::EventBase* eb) {
+#if FOLLY_HAS_LIBURING
     return AsyncIoUringSocket::supports(eb);
 #else
     return false;
@@ -31,8 +32,8 @@ class AsyncIoUringSocketFactory {
   }
 
   template <class TWrapper, class... Args>
-  static TWrapper create(FOLLY_MAYBE_UNUSED Args&&... args) {
-#if defined(__linux__) && __has_include(<liburing.h>)
+  static TWrapper create([[maybe_unused]] Args&&... args) {
+#if FOLLY_HAS_LIBURING
     return TWrapper(new AsyncIoUringSocket(std::forward<Args>(args)...));
 #else
     throw std::runtime_error("AsyncIoUringSocket not supported");
@@ -40,9 +41,9 @@ class AsyncIoUringSocketFactory {
   }
 
   static bool asyncDetachFd(
-      FOLLY_MAYBE_UNUSED AsyncTransport& transport,
-      FOLLY_MAYBE_UNUSED AsyncDetachFdCallback* callback) {
-#if defined(__linux__) && __has_include(<liburing.h>)
+      [[maybe_unused]] AsyncTransport& transport,
+      [[maybe_unused]] AsyncDetachFdCallback* callback) {
+#if FOLLY_HAS_LIBURING
     AsyncIoUringSocket* socket =
         transport.getUnderlyingTransport<AsyncIoUringSocket>();
     if (socket) {
