@@ -52,12 +52,17 @@ extern "C" FOLLY_ATTR_WEAK void eb_poll_loop_post_hook(
 #else
 #define FOLLY_IO_URING_UP_TO_DATE 0
 #endif
+
+#if FOLLY_IO_URING_UP_TO_DATE
+#include <folly/experimental/io/IoUringProvidedBufferRing.h>
+#endif
+
 namespace folly {
 
 namespace {
 
 #if FOLLY_IO_URING_UP_TO_DATE
-int ioUringEnableRings(FOLLY_MAYBE_UNUSED struct io_uring* ring) {
+int ioUringEnableRings([[maybe_unused]] struct io_uring* ring) {
   // Ideally this would call ::io_uring_enable_rings directly which just runs
   // the below however this was missing from a stable version of liburing, which
   // means that some distributions were not able to compile it. see
@@ -148,7 +153,7 @@ void SignalRegistry::setNotifyFd(int sig, int fd) {
   }
 }
 
-void checkLogOverflow(FOLLY_MAYBE_UNUSED struct io_uring* ring) {
+void checkLogOverflow([[maybe_unused]] struct io_uring* ring) {
 #if FOLLY_IO_URING_UP_TO_DATE
   if (::io_uring_cq_has_overflow(ring)) {
     FB_LOG_EVERY_MS(ERROR, 10000)
@@ -243,7 +248,6 @@ class SQGroupInfoRegistry {
 
  public:
   SQGroupInfoRegistry() = default;
-  ~SQGroupInfoRegistry() = default;
 
   using FDCreateFunc = folly::Function<int(struct io_uring_params&)>;
   using FDCloseFunc = folly::Function<void()>;
@@ -321,294 +325,12 @@ class SQGroupInfoRegistry {
 
 static folly::Indestructible<SQGroupInfoRegistry> sSQGroupInfoRegistry;
 
-std::chrono::time_point<std::chrono::steady_clock> getTimerExpireTime(
-    const struct timeval& timeout,
-    std::chrono::steady_clock::time_point now =
-        std::chrono::steady_clock::now()) {
-  using namespace std::chrono;
-  microseconds const us = duration_cast<microseconds>(seconds(timeout.tv_sec)) +
-      microseconds(timeout.tv_usec);
-  return now + us;
-}
-
 #if FOLLY_IO_URING_UP_TO_DATE
 
-class ProvidedBuffersBuffer {
- public:
-  static constexpr size_t kHugePageMask = (1LLU << 21) - 1; // 2MB
-  static constexpr size_t kPageMask = (1LLU << 12) - 1; // 4095
-  static size_t calcBufferSize(int bufferShift) {
-    if (bufferShift < 5) {
-      bufferShift = 5;
-    }
-    return 1LLU << bufferShift;
-  }
-
-  ProvidedBuffersBuffer(
-      int count, int bufferShift, int ringCountShift, bool huge_pages)
-      : bufferShift_(bufferShift), bufferCount_(count) {
-    // space for the ring
-    int ringCount = 1 << ringCountShift;
-    ringMask_ = ringCount - 1;
-    ringSize_ = sizeof(struct io_uring_buf) * ringCount;
-
-    allSize_ = (ringSize_ + 31) & ~32LLU;
-
-    if (bufferShift_ < 5) {
-      bufferShift_ = 5; // for alignment
-    }
-
-    sizePerBuffer_ = calcBufferSize(bufferShift_);
-    bufferSize_ = sizePerBuffer_ * count;
-    allSize_ += bufferSize_;
-
-    int pages;
-    if (huge_pages) {
-      allSize_ = (allSize_ + kHugePageMask) & (~kHugePageMask);
-      pages = allSize_ / (1 + kHugePageMask);
-    } else {
-      allSize_ = (kPageMask + kPageMask) & ~kPageMask;
-      pages = allSize_ / (1 + kPageMask);
-    }
-
-    buffer_ = mmap(
-        nullptr,
-        allSize_,
-        PROT_READ | PROT_WRITE,
-        MAP_ANONYMOUS | MAP_PRIVATE,
-        -1,
-        0);
-
-    if (buffer_ == MAP_FAILED) {
-      auto errnoCopy = errno;
-      throw std::runtime_error(folly::to<std::string>(
-          "unable to allocate pages of size ",
-          allSize_,
-          " pages=",
-          pages,
-          ": ",
-          folly::errnoStr(errnoCopy)));
-    }
-
-    bufferBuffer_ = ((char*)buffer_) + ringSize_;
-    ringPtr_ = (struct io_uring_buf_ring*)buffer_;
-
-    if (huge_pages) {
-      int ret = madvise(buffer_, allSize_, MADV_HUGEPAGE);
-      PLOG_IF(ERROR, ret) << "cannot enable huge pages";
-    }
-  }
-
-  struct io_uring_buf_ring* ring() const noexcept {
-    return ringPtr_;
-  }
-
-  struct io_uring_buf* ringBuf(int idx) const noexcept {
-    return &ringPtr_->bufs[idx & ringMask_];
-  }
-
-  uint32_t bufferCount() const noexcept { return bufferCount_; }
-  uint32_t ringCount() const noexcept { return 1 + ringMask_; }
-
-  char* buffer(uint16_t idx) {
-    size_t offset = (size_t)idx << bufferShift_;
-    return bufferBuffer_ + offset;
-  }
-
-  ~ProvidedBuffersBuffer() { munmap(buffer_, allSize_); }
-
-  size_t sizePerBuffer() const { return sizePerBuffer_; }
-
- private:
-  void* buffer_;
-  size_t allSize_;
-
-  size_t ringSize_;
-  struct io_uring_buf_ring* ringPtr_;
-  int ringMask_;
-
-  size_t bufferSize_;
-  size_t bufferShift_;
-  size_t sizePerBuffer_;
-  char* bufferBuffer_;
-  uint32_t bufferCount_;
-};
-
-class ProvidedBufferRing : public IoUringBufferProviderBase {
- public:
-  ProvidedBufferRing(
-      IoUringBackend* backend,
-      uint16_t gid,
-      int count,
-      int bufferShift,
-      int ringSizeShift)
-      : IoUringBufferProviderBase(
-            gid, ProvidedBuffersBuffer::calcBufferSize(bufferShift)),
-        backend_(backend),
-        buffer_(count, bufferShift, ringSizeShift, true) {
-    if (count > std::numeric_limits<uint16_t>::max()) {
-      throw std::runtime_error("too many buffers");
-    }
-    if (count <= 0) {
-      throw std::runtime_error("not enough buffers");
-    }
-
-    ioBufCallbacks_.assign((count + (sizeof(void*) - 1)) / sizeof(void*), this);
-
-    initialRegister();
-
-    gottenBuffers_ += count;
-    for (int i = 0; i < count; i++) {
-      returnBuffer(i);
-    }
-  }
-
-  void enobuf() noexcept override {
-    {
-      // what we want to do is something like
-      // if (cachedTail_ != localTail_) {
-      //   publish();
-      //   enobuf_ = false;
-      // }
-      // but if we are processing a batch it doesn't really work
-      // because we'll likely get an ENOBUF straight after
-      enobuf_.store(true, std::memory_order_relaxed);
-    }
-    VLOG_EVERY_N(1, 500) << "enobuf";
-  }
-
-  void unusedBuf(uint16_t i) noexcept override {
-    gottenBuffers_++;
-    returnBuffer(i);
-  }
-
-  uint32_t count() const noexcept override { return buffer_.bufferCount(); }
-
-  void destroy() noexcept override {
-    ::io_uring_unregister_buf_ring(backend_->ioRingPtr(), gid());
-    shutdownReferences_ = 1;
-    auto returned = returnedBuffers_.load();
-    {
-      std::lock_guard<std::mutex> guard(shutdownMutex_);
-      wantsShutdown_ = true;
-      // add references for every missing one
-      // we can assume that there will be no more from the kernel side.
-      // there is a race condition here between reading wantsShutdown_ and
-      // a return incrementing the number of returned references, but it is very
-      // unlikely to trigger as everything is shutting down, so there should not
-      // actually be any buffer returns. Worse case this will leak, but since
-      // everything is shutting down anyway it should not be a problem.
-      uint64_t const gotten = gottenBuffers_;
-      DCHECK(gottenBuffers_ >= returned);
-      uint32_t outstanding = (gotten - returned);
-      shutdownReferences_ += outstanding;
-    }
-    if (shutdownReferences_.fetch_sub(1) == 1) {
-      delete this;
-    }
-  }
-
-  std::unique_ptr<IOBuf> getIoBuf(uint16_t i, size_t length) noexcept override {
-    std::unique_ptr<IOBuf> ret;
-    DCHECK(!wantsShutdown_);
-
-    // use a weird convention: userData = ioBufCallbacks_.data() + i
-    // ioBufCallbacks_ is just a list of the same pointer, to this
-    // so we don't need to malloc anything
-    auto free_fn = [](void*, void* userData) {
-      size_t pprov = (size_t)userData & ~((size_t)(sizeof(void*) - 1));
-      ProvidedBufferRing* prov = *(ProvidedBufferRing**)pprov;
-      uint16_t idx = (size_t)userData - (size_t)prov->ioBufCallbacks_.data();
-      prov->returnBuffer(idx);
-    };
-
-    ret = IOBuf::takeOwnership(
-        (void*)getData(i),
-        sizePerBuffer_,
-        length,
-        free_fn,
-        (void*)(((size_t)ioBufCallbacks_.data()) + i));
-    gottenBuffers_++;
-    return ret;
-  }
-
-  bool available() const noexcept override {
-    return !enobuf_.load(std::memory_order_relaxed);
-  }
-
- private:
-  void initialRegister() {
-    struct io_uring_buf_reg reg;
-    memset(&reg, 0, sizeof(reg));
-    reg.ring_addr = (__u64)buffer_.ring();
-    reg.ring_entries = buffer_.ringCount();
-    reg.bgid = gid();
-
-    int ret = ::io_uring_register_buf_ring(backend_->ioRingPtr(), &reg, 0);
-
-    if (ret) {
-      throw IoUringBackend::NotAvailable(folly::to<std::string>(
-          "unable to register provided buffer ring ",
-          -ret,
-          ": ",
-          folly::errnoStr(-ret)));
-    }
-  }
-
-  std::atomic<uint16_t>* sharedTail() {
-    return reinterpret_cast<std::atomic<uint16_t>*>(&buffer_.ring()->tail);
-  }
-
-  bool tryPublish(uint16_t expected, uint16_t value) noexcept {
-    return sharedTail()->compare_exchange_strong(
-        expected, value, std::memory_order_release);
-  }
-
-  void returnBufferInShutdown() noexcept {
-    { std::lock_guard<std::mutex> guard(shutdownMutex_); }
-    if (shutdownReferences_.fetch_sub(1) == 1) {
-      delete this;
-    }
-  }
-
-  void returnBuffer(uint16_t i) noexcept {
-    if (FOLLY_UNLIKELY(wantsShutdown_)) {
-      returnBufferInShutdown();
-      return;
-    }
-    uint16_t this_idx = static_cast<uint16_t>(returnedBuffers_++);
-    __u64 addr = (__u64)buffer_.buffer(i);
-    uint16_t next_tail = this_idx + 1;
-    auto* r = buffer_.ringBuf(this_idx);
-    r->addr = addr;
-    r->len = buffer_.sizePerBuffer();
-    r->bid = i;
-
-    if (tryPublish(this_idx, next_tail)) {
-      enobuf_.store(false, std::memory_order_relaxed);
-    }
-    DVLOG(9) << "returnBuffer(" << i << ")@" << this_idx;
-  }
-
-  char const* getData(uint16_t i) { return buffer_.buffer(i); }
-
-  IoUringBackend* backend_;
-  ProvidedBuffersBuffer buffer_;
-  std::atomic<bool> enobuf_{false};
-  std::vector<ProvidedBufferRing*> ioBufCallbacks_;
-
-  uint64_t gottenBuffers_{0};
-  std::atomic<uint64_t> returnedBuffers_{0};
-
-  std::atomic<bool> wantsShutdown_{false};
-  std::atomic<uint32_t> shutdownReferences_;
-  std::mutex shutdownMutex_;
-};
-
 template <class... Args>
-ProvidedBufferRing::UniquePtr makeProvidedBufferRing(Args&&... args) {
-  return ProvidedBufferRing::UniquePtr(
-      new ProvidedBufferRing(std::forward<Args>(args)...));
+IoUringProvidedBufferRing::UniquePtr makeProvidedBufferRing(Args&&... args) {
+  return IoUringProvidedBufferRing::UniquePtr(
+      new IoUringProvidedBufferRing(std::forward<Args>(args)...));
 }
 
 #else
@@ -687,9 +409,8 @@ IoUringFdRegistrationRecord* IoUringBackend::FdRegistry::alloc(
     // this usually happens when we hit the file desc limit
     // and retrying this operation for every request is expensive
     err_ = true;
-    LOG(ERROR) << "io_uring_register_files(1) "
-               << "failed errno = " << errno << ":\"" << folly::errnoStr(errno)
-               << "\" " << this;
+    LOG(ERROR) << "io_uring_register_files(1) " << "failed errno = " << errno
+               << ":\"" << folly::errnoStr(errno) << "\" " << this;
     return nullptr;
   }
 
@@ -751,14 +472,14 @@ void IoSqeBase::internalSubmit(struct io_uring_sqe* sqe) noexcept {
   ::io_uring_sqe_set_data(sqe, this);
 }
 
-void IoSqeBase::internalCallback(int res, uint32_t flags) noexcept {
-  if (!(flags & IORING_CQE_F_MORE)) {
+void IoSqeBase::internalCallback(const io_uring_cqe* cqe) noexcept {
+  if (!(cqe->flags & IORING_CQE_F_MORE)) {
     inFlight_ = false;
   }
   if (cancelled_) {
-    callbackCancelled(res, flags);
+    callbackCancelled(cqe);
   } else {
-    callback(res, flags);
+    callback(cqe);
   }
 }
 
@@ -1064,12 +785,21 @@ void timerUserDataFreeFunction(void* v) {
 
 void IoUringBackend::addTimerEvent(
     Event& event, const struct timeval* timeout) {
+  auto getTimerExpireTime = [](const auto& timeout) {
+    using namespace std::chrono;
+    auto now = steady_clock::now();
+
+    auto us = duration_cast<microseconds>(seconds(timeout.tv_sec)) +
+        microseconds(timeout.tv_usec);
+    return now + us;
+  };
+
   auto expire = getTimerExpireTime(*timeout);
 
   TimerUserData* td = (TimerUserData*)event.getUserData();
-  DVLOG(6) << "addTimerEvent this=" << this << " event=" << &event
-           << " td=" << td << " changed_=" << timerChanged_
-           << " u=" << timeout->tv_usec;
+  VLOG(6) << "addTimerEvent this=" << this << " event=" << &event
+          << " td=" << td << " changed_=" << timerChanged_
+          << " u=" << timeout->tv_usec;
   if (td) {
     CHECK_EQ(event.getFreeFunction(), timerUserDataFreeFunction);
     if (td->iter == timers_.end()) {
@@ -1083,7 +813,7 @@ void IoUringBackend::addTimerEvent(
     auto it = timers_.emplace(expire, &event);
     td = new TimerUserData();
     td->iter = it;
-    DVLOG(6) << "addTimerEvent::alloc " << td << " event=" << &event;
+    VLOG(6) << "addTimerEvent::alloc " << td << " event=" << &event;
     event.setUserData(td, timerUserDataFreeFunction);
   }
   timerChanged_ |= td->iter == timers_.begin();
@@ -1091,8 +821,8 @@ void IoUringBackend::addTimerEvent(
 
 void IoUringBackend::removeTimerEvent(Event& event) {
   TimerUserData* td = (TimerUserData*)event.getUserData();
-  DVLOG(6) << "removeTimerEvent this=" << this << " event=" << &event
-           << " td=" << td;
+  VLOG(6) << "removeTimerEvent this=" << this << " event=" << &event
+          << " td=" << td;
   CHECK(td && event.getFreeFunction() == timerUserDataFreeFunction);
   timerChanged_ |= td->iter == timers_.begin();
   timers_.erase(td->iter);
@@ -1102,7 +832,7 @@ void IoUringBackend::removeTimerEvent(Event& event) {
 }
 
 size_t IoUringBackend::processTimers() {
-  DVLOG(3) << "IoUringBackend::processTimers " << timers_.size();
+  VLOG(3) << "IoUringBackend::processTimers " << timers_.size();
   size_t ret = 0;
   uint64_t data = 0;
   // this can fail with but it is OK since the fd
@@ -1118,7 +848,7 @@ size_t IoUringBackend::processTimers() {
     timerChanged_ = true;
     Event* e = it->second;
     TimerUserData* td = (TimerUserData*)e->getUserData();
-    DVLOG(5) << "processTimer " << e << " td=" << td;
+    VLOG(5) << "processTimer " << e << " td=" << td;
     CHECK(td && e->getFreeFunction() == timerUserDataFreeFunction);
     td->iter = timers_.end();
     timers_.erase(it);
@@ -1130,8 +860,8 @@ size_t IoUringBackend::processTimers() {
     ++ret;
   }
 
-  DVLOG(3) << "IoUringBackend::processTimers done, changed= " << timerChanged_
-           << " count=" << ret;
+  VLOG(3) << "IoUringBackend::processTimers done, changed= " << timerChanged_
+          << " count=" << ret;
   return ret;
 }
 
@@ -1223,7 +953,9 @@ void IoUringBackend::processPollIo(
   if (ev) {
     if (flags & IORING_CQE_F_MORE) {
       ioSqe->useCount_++;
-      SCOPE_EXIT { ioSqe->useCount_--; };
+      SCOPE_EXIT {
+        ioSqe->useCount_--;
+      };
       if (ioSqe->cbData_.processCb(this, res, flags)) {
         return;
       }
@@ -1348,7 +1080,7 @@ void IoUringBackend::initSubmissionLinked() {
     fdRegistry_.init();
   }
 
-  if (options_.initalProvidedBuffersCount) {
+  if (options_.initialProvidedBuffersCount) {
     auto get_shift = [](int x) -> int {
       int shift = findLastSet(x) - 1;
       if (x != (1 << shift)) {
@@ -1358,16 +1090,20 @@ void IoUringBackend::initSubmissionLinked() {
     };
 
     int sizeShift =
-        std::max<int>(get_shift(options_.initalProvidedBuffersEachSize), 5);
+        std::max<int>(get_shift(options_.initialProvidedBuffersEachSize), 5);
     int ringShift =
-        std::max<int>(get_shift(options_.initalProvidedBuffersCount), 1);
+        std::max<int>(get_shift(options_.initialProvidedBuffersCount), 1);
 
-    bufferProvider_ = makeProvidedBufferRing(
-        this,
-        nextBufferProviderGid(),
-        options_.initalProvidedBuffersCount,
-        sizeShift,
-        ringShift);
+    try {
+      bufferProvider_ = makeProvidedBufferRing(
+          this->ioRingPtr(),
+          nextBufferProviderGid(),
+          options_.initialProvidedBuffersCount,
+          sizeShift,
+          ringShift);
+    } catch (const IoUringProvidedBufferRing::LibUringCallError& ex) {
+      throw NotAvailable(ex.what());
+    }
   }
 }
 
@@ -1411,16 +1147,18 @@ void IoUringBackend::delayedInit() {
 int IoUringBackend::eb_event_base_loop(int flags) {
   delayedInit();
 
-  bool done = false;
-  auto waitForEvents = (flags & EVLOOP_NONBLOCK) ? WaitForEventsMode::DONT_WAIT
-                                                 : WaitForEventsMode::WAIT;
-  while (!done) {
+  const auto waitForEvents = (flags & EVLOOP_NONBLOCK)
+      ? WaitForEventsMode::DONT_WAIT
+      : WaitForEventsMode::WAIT;
+
+  bool hadEvents = true;
+  for (bool done = false; !done;) {
     scheduleTimeout();
 
     // check if we need to break here
     if (loopBreak_) {
       loopBreak_ = false;
-      break;
+      return 0;
     }
 
     prepList(submitList_);
@@ -1472,9 +1210,8 @@ int IoUringBackend::eb_event_base_loop(int flags) {
       }
     }
 
-    if (!done &&
-        (numProcessedTimers || numProcessedSignals || processedEvents) &&
-        (flags & EVLOOP_ONCE)) {
+    hadEvents = numProcessedTimers || numProcessedSignals || processedEvents;
+    if (hadEvents && (flags & EVLOOP_ONCE)) {
       done = true;
     }
 
@@ -1487,7 +1224,7 @@ int IoUringBackend::eb_event_base_loop(int flags) {
     }
   }
 
-  return 0;
+  return hadEvents ? 0 : 2;
 }
 
 int IoUringBackend::eb_event_base_loopbreak() {
@@ -1497,7 +1234,7 @@ int IoUringBackend::eb_event_base_loopbreak() {
 }
 
 int IoUringBackend::eb_event_add(Event& event, const struct timeval* timeout) {
-  DVLOG(4) << "Add event " << &event;
+  VLOG(4) << "Add event " << &event;
   auto* ev = event.getEvent();
   CHECK(ev);
   CHECK(!(event_ref_flags(ev) & ~EVLIST_ALL));
@@ -1533,7 +1270,7 @@ int IoUringBackend::eb_event_add(Event& event, const struct timeval* timeout) {
 }
 
 int IoUringBackend::eb_event_del(Event& event) {
-  DVLOG(4) << "Del event " << &event;
+  VLOG(4) << "Del event " << &event;
   if (!event.eb_ev_base()) {
     return -1;
   }
@@ -1602,7 +1339,7 @@ int IoUringBackend::eb_event_del(Event& event) {
 }
 
 int IoUringBackend::eb_event_modify_inserted(Event& event, IoSqe* ioSqe) {
-  DVLOG(4) << "Modify event " << &event;
+  VLOG(4) << "Modify event " << &event;
   // unlink and append
   ioSqe->unlink();
   if (event_ref_flags(event.getEvent()) & EVLIST_INTERNAL) {
@@ -1665,7 +1402,7 @@ void IoUringBackend::cancel(IoSqeBase* ioSqe) {
     skip = true;
   }
 #endif
-  DVLOG(4) << "Cancel " << ioSqe << " skip=" << skip;
+  VLOG(4) << "Cancel " << ioSqe << " skip=" << skip;
 }
 
 int IoUringBackend::cancelOne(IoSqe* ioSqe) {
@@ -1691,7 +1428,9 @@ size_t IoUringBackend::getActiveEvents(WaitForEventsMode waitForEvents) {
   struct io_uring_cqe* cqe;
 
   setGetActiveEvents();
-  SCOPE_EXIT { doneGetActiveEvents(); };
+  SCOPE_EXIT {
+    doneGetActiveEvents();
+  };
 
   auto inner_do_wait = [&]() -> int {
     if (waitingToSubmit_) {
@@ -1701,7 +1440,7 @@ size_t IoUringBackend::getActiveEvents(WaitForEventsMode waitForEvents) {
     } else if (useReqBatching()) {
       struct __kernel_timespec timeout;
       timeout.tv_sec = 0;
-      timeout.tv_nsec = options_.timeout * 1000;
+      timeout.tv_nsec = options_.timeout.count() * 1000;
       int ret = ::io_uring_wait_cqes(
           &ioRing_, &cqe, options_.batchSize, &timeout, nullptr);
       return ret;
@@ -1814,7 +1553,7 @@ unsigned int IoUringBackend::internalProcessCqe(
         if (FOLLY_UNLIKELY(mode == InternalProcessCqeMode::CANCEL_ALL)) {
           sqe->markCancelled();
         }
-        sqe->internalCallback(cqe->res, cqe->flags);
+        sqe->internalCallback(cqe);
       } else {
         // untracked, do not increment count
       }
@@ -1877,7 +1616,7 @@ int IoUringBackend::submitBusyCheck(
           struct io_uring_cqe* cqe;
           struct __kernel_timespec timeout;
           timeout.tv_sec = 0;
-          timeout.tv_nsec = options_.timeout * 1000;
+          timeout.tv_nsec = options_.timeout.count() * 1000;
           res = ::io_uring_submit_and_wait_timeout(
               &ioRing_,
               &cqe,
@@ -2136,11 +1875,13 @@ static bool doKernelSupportsRecvmsgMultishot() {
         sqe->ioprio |= kMultishotFlag;
       }
 
-      void callback(int res, uint32_t) noexcept override {
-        supported = res != -EINVAL;
+      void callback(const io_uring_cqe* cqe) noexcept override {
+        supported = cqe->res != -EINVAL;
       }
 
-      void callbackCancelled(int, uint32_t) noexcept override { delete this; }
+      void callbackCancelled(const io_uring_cqe*) noexcept override {
+        delete this;
+      }
 
       IoUringBufferProviderBase* bp_;
       bool supported = false;
@@ -2193,7 +1934,9 @@ static bool doKernelSupportsSendZC() {
         << "doKernelSupportsSendZC: Unexpectedly io_uring_queue_init failed";
     return false;
   }
-  SCOPE_EXIT { io_uring_queue_exit(&ring); };
+  SCOPE_EXIT {
+    io_uring_queue_exit(&ring);
+  };
 
   auto* sqe = ::io_uring_get_sqe(&ring);
   if (!sqe) {
