@@ -33,8 +33,10 @@
 #include <folly/CppAttributes.h>
 #include <folly/ExceptionString.h>
 #include <folly/Function.h>
+#include <folly/Optional.h>
 #include <folly/Range.h>
 #include <folly/experimental/io/IoUringBase.h>
+#include <folly/experimental/io/Liburing.h>
 #include <folly/io/IOBuf.h>
 #include <folly/io/async/EventBaseBackendBase.h>
 #include <folly/portability/Asm.h>
@@ -44,11 +46,9 @@
 #include <poll.h>
 #endif
 
-#if defined(__linux__) && __has_include(<liburing.h>)
-#include <liburing.h>
-#endif
+#if FOLLY_HAS_LIBURING
 
-#if defined(__linux__) && __has_include(<liburing.h>)
+#include <liburing.h> // @manual
 
 namespace folly {
 
@@ -153,8 +153,8 @@ class IoUringBackend : public EventBaseBackendBase {
     }
 
     Options& setInitialProvidedBuffers(size_t eachSize, size_t count) {
-      initalProvidedBuffersCount = count;
-      initalProvidedBuffersEachSize = eachSize;
+      initialProvidedBuffersCount = count;
+      initialProvidedBuffersEachSize = eachSize;
       return *this;
     }
 
@@ -176,24 +176,49 @@ class IoUringBackend : public EventBaseBackendBase {
       return *this;
     }
 
+    Options& setTimeout(std::chrono::microseconds v) {
+      timeout = v;
+
+      return *this;
+    }
+
+    Options& setBatchSize(int v) {
+      batchSize = v;
+
+      return *this;
+    }
+
+    ssize_t sqeSize{-1};
+
     size_t capacity{256};
     size_t minCapacity{0};
     size_t maxSubmit{128};
-    ssize_t sqeSize{-1};
     size_t maxGet{256};
     size_t registeredFds{0};
-    bool registerRingFd{false};
+    size_t sqGroupNumThreads{1};
+    size_t initialProvidedBuffersCount{0};
+    size_t initialProvidedBuffersEachSize{0};
+
     uint32_t flags{0};
+
+    // Minimum number of requests (defined as sockets with data to read) to wait
+    // for per io_uring_enter
+    int batchSize{0};
+
+    bool registerRingFd{false};
     bool taskRunCoop{false};
     bool deferTaskRun{false};
 
+    // Maximum amount of time to wait (in microseconds) per io_uring_enter
+    // Both timeout _and_ batchSize must be set for io_uring_enter wait_nr to be
+    // set!
+    std::chrono::microseconds timeout{0};
     std::chrono::milliseconds sqIdle{0};
     std::chrono::milliseconds cqIdle{0};
+
     std::set<uint32_t> sqCpus;
+
     std::string sqGroupName;
-    size_t sqGroupNumThreads{1};
-    size_t initalProvidedBuffersCount{0};
-    size_t initalProvidedBuffersEachSize{0};
   };
 
   explicit IoUringBackend(Options options);
@@ -203,11 +228,10 @@ class IoUringBackend : public EventBaseBackendBase {
   bool isWaitingToSubmit() const {
     return waitingToSubmit_ || !submitList_.empty();
   }
-  struct io_uring* ioRingPtr() {
-    return &ioRing_;
-  }
-  struct io_uring_params const& params() const {
-    return params_;
+  struct io_uring* ioRingPtr() { return &ioRing_; }
+  struct io_uring_params const& params() const { return params_; }
+  bool useReqBatching() const {
+    return options_.timeout.count() > 0 && options_.batchSize > 0;
   }
 
   // from EventBaseBackendBase
@@ -326,7 +350,6 @@ class IoUringBackend : public EventBaseBackendBase {
   void submitNextLoop(IoSqeBase& ioSqe) noexcept;
   void submitSoon(IoSqeBase& ioSqe) noexcept;
   void submitNow(IoSqeBase& ioSqe);
-  void submitNowNoCqe(IoSqeBase& ioSqe, int count = 1);
   void cancel(IoSqeBase* sqe);
 
   // built in buffer provider
@@ -350,7 +373,7 @@ class IoUringBackend : public EventBaseBackendBase {
     int writeFd() const { return fds_[0]; }
 
    private:
-    std::array<int, 2> fds_{{-1, -1}};
+    std::array<int, 2> fds_{-1, -1};
   };
 
   struct UserData {
@@ -475,12 +498,11 @@ class IoUringBackend : public EventBaseBackendBase {
         bool poolAlloc = false,
         bool persist = false)
         : backend_(backend), poolAlloc_(poolAlloc), persist_(persist) {}
-    virtual ~IoSqe() override = default;
 
-    void callback(int res, uint32_t flags) noexcept override {
-      backendCb_(backend_, this, res, flags);
+    void callback(const io_uring_cqe* cqe) noexcept override {
+      backendCb_(backend_, this, cqe->res, cqe->flags);
     }
-    void callbackCancelled(int, uint32_t) noexcept override { release(); }
+    void callbackCancelled(const io_uring_cqe*) noexcept override { release(); }
     virtual void release() noexcept;
 
     IoUringBackend* backend_;
@@ -651,24 +673,14 @@ class IoUringBackend : public EventBaseBackendBase {
         const struct iovec* iov,
         off_t offset,
         bool registerFd) noexcept {
-      CHECK(sqe);
-      if (registerFd && !fdRecord_) {
-        fdRecord_ = backend_->registerFd(fd);
-      }
-
-      if (fdRecord_) {
-        ::io_uring_prep_read(
-            sqe,
-            fdRecord_->idx_,
-            iov->iov_base,
-            (unsigned int)iov->iov_len,
-            offset);
-        sqe->flags |= IOSQE_FIXED_FILE;
-      } else {
-        ::io_uring_prep_read(
-            sqe, fd, iov->iov_base, (unsigned int)iov->iov_len, offset);
-      }
-      ::io_uring_sqe_set_data(sqe, this);
+      prepUtilFunc(
+          ::io_uring_prep_read,
+          sqe,
+          registerFd,
+          fd,
+          iov->iov_base,
+          (unsigned int)iov->iov_len,
+          offset);
     }
 
     void prepWrite(
@@ -677,24 +689,14 @@ class IoUringBackend : public EventBaseBackendBase {
         const struct iovec* iov,
         off_t offset,
         bool registerFd) noexcept {
-      CHECK(sqe);
-      if (registerFd && !fdRecord_) {
-        fdRecord_ = backend_->registerFd(fd);
-      }
-
-      if (fdRecord_) {
-        ::io_uring_prep_write(
-            sqe,
-            fdRecord_->idx_,
-            iov->iov_base,
-            (unsigned int)iov->iov_len,
-            offset);
-        sqe->flags |= IOSQE_FIXED_FILE;
-      } else {
-        ::io_uring_prep_write(
-            sqe, fd, iov->iov_base, (unsigned int)iov->iov_len, offset);
-      }
-      ::io_uring_sqe_set_data(sqe, this);
+      prepUtilFunc(
+          ::io_uring_prep_write,
+          sqe,
+          registerFd,
+          fd,
+          iov->iov_base,
+          (unsigned int)iov->iov_len,
+          offset);
     }
 
     void prepRecvmsg(
@@ -702,17 +704,29 @@ class IoUringBackend : public EventBaseBackendBase {
         int fd,
         struct msghdr* msg,
         bool registerFd) noexcept {
+      prepUtilFunc(
+          ::io_uring_prep_recvmsg, sqe, registerFd, fd, msg, MSG_TRUNC);
+    }
+
+    template <typename Fn, typename... Args>
+    void prepUtilFunc(
+        Fn fn,
+        struct io_uring_sqe* sqe,
+        bool registerFd,
+        int fd,
+        Args... args) {
       CHECK(sqe);
       if (registerFd && !fdRecord_) {
         fdRecord_ = backend_->registerFd(fd);
       }
 
       if (fdRecord_) {
-        ::io_uring_prep_recvmsg(sqe, fdRecord_->idx_, msg, MSG_TRUNC);
+        fn(sqe, fdRecord_->idx_, std::forward<Args>(args)...);
         sqe->flags |= IOSQE_FIXED_FILE;
       } else {
-        ::io_uring_prep_recvmsg(sqe, fd, msg, 0);
+        fn(sqe, fd, std::forward<Args>(args)...);
       }
+
       ::io_uring_sqe_set_data(sqe, this);
     }
 
@@ -749,8 +763,6 @@ class IoUringBackend : public EventBaseBackendBase {
     FileOpIoSqe(IoUringBackend* backend, int fd, FileOpCallback&& cb)
         : IoSqe(backend, false), fd_(fd), cb_(std::move(cb)) {}
 
-    ~FileOpIoSqe() override = default;
-
     void processActive() override { cb_(res_); }
 
     int fd_{-1};
@@ -777,8 +789,6 @@ class IoUringBackend : public EventBaseBackendBase {
         FileOpCallback&& cb)
         : FileOpIoSqe(backend, fd, std::move(cb)), iov_(iov), offset_(offset) {}
 
-    ~ReadWriteIoSqe() override = default;
-
     void processActive() override { cb_(res_); }
 
     static constexpr size_t kNumInlineIoVec = 4;
@@ -789,8 +799,6 @@ class IoUringBackend : public EventBaseBackendBase {
   struct ReadIoSqe : public ReadWriteIoSqe {
     using ReadWriteIoSqe::ReadWriteIoSqe;
 
-    ~ReadIoSqe() override = default;
-
     void processSubmit(struct io_uring_sqe* sqe) noexcept override {
       prepRead(sqe, fd_, iov_.data(), offset_, false);
     }
@@ -798,7 +806,6 @@ class IoUringBackend : public EventBaseBackendBase {
 
   struct WriteIoSqe : public ReadWriteIoSqe {
     using ReadWriteIoSqe::ReadWriteIoSqe;
-    ~WriteIoSqe() override = default;
 
     void processSubmit(struct io_uring_sqe* sqe) noexcept override {
       prepWrite(sqe, fd_, iov_.data(), offset_, false);
@@ -808,8 +815,6 @@ class IoUringBackend : public EventBaseBackendBase {
   struct ReadvIoSqe : public ReadWriteIoSqe {
     using ReadWriteIoSqe::ReadWriteIoSqe;
 
-    ~ReadvIoSqe() override = default;
-
     void processSubmit(struct io_uring_sqe* sqe) noexcept override {
       ::io_uring_prep_readv(
           sqe, fd_, iov_.data(), (unsigned int)iov_.size(), offset_);
@@ -819,7 +824,6 @@ class IoUringBackend : public EventBaseBackendBase {
 
   struct WritevIoSqe : public ReadWriteIoSqe {
     using ReadWriteIoSqe::ReadWriteIoSqe;
-    ~WritevIoSqe() override = default;
 
     void processSubmit(struct io_uring_sqe* sqe) noexcept override {
       ::io_uring_prep_writev(
@@ -838,8 +842,6 @@ class IoUringBackend : public EventBaseBackendBase {
         IoUringBackend* backend, int fd, FSyncFlags flags, FileOpCallback&& cb)
         : FileOpIoSqe(backend, fd, std::move(cb)), flags_(flags) {}
 
-    ~FSyncIoSqe() override = default;
-
     void processSubmit(struct io_uring_sqe* sqe) noexcept override {
       unsigned int fsyncFlags = 0;
       switch (flags_) {
@@ -871,8 +873,6 @@ class IoUringBackend : public EventBaseBackendBase {
           flags_(flags),
           mode_(mode) {}
 
-    ~FOpenAtIoSqe() override = default;
-
     void processSubmit(struct io_uring_sqe* sqe) noexcept override {
       ::io_uring_prep_openat(sqe, fd_, path_.c_str(), flags_, mode_);
       ::io_uring_sqe_set_data(sqe, this);
@@ -892,8 +892,6 @@ class IoUringBackend : public EventBaseBackendBase {
         FileOpCallback&& cb)
         : FileOpIoSqe(backend, dfd, std::move(cb)), path_(path), how_(*how) {}
 
-    ~FOpenAt2IoSqe() override = default;
-
     void processSubmit(struct io_uring_sqe* sqe) noexcept override {
       ::io_uring_prep_openat2(sqe, fd_, path_.c_str(), &how_);
       ::io_uring_sqe_set_data(sqe, this);
@@ -906,8 +904,6 @@ class IoUringBackend : public EventBaseBackendBase {
   struct FCloseIoSqe : public FileOpIoSqe {
     using FileOpIoSqe::FileOpIoSqe;
 
-    ~FCloseIoSqe() override = default;
-
     void processSubmit(struct io_uring_sqe* sqe) noexcept override {
       ::io_uring_prep_close(sqe, fd_);
       ::io_uring_sqe_set_data(sqe, this);
@@ -929,8 +925,6 @@ class IoUringBackend : public EventBaseBackendBase {
           mask_(mask),
           statxbuf_(statxbuf) {}
 
-    ~FStatxIoSqe() override = default;
-
     void processSubmit(struct io_uring_sqe* sqe) noexcept override {
       ::io_uring_prep_statx(sqe, fd_, path_, flags_, mask_, statxbuf_);
       ::io_uring_sqe_set_data(sqe, this);
@@ -955,8 +949,6 @@ class IoUringBackend : public EventBaseBackendBase {
           offset_(offset),
           len_(len) {}
 
-    ~FAllocateIoSqe() override = default;
-
     void processSubmit(struct io_uring_sqe* sqe) noexcept override {
       ::io_uring_prep_fallocate(sqe, fd_, mode_, offset_, len_);
       ::io_uring_sqe_set_data(sqe, this);
@@ -976,8 +968,6 @@ class IoUringBackend : public EventBaseBackendBase {
         FileOpCallback&& cb)
         : FileOpIoSqe(backend, fd, std::move(cb)), msg_(msg), flags_(flags) {}
 
-    ~SendmsgIoSqe() override = default;
-
     void processSubmit(struct io_uring_sqe* sqe) noexcept override {
       ::io_uring_prep_sendmsg(sqe, fd_, msg_, flags_);
       ::io_uring_sqe_set_data(sqe, this);
@@ -996,8 +986,6 @@ class IoUringBackend : public EventBaseBackendBase {
         FileOpCallback&& cb)
         : FileOpIoSqe(backend, fd, std::move(cb)), msg_(msg), flags_(flags) {}
 
-    ~RecvmsgIoSqe() override = default;
-
     void processSubmit(struct io_uring_sqe* sqe) noexcept override {
       ::io_uring_prep_recvmsg(sqe, fd_, msg_, flags_);
       ::io_uring_sqe_set_data(sqe, this);
@@ -1077,6 +1065,7 @@ class IoUringBackend : public EventBaseBackendBase {
   size_t waitingToSubmit_{0};
   size_t numInsertedEvents_{0};
   size_t numInternalEvents_{0};
+  size_t numSendEvents_{0};
 
   // number of pooled IoSqe instances in use
   size_t numPooledIoSqeInUse_{0};
@@ -1113,4 +1102,4 @@ class IoUringBackend : public EventBaseBackendBase {
 using PollIoBackend = IoUringBackend;
 } // namespace folly
 
-#endif // __has_include(<liburing.h>)
+#endif
