@@ -21,10 +21,10 @@
 #include <folly/Format.h>
 #include <folly/Random.h>
 #include <folly/String.h>
+#include <folly/compression/elias_fano/EliasFanoCoding.h>
 #include <folly/container/Enumerate.h>
 #include <folly/container/F14Set.h>
 #include <folly/container/Foreach.h>
-#include <folly/experimental/EliasFanoCoding.h>
 #include <folly/experimental/test/CodingTestUtils.h>
 #include <folly/init/Init.h>
 
