@@ -136,11 +136,16 @@ inline void FiberManager::runReadyFiber(Fiber* fiber) {
   (void)folly::exchangeCurrentAsyncStackRoot(
       std::exchange(fiber->asyncRoot_, nullptr));
 
-  if (!observerList_.empty()) {
-    for (auto& observer : observerList_) {
-      observer.starting(reinterpret_cast<uintptr_t>(fiber));
-    }
-  }
+  folly::Optional<ExecutionObserverScopeGuard> observersGuard{
+      std::in_place,
+      &observerList_,
+      fiber,
+      folly::ExecutionObserver::CallbackType::Fiber};
+  SCOPE_EXIT {
+    // Ensure that the guard is explicitly destroyed for all terminal states, so
+    // that it is done at the right time.
+    assert(!observersGuard);
+  };
 
   while (fiber->state_ == Fiber::NOT_STARTED ||
          fiber->state_ == Fiber::READY_TO_RUN) {
@@ -159,9 +164,7 @@ inline void FiberManager::runReadyFiber(Fiber* fiber) {
   if (fiber->state_ == Fiber::AWAITING) {
     awaitFunc_(*fiber);
     awaitFunc_ = nullptr;
-    for (auto& observer : observerList_) {
-      observer.stopped(reinterpret_cast<uintptr_t>(fiber));
-    }
+    observersGuard.reset();
     currentFiber_ = nullptr;
     fiber->rcontext_ = RequestContext::saveContext();
     fiber->asyncRoot_ = folly::exchangeCurrentAsyncStackRoot(nullptr);
@@ -182,11 +185,9 @@ inline void FiberManager::runReadyFiber(Fiber* fiber) {
       }
       fiber->finallyFunc_ = nullptr;
     }
-    // Make sure LocalData is not accessible from its destructor
-    for (auto& observer : observerList_) {
-      observer.stopped(reinterpret_cast<uintptr_t>(fiber));
-    }
+    observersGuard.reset();
 
+    // Make sure LocalData is not accessible from its destructor
     currentFiber_ = nullptr;
     fiber->rcontext_ = RequestContext::saveContext();
     // Async stack roots should have been popped by the time the
@@ -199,6 +200,7 @@ inline void FiberManager::runReadyFiber(Fiber* fiber) {
 
     if (fibersPoolSize_ < options_.maxFibersPoolSize ||
         options_.fibersPoolResizePeriodMs > 0) {
+      fiber->fiberStackHighWatermark_ = 0;
       fibersPool_.push_front(*fiber);
       ++fibersPoolSize_;
     } else {
@@ -207,9 +209,7 @@ inline void FiberManager::runReadyFiber(Fiber* fiber) {
       --fibersAllocated_;
     }
   } else if (fiber->state_ == Fiber::YIELDED) {
-    for (auto& observer : observerList_) {
-      observer.stopped(reinterpret_cast<uintptr_t>(fiber));
-    }
+    observersGuard.reset();
     currentFiber_ = nullptr;
     fiber->rcontext_ = RequestContext::saveContext();
     fiber->asyncRoot_ = folly::exchangeCurrentAsyncStackRoot(nullptr);
@@ -224,7 +224,7 @@ inline void FiberManager::loopUntilNoReady() {
 
 template <typename LoopFunc>
 void FiberManager::runFibersHelper(LoopFunc&& loopFunc) {
-  if (UNLIKELY(!alternateSignalStackRegistered_)) {
+  if (FOLLY_UNLIKELY(!alternateSignalStackRegistered_)) {
     maybeRegisterAlternateSignalStack();
   }
 
@@ -271,7 +271,9 @@ inline size_t FiberManager::recordStackPosition(size_t position) {
 
 inline void FiberManager::loopUntilNoReadyImpl() {
   runFibersHelper([&] {
-    SCOPE_EXIT { isLoopScheduled_ = false; };
+    SCOPE_EXIT {
+      isLoopScheduled_ = false;
+    };
 
     bool hadRemote = true;
     while (hadRemote) {
@@ -317,7 +319,9 @@ inline void FiberManager::runEagerFiber(Fiber* fiber) {
 inline void FiberManager::runEagerFiberImpl(Fiber* fiber) {
   folly::fibers::runInMainContext([&] {
     auto prevCurrentFiber = std::exchange(currentFiber_, fiber);
-    SCOPE_EXIT { currentFiber_ = prevCurrentFiber; };
+    SCOPE_EXIT {
+      currentFiber_ = prevCurrentFiber;
+    };
     runFibersHelper([&] { runReadyFiber(fiber); });
   });
 }
@@ -541,7 +545,7 @@ void FiberManager::addTaskFinallyEager(F&& func, G&& finally) {
 
 template <typename F>
 invoke_result_t<F> FiberManager::runInMainContext(F&& func) {
-  if (UNLIKELY(activeFiber_ == nullptr)) {
+  if (FOLLY_UNLIKELY(activeFiber_ == nullptr)) {
     return runNoInline(std::forward<F>(func));
   }
 
@@ -573,12 +577,7 @@ inline bool FiberManager::hasActiveFiber() const {
 
 inline folly::Optional<std::chrono::nanoseconds>
 FiberManager::getCurrentTaskRunningTime() const {
-  if (activeFiber_ && activeFiber_->taskOptions_.logRunningTime &&
-      activeFiber_->state_ == Fiber::RUNNING) {
-    return activeFiber_->prevDuration_ + thread_clock::now() -
-        activeFiber_->currStartTime_;
-  }
-  return folly::none;
+  return currentFiber_ ? currentFiber_->getRunningTime() : folly::none;
 }
 
 inline void FiberManager::yield() {
@@ -635,7 +634,7 @@ typename FirstArgOf<F>::type::value_type inline await_async(F&& func) {
 template <typename F>
 invoke_result_t<F> inline runInMainContext(F&& func) {
   auto fm = FiberManager::getFiberManagerUnsafe();
-  if (UNLIKELY(fm == nullptr)) {
+  if (FOLLY_UNLIKELY(fm == nullptr)) {
     return runNoInline(std::forward<F>(func));
   }
   return fm->runInMainContext(std::forward<F>(func));
