@@ -33,8 +33,10 @@
 #include <folly/lang/CustomizationPoint.h>
 
 #define FOLLY_DETAIL_FORWARD_REF(a) static_cast<decltype(a)&&>(a)
-#define FOLLY_DETAIL_FORWARD_BODY(e) \
-  noexcept(noexcept(e))->decltype(e) { return e; }
+#define FOLLY_DETAIL_FORWARD_BODY(e)   \
+  noexcept(noexcept(e))->decltype(e) { \
+    return e;                          \
+  }
 
 /**
  *  include or backport:
@@ -72,7 +74,7 @@ struct invoke_fn {
   }
 };
 
-FOLLY_INLINE_VARIABLE constexpr invoke_fn invoke;
+inline constexpr invoke_fn invoke;
 
 } // namespace folly
 
@@ -88,9 +90,9 @@ namespace invoke_detail {
 struct ok_one_ {
   template <typename T>
   static constexpr bool pass_v = ( //
-      std::is_void<T>::value || //
-      std::is_reference<T>::value || //
-      std::is_function<T>::value || //
+      std::is_void_v<T> || //
+      std::is_reference_v<T> || //
+      std::is_function_v<T> || //
       is_unbounded_array_v<T> || //
       false);
 
@@ -134,7 +136,7 @@ template <typename F>
 struct traits {
   template <typename... A>
   using result = decltype( //
-      FOLLY_DECLVAL(F &&)(FOLLY_DECLVAL(A &&)...));
+      FOLLY_DECLVAL(F&&)(FOLLY_DECLVAL(A&&)...));
   template <typename... A>
   static constexpr bool nothrow = noexcept( //
       FOLLY_DECLVAL(F&&)(FOLLY_DECLVAL(A&&)...));
@@ -143,7 +145,7 @@ template <typename P>
 struct traits_member_ptr {
   template <typename... A>
   using result = decltype( //
-      std::mem_fn(FOLLY_DECLVAL(P))(FOLLY_DECLVAL(A &&)...));
+      std::mem_fn(FOLLY_DECLVAL(P))(FOLLY_DECLVAL(A&&)...));
   template <typename... A>
   static constexpr bool nothrow = noexcept( //
       std::mem_fn(FOLLY_DECLVAL(P))(FOLLY_DECLVAL(A&&)...));
@@ -161,6 +163,33 @@ struct traits<M C::*&&> : traits_member_ptr<M C::*> {};
 template <typename M, typename C>
 struct traits<M C::*const&&> : traits_member_ptr<M C::*> {};
 
+template <bool IsVoid>
+struct conv_r_;
+template <>
+struct conv_r_<true> {
+  template <bool NX, typename R, typename FR>
+  using apply = std::true_type;
+};
+template <>
+struct conv_r_<false> {
+  template <typename R>
+  static void conv(R, decay_t<R>* = nullptr) noexcept;
+  template <
+      bool NX,
+      typename R,
+      typename FR,
+      bool C = noexcept(conv<R>(FOLLY_DECLVAL(FR)))>
+  static std::bool_constant<!NX || C> test(int);
+  template <bool NX, typename R, typename FR>
+  static std::false_type test(...);
+  template <bool NX, typename R, typename FR>
+  using apply = decltype(test<NX, R, FR>(0));
+};
+
+template <bool NX, typename R, typename FR>
+static inline constexpr bool conv_r_v_ =
+    conv_r_<std::is_void_v<R>>::template apply<NX, R, FR>::value;
+
 //  adapted from: http://en.cppreference.com/w/cpp/types/result_of, CC-BY-SA
 
 template <typename F, typename... A>
@@ -175,44 +204,39 @@ struct invoke_result<void_t<invoke_result_t<F, A...>>, F, A...> {
 };
 
 template <typename Void, typename F, typename... A>
-FOLLY_INLINE_VARIABLE constexpr bool is_invocable_v = ok_<bool, F, A...>{false};
+inline constexpr bool is_invocable_v = ok_<bool, F, A...>{false};
 
 template <typename F, typename... A>
-FOLLY_INLINE_VARIABLE constexpr bool
+inline constexpr bool
     is_invocable_v<void_t<invoke_result_t<F, A...>>, F, A...> = true;
 
 template <typename Void, typename R, typename F, typename... A>
-FOLLY_INLINE_VARIABLE constexpr bool is_invocable_r_v =
-    ok_<bool, R, F, A...>{false};
+inline constexpr bool is_invocable_r_v = ok_<bool, R, F, A...>{false};
 
 // clang-format off
 template <typename R, typename F, typename... A>
-FOLLY_INLINE_VARIABLE constexpr bool
+inline constexpr bool
     is_invocable_r_v<void_t<invoke_result_t<F, A...>>, R, F, A...> =
-        std::is_void<R>::value ||
-        std::is_convertible<invoke_result_t<F, A...>, R>::value;
+        conv_r_v_<false, R, invoke_result_t<F, A...>>;
 // clang-format on
 
 template <typename Void, typename F, typename... A>
-FOLLY_INLINE_VARIABLE constexpr bool is_nothrow_invocable_v =
-    ok_<bool, F, A...>{false};
+inline constexpr bool is_nothrow_invocable_v = ok_<bool, F, A...>{false};
 
 template <typename F, typename... A>
-FOLLY_INLINE_VARIABLE constexpr bool
+inline constexpr bool
     is_nothrow_invocable_v<void_t<invoke_result_t<F, A...>>, F, A...> =
         traits<F>::template nothrow<A...>;
 
 template <typename Void, typename R, typename F, typename... A>
-FOLLY_INLINE_VARIABLE constexpr bool is_nothrow_invocable_r_v =
-    ok_<bool, R, F, A...>{false};
+inline constexpr bool is_nothrow_invocable_r_v = ok_<bool, R, F, A...>{false};
 
 // clang-format off
 template <typename R, typename F, typename... A>
-FOLLY_INLINE_VARIABLE constexpr bool
+inline constexpr bool
     is_nothrow_invocable_r_v<void_t<invoke_result_t<F, A...>>, R, F, A...> =
-        traits<F>::template nothrow<A...> && (
-            std::is_void<R>::value ||
-            is_nothrow_convertible_v<invoke_result_t<F, A...>, R>);
+        traits<F>::template nothrow<A...> &&
+            conv_r_v_<true, R, invoke_result_t<F, A...>>;
 // clang-format on
 
 } // namespace invoke_detail
@@ -226,40 +250,41 @@ using invoke_detail::invoke_result_t;
 
 //  mimic: std::is_invocable_v, C++17
 template <typename F, typename... A>
-FOLLY_INLINE_VARIABLE constexpr bool is_invocable_v =
+inline constexpr bool is_invocable_v =
     invoke_detail::is_invocable_v<void, F, A...>;
 
 //  mimic: std::is_invocable, C++17
 template <typename F, typename... A>
-struct is_invocable : bool_constant<is_invocable_v<F, A...>> {};
+struct is_invocable : std::bool_constant<is_invocable_v<F, A...>> {};
 
 //  mimic: std::is_invocable_r_v, C++17
 template <typename R, typename F, typename... A>
-FOLLY_INLINE_VARIABLE constexpr bool is_invocable_r_v =
+inline constexpr bool is_invocable_r_v =
     invoke_detail::is_invocable_r_v<void, R, F, A...>;
 
 //  mimic: std::is_invocable_r, C++17
 template <typename R, typename F, typename... A>
-struct is_invocable_r : bool_constant<is_invocable_r_v<R, F, A...>> {};
+struct is_invocable_r : std::bool_constant<is_invocable_r_v<R, F, A...>> {};
 
 //  mimic: std::is_nothrow_invocable_v, C++17
 template <typename F, typename... A>
-FOLLY_INLINE_VARIABLE constexpr bool is_nothrow_invocable_v =
+inline constexpr bool is_nothrow_invocable_v =
     invoke_detail::is_nothrow_invocable_v<void, F, A...>;
 
 //  mimic: std::is_nothrow_invocable, C++17
 template <typename F, typename... A>
-struct is_nothrow_invocable : bool_constant<is_nothrow_invocable_v<F, A...>> {};
+struct is_nothrow_invocable
+    : std::bool_constant<is_nothrow_invocable_v<F, A...>> {};
 
 //  mimic: std::is_nothrow_invocable_r_v, C++17
 template <typename R, typename F, typename... Args>
-FOLLY_INLINE_VARIABLE constexpr bool is_nothrow_invocable_r_v =
+inline constexpr bool is_nothrow_invocable_r_v =
     invoke_detail::is_nothrow_invocable_r_v<void, R, F, Args...>;
 
 //  mimic: std::is_nothrow_invocable_r, C++17
 template <typename R, typename F, typename... A>
 struct is_nothrow_invocable_r
-    : bool_constant<is_nothrow_invocable_r_v<R, F, A...>> {};
+    : std::bool_constant<is_nothrow_invocable_r_v<R, F, A...>> {};
 
 } // namespace folly
 
@@ -275,7 +300,7 @@ template <typename I>
 struct invoke_traits_base_<false, I> {};
 template <typename I>
 struct invoke_traits_base_<true, I> {
-  FOLLY_INLINE_VARIABLE static constexpr I invoke{};
+  inline static constexpr I invoke{};
 };
 template <typename I>
 using invoke_traits_base =
@@ -325,29 +350,31 @@ struct invoke_traits : detail::invoke_traits_base<I> {
   template <typename... A>
   using invoke_result_t = invoke_detail::invoke_result_t<I, A...>;
   template <typename... A>
-  FOLLY_INLINE_VARIABLE static constexpr bool is_invocable_v =
+  inline static constexpr bool is_invocable_v =
       invoke_detail::is_invocable_v<void, I, A...>;
   template <typename... A>
   struct is_invocable //
-      : bool_constant<invoke_detail::is_invocable_v<void, I, A...>> {};
+      : std::bool_constant<invoke_detail::is_invocable_v<void, I, A...>> {};
   template <typename R, typename... A>
-  FOLLY_INLINE_VARIABLE static constexpr bool is_invocable_r_v =
+  inline static constexpr bool is_invocable_r_v =
       invoke_detail::is_invocable_r_v<void, R, I, A...>;
   template <typename R, typename... A>
   struct is_invocable_r //
-      : bool_constant<invoke_detail::is_invocable_r_v<void, R, I, A...>> {};
+      : std::bool_constant< //
+            invoke_detail::is_invocable_r_v<void, R, I, A...>> {};
   template <typename... A>
-  FOLLY_INLINE_VARIABLE static constexpr bool is_nothrow_invocable_v =
+  inline static constexpr bool is_nothrow_invocable_v =
       invoke_detail::is_nothrow_invocable_v<void, I, A...>;
   template <typename... A>
   struct is_nothrow_invocable //
-      : bool_constant<invoke_detail::is_nothrow_invocable_v<void, I, A...>> {};
+      : std::bool_constant<
+            invoke_detail::is_nothrow_invocable_v<void, I, A...>> {};
   template <typename R, typename... A>
-  FOLLY_INLINE_VARIABLE static constexpr bool is_nothrow_invocable_r_v =
+  inline static constexpr bool is_nothrow_invocable_r_v =
       invoke_detail::is_nothrow_invocable_r_v<void, R, I, A...>;
   template <typename R, typename... A>
   struct is_nothrow_invocable_r //
-      : bool_constant<
+      : std::bool_constant<
             invoke_detail::is_nothrow_invocable_r_v<void, R, I, A...>> {};
 };
 
@@ -470,22 +497,21 @@ struct invoke_first_match : private Invoker... {
  *    HasData a, b;
  *    traits::invoke(a, b); // throw 7
  */
-#define FOLLY_CREATE_FREE_INVOKER(classname, funcname, ...)                \
-  namespace classname##__folly_detail_invoke_ns {                          \
-    FOLLY_MAYBE_UNUSED void funcname(                                      \
-        ::folly::detail::invoke_private_overload&);                        \
-    FOLLY_DETAIL_CREATE_FREE_INVOKE_TRAITS_USING(_, funcname, __VA_ARGS__) \
-    struct __folly_detail_invoke_obj {                                     \
-      template <typename... Args>                                          \
-      FOLLY_MAYBE_UNUSED FOLLY_ERASE_HACK_GCC constexpr auto operator()(   \
-          Args&&... args) const                                            \
-          noexcept(noexcept(funcname(static_cast<Args&&>(args)...)))       \
-              -> decltype(funcname(static_cast<Args&&>(args)...)) {        \
-        return funcname(static_cast<Args&&>(args)...);                     \
-      }                                                                    \
-    };                                                                     \
-  }                                                                        \
-  struct classname                                                         \
+#define FOLLY_CREATE_FREE_INVOKER(classname, funcname, ...)                    \
+  namespace classname##__folly_detail_invoke_ns {                              \
+    [[maybe_unused]] void funcname(::folly::detail::invoke_private_overload&); \
+    FOLLY_DETAIL_CREATE_FREE_INVOKE_TRAITS_USING(_, funcname, __VA_ARGS__)     \
+    struct __folly_detail_invoke_obj {                                         \
+      template <typename... Args>                                              \
+      [[maybe_unused]] FOLLY_ERASE_HACK_GCC constexpr auto operator()(         \
+          Args&&... args) const                                                \
+          noexcept(noexcept(funcname(static_cast<Args&&>(args)...)))           \
+              -> decltype(funcname(static_cast<Args&&>(args)...)) {            \
+        return funcname(static_cast<Args&&>(args)...);                         \
+      }                                                                        \
+    };                                                                         \
+  }                                                                            \
+  struct classname                                                             \
       : classname##__folly_detail_invoke_ns::__folly_detail_invoke_obj {}
 
 /***
@@ -499,7 +525,7 @@ struct invoke_first_match : private Invoker... {
  */
 #define FOLLY_CREATE_FREE_INVOKER_SUITE(funcname, ...)             \
   FOLLY_CREATE_FREE_INVOKER(funcname##_fn, funcname, __VA_ARGS__); \
-  FOLLY_MAYBE_UNUSED FOLLY_INLINE_VARIABLE constexpr funcname##_fn funcname {}
+  [[maybe_unused]] inline constexpr funcname##_fn funcname {}
 
 /***
  *  FOLLY_CREATE_QUAL_INVOKER
@@ -510,12 +536,11 @@ struct invoke_first_match : private Invoker... {
  *  it is required that the name be in scope and that it is not possible to
  *  provide a list of namespaces in which to look up the name..
  */
-#define FOLLY_CREATE_QUAL_INVOKER(classname, funcpath)                 \
-  struct classname {                                                   \
-    template <typename... A>                                           \
-    FOLLY_MAYBE_UNUSED FOLLY_ERASE_HACK_GCC constexpr auto operator()( \
-        A&&... a) const                                                \
-        FOLLY_DETAIL_FORWARD_BODY(funcpath(static_cast<A&&>(a)...))    \
+#define FOLLY_CREATE_QUAL_INVOKER(classname, funcpath)                        \
+  struct classname {                                                          \
+    template <typename... A>                                                  \
+    [[maybe_unused]] FOLLY_ERASE_HACK_GCC constexpr auto operator()(A&&... a) \
+        const FOLLY_DETAIL_FORWARD_BODY(funcpath(static_cast<A&&>(a)...))     \
   }
 
 /***
@@ -528,7 +553,7 @@ struct invoke_first_match : private Invoker... {
  */
 #define FOLLY_CREATE_QUAL_INVOKER_SUITE(name, funcpath) \
   FOLLY_CREATE_QUAL_INVOKER(name##_fn, funcpath);       \
-  FOLLY_MAYBE_UNUSED FOLLY_INLINE_VARIABLE constexpr name##_fn name {}
+  [[maybe_unused]] inline constexpr name##_fn name {}
 
 /***
  *  FOLLY_INVOKE_QUAL
@@ -583,7 +608,7 @@ struct invoke_first_match : private Invoker... {
 #define FOLLY_CREATE_MEMBER_INVOKER(classname, membername)                 \
   struct classname {                                                       \
     template <typename O, typename... Args>                                \
-    FOLLY_MAYBE_UNUSED FOLLY_ERASE_HACK_GCC constexpr auto operator()(     \
+    [[maybe_unused]] FOLLY_ERASE_HACK_GCC constexpr auto operator()(       \
         O&& o, Args&&... args) const                                       \
         noexcept(noexcept(                                                 \
             static_cast<O&&>(o).membername(static_cast<Args&&>(args)...))) \
@@ -602,10 +627,9 @@ struct invoke_first_match : private Invoker... {
  *
  *  See FOLLY_CREATE_MEMBER_INVOKER.
  */
-#define FOLLY_CREATE_MEMBER_INVOKER_SUITE(membername)                \
-  FOLLY_CREATE_MEMBER_INVOKER(membername##_fn, membername);          \
-  FOLLY_MAYBE_UNUSED FOLLY_INLINE_VARIABLE constexpr membername##_fn \
-      membername {}
+#define FOLLY_CREATE_MEMBER_INVOKER_SUITE(membername)       \
+  FOLLY_CREATE_MEMBER_INVOKER(membername##_fn, membername); \
+  [[maybe_unused]] inline constexpr membername##_fn membername {}
 
 /***
  *  FOLLY_INVOKE_MEMBER
@@ -681,7 +705,7 @@ struct invoke_first_match : private Invoker... {
   template <typename T>                                                 \
   struct classname {                                                    \
     template <typename... Args, typename U = T>                         \
-    FOLLY_MAYBE_UNUSED FOLLY_ERASE_HACK_GCC constexpr auto operator()(  \
+    [[maybe_unused]] FOLLY_ERASE_HACK_GCC constexpr auto operator()(    \
         Args&&... args) const                                           \
         noexcept(noexcept(U::membername(static_cast<Args&&>(args)...))) \
             -> decltype(U::membername(static_cast<Args&&>(args)...)) {  \
@@ -699,11 +723,10 @@ struct invoke_first_match : private Invoker... {
  *
  *  See FOLLY_CREATE_STATIC_MEMBER_INVOKER.
  */
-#define FOLLY_CREATE_STATIC_MEMBER_INVOKER_SUITE(membername)            \
-  FOLLY_CREATE_STATIC_MEMBER_INVOKER(membername##_fn, membername);      \
-  template <typename T>                                                 \
-  FOLLY_MAYBE_UNUSED FOLLY_INLINE_VARIABLE constexpr membername##_fn<T> \
-      membername {}
+#define FOLLY_CREATE_STATIC_MEMBER_INVOKER_SUITE(membername)       \
+  FOLLY_CREATE_STATIC_MEMBER_INVOKER(membername##_fn, membername); \
+  template <typename T>                                            \
+  [[maybe_unused]] inline constexpr membername##_fn<T> membername {}
 
 namespace folly {
 
@@ -738,10 +761,7 @@ using tag_invoke_result_t = decltype(tag_invoke(
 template <typename Tag, typename... Args>
 auto try_tag_invoke(int) noexcept(
     noexcept(tag_invoke(FOLLY_DECLVAL(Tag&&), FOLLY_DECLVAL(Args&&)...)))
-    -> decltype(
-        static_cast<void>(
-            tag_invoke(FOLLY_DECLVAL(Tag &&), FOLLY_DECLVAL(Args&&)...)),
-        std::true_type{});
+    -> decltype(static_cast<void>(tag_invoke(FOLLY_DECLVAL(Tag&&), FOLLY_DECLVAL(Args&&)...)), std::true_type{});
 
 template <typename Tag, typename... Args>
 std::false_type try_tag_invoke(...) noexcept(false);
@@ -794,23 +814,24 @@ FOLLY_DEFINE_CPO(detail_tag_invoke_fn::tag_invoke_fn, tag_invoke)
 // by ADL can be invoked with the specified types.
 
 template <typename Tag, typename... Args>
-FOLLY_INLINE_VARIABLE constexpr bool is_tag_invocable_v =
+inline constexpr bool is_tag_invocable_v =
     decltype(detail_tag_invoke_fn::try_tag_invoke<Tag, Args...>(0))::value;
 
 template <typename Tag, typename... Args>
-struct is_tag_invocable : bool_constant<is_tag_invocable_v<Tag, Args...>> {};
+struct is_tag_invocable //
+    : std::bool_constant<is_tag_invocable_v<Tag, Args...>> {};
 
 // Query whether the 'folly::tag_invoke()' CPO can be invoked with a tag
 // and arguments of the specified type and that such an invocation is
 // noexcept.
 
 template <typename Tag, typename... Args>
-FOLLY_INLINE_VARIABLE constexpr bool is_nothrow_tag_invocable_v =
+inline constexpr bool is_nothrow_tag_invocable_v =
     noexcept(detail_tag_invoke_fn::try_tag_invoke<Tag, Args...>(0));
 
 template <typename Tag, typename... Args>
 struct is_nothrow_tag_invocable
-    : bool_constant<is_nothrow_tag_invocable_v<Tag, Args...>> {};
+    : std::bool_constant<is_nothrow_tag_invocable_v<Tag, Args...>> {};
 
 // Versions of the above that check in addition that the result is
 // convertible to the given return type R.
@@ -820,7 +841,7 @@ using is_tag_invocable_r =
     folly::is_invocable_r<R, decltype(folly::tag_invoke), Tag, Args...>;
 
 template <typename R, typename Tag, typename... Args>
-FOLLY_INLINE_VARIABLE constexpr bool is_tag_invocable_r_v =
+inline constexpr bool is_tag_invocable_r_v =
     is_tag_invocable_r<R, Tag, Args...>::value;
 
 template <typename R, typename Tag, typename... Args>
@@ -828,7 +849,7 @@ using is_nothrow_tag_invocable_r =
     folly::is_nothrow_invocable_r<R, decltype(folly::tag_invoke), Tag, Args...>;
 
 template <typename R, typename Tag, typename... Args>
-FOLLY_INLINE_VARIABLE constexpr bool is_nothrow_tag_invocable_r_v =
+inline constexpr bool is_nothrow_tag_invocable_r_v =
     is_nothrow_tag_invocable_r<R, Tag, Args...>::value;
 
 using detail_tag_invoke_fn::tag_invoke_result_t;
