@@ -107,11 +107,25 @@ class Future;
 template <class T>
 class SemiFuture;
 
+template <class T>
+struct PromiseContract {
+  Promise<T> promise;
+  Future<T> future;
+};
+
+template <class T>
+struct SemiPromiseContract {
+  Promise<T> promise;
+  SemiFuture<T> future;
+};
+
 template <class T>
 class FutureSplitter;
 
 namespace futures {
 namespace detail {
+class FutureBaseHelper;
+
 template <class T>
 class FutureBase {
  protected:
@@ -150,8 +164,8 @@ class FutureBase {
       class... Args,
       typename std::enable_if<std::is_constructible<T, Args&&...>::value, int>::
           type = 0>
-  explicit FutureBase(in_place_t, Args&&... args)
-      : core_(Core::make(in_place, static_cast<Args&&>(args)...)) {}
+  explicit FutureBase(std::in_place_t, Args&&... args)
+      : core_(Core::make(std::in_place, static_cast<Args&&>(args)...)) {}
 
   FutureBase(FutureBase<T> const&) = delete;
   FutureBase(SemiFuture<T>&&) noexcept;
@@ -349,6 +363,7 @@ class FutureBase {
   void cancel() { raise(FutureCancellation()); }
 
  protected:
+  friend class FutureBaseHelper;
   friend class Promise<T>;
   template <class>
   friend class SemiFuture;
@@ -415,13 +430,17 @@ class FutureBase {
   // Variant: returns a value
   // e.g. f.thenTry([](Try<T> t){ return t.value(); });
   template <typename F, typename R>
-  typename std::enable_if<!R::ReturnsFuture::value, typename R::Return>::type
+  typename std::enable_if< //
+      !R::ReturnsFuture::value,
+      Future<typename R::value_type>>::type
   thenImplementation(F&& func, R, InlineContinuation);
 
   // Variant: returns a Future
   // e.g. f.thenTry([](Try<T> t){ return makeFuture<T>(t); });
   template <typename F, typename R>
-  typename std::enable_if<R::ReturnsFuture::value, typename R::Return>::type
+  typename std::enable_if< //
+      R::ReturnsFuture::value,
+      Future<typename R::value_type>>::type
   thenImplementation(F&& func, R, InlineContinuation);
 };
 template <class T>
@@ -549,8 +568,8 @@ class SemiFuture : private futures::detail::FutureBase<T> {
       class... Args,
       typename std::enable_if<std::is_constructible<T, Args&&...>::value, int>::
           type = 0>
-  explicit SemiFuture(in_place_t, Args&&... args)
-      : Base(in_place, static_cast<Args&&>(args)...) {}
+  explicit SemiFuture(std::in_place_t, Args&&... args)
+      : Base(std::in_place, static_cast<Args&&>(args)...) {}
 
   SemiFuture(SemiFuture<T> const&) = delete;
   // movable
@@ -667,6 +686,11 @@ class SemiFuture : private futures::detail::FutureBase<T> {
 
   /// Returns a Future which will call back on the other side of executor.
   Future<T> via(Executor::KeepAlive<> executor) &&;
+  /// Same as via() but:
+  /// - executor MUST be identical to the executor running the task from which
+  ///   viaInlineUnsafe is called.
+  /// - MAY run some deferred callbacks inline
+  Future<T> viaInlineUnsafe(Executor::KeepAlive<> executor) &&;
   Future<T> via(Executor::KeepAlive<> executor, int8_t priority) &&;
 
   /// Defer work to run on the consumer of the future.
@@ -939,13 +963,14 @@ class SemiFuture : private futures::detail::FutureBase<T> {
   // directly awaited within a folly::coro::Task coroutine.
   friend Future<T> co_viaIfAsync(
       folly::Executor::KeepAlive<> executor, SemiFuture<T>&& future) noexcept {
-    return std::move(future).via(std::move(executor));
+    return std::move(future).viaInlineUnsafe(std::move(executor));
   }
 
 #endif
 
  private:
   friend class Promise<T>;
+  friend class futures::detail::FutureBaseHelper;
   template <class>
   friend class futures::detail::FutureBase;
   template <class>
@@ -989,10 +1014,10 @@ class SemiFuture : private futures::detail::FutureBase<T> {
 };
 
 template <class T>
-std::pair<Promise<T>, SemiFuture<T>> makePromiseContract() {
+SemiPromiseContract<T> makePromiseContract() {
   auto p = Promise<T>();
   auto f = p.getSemiFuture();
-  return std::make_pair(std::move(p), std::move(f));
+  return {std::move(p), std::move(f)};
 }
 
 /// The interface (along with SemiFuture) for the consumer-side of a
@@ -1073,8 +1098,8 @@ class Future : private futures::detail::FutureBase<T> {
       class... Args,
       typename std::enable_if<std::is_constructible<T, Args&&...>::value, int>::
           type = 0>
-  explicit Future(in_place_t, Args&&... args)
-      : Base(in_place, static_cast<Args&&>(args)...) {}
+  explicit Future(std::in_place_t, Args&&... args)
+      : Base(std::in_place, static_cast<Args&&>(args)...) {}
 
   Future(Future<T> const&) = delete;
   // movable
@@ -1111,7 +1136,7 @@ class Future : private futures::detail::FutureBase<T> {
           int>::type = 0>
   Future& operator=(Future<T2>&& other) {
     return operator=(
-        std::move(other).thenValue([](T2&& v) { return T(std::move(v)); }));
+        std::move(other).thenValue([](T2 && v) { return T(std::move(v)); }));
   }
 
   using Base::cancel;
@@ -1526,110 +1551,6 @@ class Future : private futures::detail::FutureBase<T> {
   /// - `RESULT.valid() == true`
   Future<Unit> unit() && { return std::move(*this).then(); }
 
-  /// Set an error continuation for this Future. The continuation should take an
-  /// argument of the type that you want to catch, and should return a value of
-  /// the same type as this Future, or a Future of that type (see overload
-  /// below).
-  ///
-  /// Example:
-  ///
-  ///   makeFuture()
-  ///     .thenValue([](folly::Unit&&) {
-  ///       throw std::runtime_error("oh no!");
-  ///       return 42;
-  ///     })
-  ///     .thenError([](folly::exception_wrapper&& e) {
-  ///       LOG(INFO) << "std::runtime_error: " << e.get_exception()->what();
-  ///       return -1; // or makeFuture<int>(-1)
-  ///     });
-  ///
-  /// Preconditions:
-  ///
-  /// - `valid() == true` (else throws FutureInvalid)
-  ///
-  /// Postconditions:
-  ///
-  /// - Calling code should act as if `valid() == false`,
-  ///   i.e., as if `*this` was moved into RESULT.
-  /// - `RESULT.valid() == true`
-  template <class F>
-  [[deprecated(
-      "onError loses the attached executor and is weakly typed. Please move to thenError instead.")]]
-  typename std::enable_if<
-      !is_invocable_v<F, exception_wrapper> &&
-          !futures::detail::Extract<F>::ReturnsFuture::value,
-      Future<T>>::type
-  onError(F&& func) && = delete;
-
-  /// Overload of onError where the error continuation returns a Future<T>
-  ///
-  /// Preconditions:
-  ///
-  /// - `valid() == true` (else throws FutureInvalid)
-  ///
-  /// Postconditions:
-  ///
-  /// - Calling code should act as if `valid() == false`,
-  ///   i.e., as if `*this` was moved into RESULT.
-  /// - `RESULT.valid() == true`
-  template <class F>
-  [[deprecated(
-      "onError loses the attached executor and is weakly typed. Please move to thenError instead.")]]
-  typename std::enable_if<
-      !is_invocable_v<F, exception_wrapper> &&
-          futures::detail::Extract<F>::ReturnsFuture::value,
-      Future<T>>::type
-  onError(F&& func) && = delete;
-
-  /// Overload of onError that takes exception_wrapper and returns Future<T>
-  ///
-  /// Preconditions:
-  ///
-  /// - `valid() == true` (else throws FutureInvalid)
-  ///
-  /// Postconditions:
-  ///
-  /// - Calling code should act as if `valid() == false`,
-  ///   i.e., as if `*this` was moved into RESULT.
-  /// - `RESULT.valid() == true`
-  template <class F>
-  [[deprecated(
-      "onError loses the attached executor and is weakly typed. Please move to thenError instead.")]]
-  typename std::enable_if<
-      is_invocable_v<F, exception_wrapper> &&
-          futures::detail::Extract<F>::ReturnsFuture::value,
-      Future<T>>::type
-  onError(F&& func) && = delete;
-
-  /// Overload of onError that takes exception_wrapper and returns T
-  ///
-  /// Preconditions:
-  ///
-  /// - `valid() == true` (else throws FutureInvalid)
-  ///
-  /// Postconditions:
-  ///
-  /// - Calling code should act as if `valid() == false`,
-  ///   i.e., as if `*this` was moved into RESULT.
-  /// - `RESULT.valid() == true`
-  template <class F>
-  [[deprecated(
-      "onError loses the attached executor and is weakly typed. Please move to thenError instead.")]]
-  typename std::enable_if<
-      is_invocable_v<F, exception_wrapper> &&
-          !futures::detail::Extract<F>::ReturnsFuture::value,
-      Future<T>>::type
-  onError(F&& func) && = delete;
-
-  template <class R, class... Args>
-  Future<T> onError(R (&func)(Args...)) && = delete;
-
-  // clang-format off
-  template <class F>
-  [[deprecated(
-      "onError loses the attached executor and is weakly typed. Please move to thenError instead.")]]
-  Future<T> onError(F&& func) & = delete;
-
   /// func is like std::function<void()> and is executed unconditionally, and
   /// the value/exception is passed through to the resulting Future.
   /// func shouldn't throw, but if it does it will be captured and propagated,
@@ -1943,6 +1864,7 @@ class Future : private futures::detail::FutureBase<T> {
 
  protected:
   friend class Promise<T>;
+  friend class futures::detail::FutureBaseHelper;
   template <class>
   friend class futures::detail::FutureBase;
   template <class>
@@ -2083,10 +2005,10 @@ class Timekeeper {
 };
 
 template <class T>
-std::pair<Promise<T>, Future<T>> makePromiseContract(Executor::KeepAlive<> e) {
+PromiseContract<T> makePromiseContract(Executor::KeepAlive<> e) {
   auto p = Promise<T>();
   auto f = p.getSemiFuture().via(std::move(e));
-  return std::make_pair(std::move(p), std::move(f));
+  return {std::move(p), std::move(f)};
 }
 
 template <class F>
@@ -2221,8 +2143,7 @@ auto mapTry(Executor& exec, Collection&& c, F&& func)
 /// thunk behaves like std::function<Future<T2>(void)> or
 /// std::function<SemiFuture<T2>(void)>
 template <class F>
-auto when(bool p, F&& thunk)
-    -> decltype(std::declval<invoke_result_t<F>>().unit());
+auto when(bool p, F&& thunk) -> decltype(static_cast<F&&>(thunk)().unit());
 
 SemiFuture<Unit> wait(std::unique_ptr<fibers::Baton> baton);
 SemiFuture<Unit> wait(std::shared_ptr<fibers::Baton> baton);
@@ -2421,8 +2342,9 @@ inline Future<Unit> via(Executor::KeepAlive<> executor, int8_t priority);
 /// This is semantically equivalent to via(executor).then(func), but
 /// easier to read and slightly more efficient.
 template <class Func>
-auto via(Executor::KeepAlive<>, Func&& func) -> Future<
-    typename isFutureOrSemiFuture<decltype(std::declval<Func>()())>::Inner>;
+auto via(Executor::KeepAlive<>, Func&& func)
+    -> Future<typename isFutureOrSemiFuture<
+        decltype(static_cast<Func&&>(func)())>::Inner>;
 
 /** When all the input Futures complete, the returned Future will complete.
   Errors do not cause early termination; this Future will always succeed
@@ -2615,8 +2537,12 @@ Future<T> reduce(It first, It last, T&& initial, F&& func);
 
 /// Sugar for the most common case
 template <class Collection, class T, class F>
-auto reduce(Collection&& c, T&& initial, F&& func) -> decltype(folly::reduce(
-    c.begin(), c.end(), static_cast<T&&>(initial), static_cast<F&&>(func))) {
+auto reduce(Collection&& c, T&& initial, F&& func)
+    -> decltype(folly::reduce(
+        c.begin(),
+        c.end(),
+        static_cast<T&&>(initial),
+        static_cast<F&&>(func))) {
   return folly::reduce(
       c.begin(), c.end(), static_cast<T&&>(initial), static_cast<F&&>(func));
 }
