@@ -215,6 +215,7 @@ int AsyncServerSocket::stopAccepting(int shutdownFlags) {
   // removeAcceptCallback().
   std::vector<CallbackInfo> callbacksCopy;
   callbacks_.swap(callbacksCopy);
+  localCallbackIndex_ = -1;
   for (const auto& callback : callbacksCopy) {
     // consumer may not be set if we are running in primary event base
     if (callback.consumer) {
@@ -463,7 +464,9 @@ void AsyncServerSocket::bind(uint16_t port) {
         "bad getaddrinfo");
   }
 
-  SCOPE_EXIT { freeaddrinfo(res0); };
+  SCOPE_EXIT {
+    freeaddrinfo(res0);
+  };
 
   auto setupAddress = [&](struct addrinfo* res) {
     auto s = netops::socket(res->ai_family, res->ai_socktype, res->ai_protocol);
@@ -673,6 +676,9 @@ void AsyncServerSocket::addAcceptCallback(
     throw;
   }
   callbacks_.back().consumer = acceptor;
+  if (localCallbackIndex_ < 0 && callbacks_.back().eventBase == eventBase_) {
+    localCallbackIndex_ = static_cast<int>(callbacks_.size() - 1);
+  }
 }
 
 void AsyncServerSocket::removeAcceptCallback(
