@@ -61,8 +61,8 @@ namespace fsp = folly::portability::sockets;
 namespace folly {
 
 void AsyncUDPSocket::fromMsg(
-    FOLLY_MAYBE_UNUSED ReadCallback::OnDataAvailableParams& params,
-    FOLLY_MAYBE_UNUSED struct msghdr& msg) {
+    [[maybe_unused]] ReadCallback::OnDataAvailableParams& params,
+    [[maybe_unused]] struct msghdr& msg) {
 #ifdef FOLLY_HAVE_MSG_ERRQUEUE
   struct cmsghdr* cmsg;
   uint16_t* grosizeptr;
@@ -730,7 +730,7 @@ ssize_t AsyncUDPSocket::writev(
 }
 
 ssize_t AsyncUDPSocket::writevImpl(
-    netops::Msgheader* msg, FOLLY_MAYBE_UNUSED WriteOptions options) {
+    netops::Msgheader* msg, [[maybe_unused]] WriteOptions options) {
 #if defined(FOLLY_HAVE_MSG_ERRQUEUE) || defined(_WIN32)
   XPLAT_CMSGHDR* cm = nullptr;
 
@@ -1070,13 +1070,6 @@ void AsyncUDPSocket::pauseRead() {
 void AsyncUDPSocket::close() {
   eventBase_->dcheckIsInEventBaseThread();
 
-  if (readCallback_) {
-    auto cob = readCallback_;
-    readCallback_ = nullptr;
-
-    cob->onReadClosed();
-  }
-
   // Unregister any events we are registered for
   unregisterHandler();
 
@@ -1085,6 +1078,12 @@ void AsyncUDPSocket::close() {
   }
 
   fd_ = NetworkSocket();
+
+  if (readCallback_) {
+    auto cob = readCallback_;
+    readCallback_ = nullptr;
+    cob->onReadClosed();
+  }
 }
 
 void AsyncUDPSocket::handlerReady(uint16_t events) noexcept {
@@ -1109,7 +1108,7 @@ void AsyncUDPSocket::releaseZeroCopyBuf(uint32_t id) {
   idZeroCopyBufMap_.erase(iter);
 }
 
-bool AsyncUDPSocket::isZeroCopyMsg(FOLLY_MAYBE_UNUSED const cmsghdr& cmsg) {
+bool AsyncUDPSocket::isZeroCopyMsg([[maybe_unused]] const cmsghdr& cmsg) {
 #ifdef FOLLY_HAVE_MSG_ERRQUEUE
   if ((cmsg.cmsg_level == SOL_IP && cmsg.cmsg_type == IP_RECVERR) ||
       (cmsg.cmsg_level == SOL_IPV6 && cmsg.cmsg_type == IPV6_RECVERR)) {
@@ -1122,8 +1121,7 @@ bool AsyncUDPSocket::isZeroCopyMsg(FOLLY_MAYBE_UNUSED const cmsghdr& cmsg) {
   return false;
 }
 
-void AsyncUDPSocket::processZeroCopyMsg(
-    FOLLY_MAYBE_UNUSED const cmsghdr& cmsg) {
+void AsyncUDPSocket::processZeroCopyMsg([[maybe_unused]] const cmsghdr& cmsg) {
 #ifdef FOLLY_HAVE_MSG_ERRQUEUE
   auto serr =
       reinterpret_cast<const struct sock_extended_err*>(CMSG_DATA(&cmsg));
@@ -1239,9 +1237,8 @@ void AsyncUDPSocket::handleRead() noexcept {
 
       auto cob = readCallback_;
       readCallback_ = nullptr;
-
-      cob->onReadError(ex);
       updateRegistration();
+      cob->onReadError(ex);
       return;
     }
 
@@ -1284,6 +1281,40 @@ void AsyncUDPSocket::handleRead() noexcept {
     } else {
       bytesRead = netops::recvfrom(fd_, buf, len, MSG_TRUNC, rawAddr, &addrLen);
     }
+#elif _WIN32
+    WSABUF wBuf;
+    wBuf.buf = (CHAR*)buf;
+    wBuf.len = (ULONG)len;
+
+    WSAMSG wMsg{};
+    wMsg.dwBufferCount = 1;
+    wMsg.lpBuffers = &wBuf;
+    wMsg.name = rawAddr;
+    wMsg.namelen = addrLen;
+    wMsg.dwFlags = 0;
+
+    if (recvTos_) {
+      CHAR control[WSA_CMSG_SPACE(sizeof(INT))] = {0};
+      WSABUF controlBuf;
+      controlBuf.buf = control;
+      controlBuf.len = sizeof(control);
+      wMsg.Control = controlBuf;
+    }
+
+    bytesRead = netops::wsaRecvMesg(fd_, &wMsg);
+    if (recvTos_ && bytesRead > 0) {
+      int tosVal;
+      PCMSGHDR cmsg = WSA_CMSG_FIRSTHDR(&wMsg);
+      while (cmsg != NULL) {
+        if ((cmsg->cmsg_level == IPPROTO_IP && cmsg->cmsg_type == IP_TOS) ||
+            (cmsg->cmsg_level == IPPROTO_IPV6 &&
+             cmsg->cmsg_type == IPV6_TCLASS)) {
+          params.tos = *(PINT)WSA_CMSG_DATA(cmsg);
+          break;
+        }
+        cmsg = WSA_CMSG_NXTHDR(&wMsg, cmsg);
+      }
+    }
 #else
     bytesRead = netops::recvfrom(fd_, buf, len, MSG_TRUNC, rawAddr, &addrLen);
 #endif
@@ -1314,10 +1345,8 @@ void AsyncUDPSocket::handleRead() noexcept {
       // so that he can do some logging/stats collection if he wants.
       auto cob = readCallback_;
       readCallback_ = nullptr;
-
-      cob->onReadError(ex);
       updateRegistration();
-
+      cob->onReadError(ex);
       return;
     }
   }
@@ -1472,7 +1501,7 @@ bool AsyncUDPSocket::setTXTime(TXTime txTime) {
 #endif
 }
 
-bool AsyncUDPSocket::setRxZeroChksum6(FOLLY_MAYBE_UNUSED bool bVal) {
+bool AsyncUDPSocket::setRxZeroChksum6([[maybe_unused]] bool bVal) {
 #ifdef FOLLY_HAVE_MSG_ERRQUEUE
   if (address().getFamily() != AF_INET6) {
     return false;
@@ -1487,7 +1516,7 @@ bool AsyncUDPSocket::setRxZeroChksum6(FOLLY_MAYBE_UNUSED bool bVal) {
 #endif
 }
 
-bool AsyncUDPSocket::setTxZeroChksum6(FOLLY_MAYBE_UNUSED bool bVal) {
+bool AsyncUDPSocket::setTxZeroChksum6([[maybe_unused]] bool bVal) {
 #ifdef FOLLY_HAVE_MSG_ERRQUEUE
   if (address().getFamily() != AF_INET6) {
     return false;
@@ -1502,19 +1531,37 @@ bool AsyncUDPSocket::setTxZeroChksum6(FOLLY_MAYBE_UNUSED bool bVal) {
 #endif
 }
 
-void AsyncUDPSocket::setTrafficClass(uint8_t tclass) {
-  if (netops::setsockopt(
-          fd_, IPPROTO_IPV6, IPV6_TCLASS, &tclass, sizeof(tclass)) != 0) {
-    throw AsyncSocketException(
-        AsyncSocketException::NOT_OPEN, "Failed to set IPV6_TCLASS", errno);
+void AsyncUDPSocket::setTosOrTrafficClass(uint8_t tosOrTclass) {
+#ifdef _WIN32
+  // For windows, we can only set the values 0 and 1 (for the ECN bits).
+  // Any DSCP values have to be set via the QoS Policy
+  auto level = address().getFamily() == AF_INET6 ? IPPROTO_IPV6 : IPPROTO_IP;
+  if (tosOrTclass == 0) {
+    // Remove ECN cmsgs if any exist
+    defaultCmsgs_.erase({level, IP_ECN});
+  } else {
+    defaultCmsgs_[{level, IP_ECN}] = tosOrTclass;
   }
-}
-
-void AsyncUDPSocket::setTos(uint8_t tos) {
-  if (netops::setsockopt(fd_, IPPROTO_IP, IP_TOS, &tos, sizeof(tos)) != 0) {
-    throw AsyncSocketException(
-        AsyncSocketException::NOT_OPEN, "Failed to set IP_TOS", errno);
+#else
+  int valInt = tosOrTclass;
+  if (address().getFamily() == AF_INET6) {
+    if (netops::setsockopt(
+            fd_,
+            IPPROTO_IPV6,
+            IPV6_TCLASS,
+            &valInt,
+            socklen_t(sizeof(valInt))) != 0) {
+      throw AsyncSocketException(
+          AsyncSocketException::NOT_OPEN, "Failed to set IPV6_TCLASS", errno);
+    }
+  } else {
+    if (netops::setsockopt(
+            fd_, IPPROTO_IP, IP_TOS, &valInt, socklen_t(sizeof(valInt))) != 0) {
+      throw AsyncSocketException(
+          AsyncSocketException::NOT_OPEN, "Failed to set IP_TOS", errno);
+    }
   }
+#endif
 }
 
 void AsyncUDPSocket::applyOptions(
