@@ -33,36 +33,14 @@
 #include <folly/container/Access.h>
 #include <folly/io/async/ssl/OpenSSLUtils.h>
 #include <folly/portability/OpenSSL.h>
-#include <folly/ssl/OpenSSLLockTypes.h>
 #include <folly/ssl/OpenSSLPtrTypes.h>
 
 namespace folly {
 
 class OpenSSLTicketHandler;
-
-/**
- * Override the default password collector.
- */
-class PasswordCollector {
- public:
-  virtual ~PasswordCollector() = default;
-  /**
-   * Interface for customizing how to collect private key password.
-   *
-   * By default, OpenSSL prints a prompt on screen and request for password
-   * while loading private key. To implement a custom password collector,
-   * implement this interface and register it with SSLContext.
-   *
-   * @param password Pass collected password back to OpenSSL
-   * @param size     Maximum length of password including nullptr character
-   */
-  virtual void getPassword(std::string& password, int size) const = 0;
-
-  /**
-   * Return a description of this collector for logging purposes
-   */
-  virtual const std::string& describe() const = 0;
-};
+namespace ssl {
+class PasswordCollector;
+}
 
 /**
  * Run SSL_accept via a runner
@@ -283,7 +261,7 @@ class SSLContext {
    * @return true if peer verification is required.
    *
    */
-  virtual bool needsPeerVerification() {
+  virtual bool needsPeerVerification() const {
     /* TODO this is ugly and i can't think of a reason this should exist
      * will think of what i want to do with this later
      */
@@ -310,7 +288,7 @@ class SSLContext {
    *
    * @return mode flags that can be used with SSL_set_verify
    */
-  virtual int getVerificationMode();
+  virtual int getVerificationMode() const;
 
   /**
    * Enable/Disable authentication. Peer name validation can only be done
@@ -368,6 +346,16 @@ class SSLContext {
   virtual void loadCertKeyPairFromBufferPEM(
       folly::StringPiece cert, folly::StringPiece pkey);
 
+  /**
+   * Sets cert chain and key. Guaranteed to throw if cert and private key
+   * mismatch.
+   *
+   * @param certChain A vector of X509 certificates.
+   * @param key A private key.
+   */
+  virtual void setCertChainKeyPair(
+      std::vector<ssl::X509UniquePtr>&& certChain, ssl::EvpPkeyUniquePtr&& key);
+
   /**
    * Load cert and key from files. Guaranteed to throw if cert and key mismatch.
    * Equivalent to calling loadCertificate() and loadPrivateKey().
@@ -460,16 +448,17 @@ class SSLContext {
    *
    * @param collector Instance of user defined password collector
    */
-  virtual void passwordCollector(std::shared_ptr<PasswordCollector> collector);
+  virtual void passwordCollector(
+      std::shared_ptr<ssl::PasswordCollector> collector);
   /**
    * Obtain password collector.
    *
    * @return User defined password collector
    */
-  virtual std::shared_ptr<PasswordCollector> passwordCollector() {
+  virtual std::shared_ptr<ssl::PasswordCollector> passwordCollector() {
     return collector_;
   }
-#if FOLLY_OPENSSL_HAS_SNI
+
   /**
    * Provide SNI support
    */
@@ -522,7 +511,6 @@ class SSLContext {
    */
   typedef std::function<void(SSL* ssl)> ClientHelloCallback;
   virtual void addClientHelloCallback(const ClientHelloCallback& cb);
-#endif // FOLLY_OPENSSL_HAS_SNI
 
   /**
    * Create an SSL object from this context.
@@ -539,8 +527,7 @@ class SSLContext {
    */
   void setOptions(long options);
 
-#if FOLLY_OPENSSL_HAS_ALPN
-  std::string getAdvertisedNextProtocols();
+  std::string getAdvertisedNextProtocols() const;
 
   /**
    * Set the list of protocols that this SSL context supports. In client
@@ -594,8 +581,6 @@ class SSLContext {
     alpnAllowMismatch_ = allowMismatch;
   }
 
-#endif // FOLLY_OPENSSL_HAS_ALPN
-
   /**
    * Gets the underlying SSL_CTX for advanced usage
    */
@@ -609,8 +594,8 @@ class SSLContext {
    */
   static std::string getErrors(int errnoCopy);
 
-  bool checkPeerName() { return checkPeerName_; }
-  std::string peerFixedName() { return peerFixedName_; }
+  bool checkPeerName() const { return checkPeerName_; }
+  std::string peerFixedName() const { return peerFixedName_; }
 
 #if defined(SSL_MODE_HANDSHAKE_CUTTHROUGH)
   /**
@@ -633,11 +618,15 @@ class SSLContext {
     sslAcceptRunner_ = std::move(runner);
   }
 
-  const SSLAcceptRunner* sslAcceptRunner() { return sslAcceptRunner_.get(); }
+  const SSLAcceptRunner* sslAcceptRunner() const {
+    return sslAcceptRunner_.get();
+  }
 
   void setTicketHandler(std::unique_ptr<OpenSSLTicketHandler> handler);
 
-  OpenSSLTicketHandler* getTicketHandler() { return ticketHandler_.get(); }
+  OpenSSLTicketHandler* getTicketHandler() const {
+    return ticketHandler_.get();
+  }
 
   /**
    * Helper to match a hostname versus a pattern.
@@ -673,8 +662,6 @@ class SSLContext {
   void setAllowNoDheKex(bool flag);
 #endif
 
-  [[deprecated("Use folly::ssl::init")]] static void initializeOpenSSL();
-
  protected:
   SSL_CTX* ctx_;
 
@@ -691,11 +678,9 @@ class SSLContext {
 
   bool checkPeerName_;
   std::string peerFixedName_;
-  std::shared_ptr<PasswordCollector> collector_;
-#if FOLLY_OPENSSL_HAS_SNI
+  std::shared_ptr<ssl::PasswordCollector> collector_;
   ServerNameCallback serverNameCb_;
   std::vector<ClientHelloCallback> clientHelloCbs_;
-#endif
 
   ClientProtocolFilterCallback clientProtoFilter_{nullptr};
 
@@ -704,8 +689,6 @@ class SSLContext {
   std::unique_ptr<SSLAcceptRunner> sslAcceptRunner_;
   std::unique_ptr<OpenSSLTicketHandler> ticketHandler_;
 
-#if FOLLY_OPENSSL_HAS_ALPN
-
   struct AdvertisedNextProtocolsItem {
     unsigned char* protocols;
     unsigned length;
@@ -733,11 +716,8 @@ class SSLContext {
 
   bool alpnAllowMismatch_{true};
 
-#endif // FOLLY_OPENSSL_HAS_ALPN
-
   static int passwordCallback(char* password, int size, int, void* data);
 
-#if FOLLY_OPENSSL_HAS_SNI
   /**
    * The function that will be called directly from openssl
    * in order for the application to get the tlsext_hostname just after
@@ -750,7 +730,6 @@ class SSLContext {
    */
   static int baseServerNameOpenSSLCallback(
       SSL* ssl, int* al /* alert (return value) */, void* data);
-#endif
 
   std::string providedCiphersString_;
 
@@ -764,7 +743,4 @@ class SSLContext {
 
 typedef std::shared_ptr<SSLContext> SSLContextPtr;
 
-std::ostream& operator<<(
-    std::ostream& os, const folly::PasswordCollector& collector);
-
 } // namespace folly
