@@ -45,7 +45,7 @@
 #include <folly/portability/String.h>
 #include <folly/portability/Unistd.h>
 
-namespace folly {
+namespace folly::test {
 
 // The destructors of all callback classes assert that the state is
 // STATE_SUCCEEDED, for both possitive and negative tests. The tests
@@ -267,7 +267,6 @@ class ReadCallbackBase : public AsyncTransport::ReadCallback {
 
   void readEOF() noexcept override {
     std::cerr << "readEOF" << std::endl;
-
     socket_->close();
   }
 
@@ -740,17 +739,6 @@ class ConnectTimeoutCallback : public SSLServerAcceptCallbackBase {
   }
 };
 
-void getfds(NetworkSocket fds[2]);
-
-void getctx(
-    std::shared_ptr<folly::SSLContext> clientCtx,
-    std::shared_ptr<folly::SSLContext> serverCtx);
-
-void sslsocketpair(
-    EventBase* eventBase,
-    AsyncSSLSocket::UniquePtr* clientSock,
-    AsyncSSLSocket::UniquePtr* serverSock);
-
 class BlockingWriteClient : private AsyncSSLSocket::HandshakeCB,
                             private AsyncTransport::WriteCallback {
  public:
@@ -776,9 +764,7 @@ class BlockingWriteClient : private AsyncSSLSocket::HandshakeCB,
     socket_->sslConn(this, std::chrono::milliseconds(100));
   }
 
-  struct iovec* getIovec() const {
-    return iov_.get();
-  }
+  struct iovec* getIovec() const { return iov_.get(); }
   uint32_t getIovecCount() const { return iovCount_; }
 
  private:
@@ -825,9 +811,9 @@ class BlockingWriteServer : private AsyncSSLSocket::HandshakeCB,
                << ": rc=" << rc;
       }
       if (iov[n].iov_len > bytesLeft) {
-        FAIL() << "server did not read enough data: "
-               << "ended at byte " << bytesLeft << "/" << iov[n].iov_len
-               << " in iovec " << n << "/" << count;
+        FAIL() << "server did not read enough data: " << "ended at byte "
+               << bytesLeft << "/" << iov[n].iov_len << " in iovec " << n << "/"
+               << count;
       }
 
       idx += iov[n].iov_len;
@@ -974,7 +960,6 @@ class RenegotiatingServer : public AsyncSSLSocket::HandshakeCB,
   bool renegotiationError_{false};
 };
 
-#ifndef OPENSSL_NO_TLSEXT
 class SNIClient : private AsyncSSLSocket::HandshakeCB,
                   private AsyncTransport::WriteCallback {
  public:
@@ -1062,7 +1047,6 @@ class SNIServer : private AsyncSSLSocket::HandshakeCB,
   std::shared_ptr<folly::SSLContext> sniCtx_;
   std::string expectedServerName_;
 };
-#endif
 
 class SSLClient : public AsyncSocket::ConnectCallback,
                   public AsyncTransport::WriteCallback,
@@ -1466,4 +1450,5 @@ class SSLAcceptFiberRunner : public SSLAcceptEvbRunner {
         });
   }
 };
-} // namespace folly
+
+} // namespace folly::test
