@@ -86,7 +86,7 @@ class UDPAcceptor : public AsyncUDPServerSocket::Callback {
         writeSocket->setReusePort(true);
         writeSocket->bind(serverAddress_);
       }
-      writeSocket->setTos(1);
+      writeSocket->setTosOrTrafficClass(2);
       writeSocket->write(lastClient_, folly::IOBuf::copyBuffer(lastMsg_));
     } catch (const std::exception& ex) {
       VLOG(4) << "Failed to send PONG " << ex.what();
@@ -197,6 +197,7 @@ class UDPClient : private AsyncUDPSocket::ReadCallback, private AsyncTimeout {
     server_ = server;
     socket_ = std::make_unique<AsyncUDPSocket>(evb_);
     socket_->setRecvTos(recvTos_);
+    ASSERT_EQ(socket_->getRecvTos(), recvTos_);
 
     try {
       if (bindSocket_ == BindSocket::YES) {
