@@ -23,7 +23,7 @@ using namespace folly;
 static auto constexpr kMaxLoops = 20;
 static auto constexpr kBufferSize = 4096;
 
-TEST(ZeroCopyTest, zero_copy_in_progress) {
+TEST(ZeroCopyTest, zeroCopyInProgress) {
   ZeroCopyTest test(1, kMaxLoops, true, kBufferSize);
   CHECK(test.run());
 }
