@@ -40,7 +40,47 @@
 
 #if defined(__GLIBCXX__)
 
-//  nada
+//  https://github.com/gcc-mirror/gcc/blob/releases/gcc-10.2.0/libstdc++-v3/libsupc++/unwind-cxx.h
+
+#include <cxxabi.h>
+#include <unwind.h>
+
+//  the definition of _Unwind_Ptr in libgcc/unwind-generic.h since unwind.h in
+//  libunwind does not have this typedef
+#if defined(__ia64__) && defined(__hpux__)
+typedef unsigned _Unwind_Ptr __attribute__((__mode__(__word__)));
+#else
+typedef unsigned _Unwind_Ptr __attribute__((__mode__(__pointer__)));
+#endif
+
+namespace __cxxabiv1 {
+
+struct __cxa_exception {
+  std::type_info* exceptionType;
+  void(_GLIBCXX_CDTOR_CALLABI* exceptionDestructor)(void*);
+  std::unexpected_handler unexpectedHandler;
+  std::terminate_handler terminateHandler;
+  __cxa_exception* nextException;
+  int handlerCount;
+#ifdef __ARM_EABI_UNWINDER__
+  __cxa_exception* nextPropagatingException;
+  int propagationCount;
+#else
+  int handlerSwitchValue;
+  const unsigned char* actionRecord;
+  const unsigned char* languageSpecificData;
+  _Unwind_Ptr catchTemp;
+  void* adjustedPtr;
+#endif
+  _Unwind_Exception unwindHeader;
+};
+
+struct __cxa_refcounted_exception {
+  _Atomic_word referenceCount;
+  __cxa_exception exc;
+};
+
+} // namespace __cxxabiv1
 
 #endif // defined(__GLIBCXX__)
 
@@ -203,7 +243,12 @@ bool exception_ptr_access_rt_v_() noexcept {
 
 std::type_info const* exception_ptr_get_type_(
     std::exception_ptr const& ptr) noexcept {
-  return !ptr ? nullptr : ptr.__cxa_exception_type();
+  if (!ptr) {
+    return nullptr;
+  }
+  auto object = reinterpret_cast<void* const&>(ptr);
+  auto exception = static_cast<abi::__cxa_exception*>(object) - 1;
+  return exception->exceptionType;
 }
 
 void* exception_ptr_get_object_(
@@ -213,7 +258,7 @@ void* exception_ptr_get_object_(
     return nullptr;
   }
   auto object = reinterpret_cast<void* const&>(ptr);
-  auto type = ptr.__cxa_exception_type();
+  auto type = exception_ptr_get_type_(ptr);
   return !target || target->__do_catch(type, &object, 1) ? object : nullptr;
 }
 
@@ -482,8 +527,9 @@ struct exception_shared_string::state {
     return sizeof(state) + len + 1u;
   }
   static state* make(char const* const str, std::size_t const len) {
+    constexpr auto align = std::align_val_t{alignof(state)};
     assert(len == std::strlen(str));
-    auto addr = operator_new(object_size(len), align_val_t{alignof(state)});
+    auto addr = operator_new(object_size(len), align);
     return new (addr) state(str, len);
   }
   state(char const* const str, std::size_t const len) noexcept : size{len} {
@@ -494,8 +540,9 @@ struct exception_shared_string::state {
   }
   void copy() noexcept { refs.fetch_add(1u, relaxed); }
   void ruin() noexcept {
+    constexpr auto align = std::align_val_t{alignof(state)};
     if (!refs.load(relaxed) || !refs.fetch_sub(1u, relaxed)) {
-      operator_delete(this, object_size(size), align_val_t{alignof(state)});
+      operator_delete(this, object_size(size), align);
     }
   }
 };
