@@ -18,6 +18,7 @@
 
 #include <atomic>
 #include <exception>
+#include <string>
 #include <type_traits>
 #include <utility>
 
@@ -37,7 +38,7 @@ namespace folly {
 /// Throw an exception if exceptions are enabled, or terminate if compiled with
 /// -fno-exceptions.
 template <typename Ex>
-[[noreturn]] FOLLY_NOINLINE FOLLY_COLD void throw_exception(Ex&& ex) {
+[[noreturn, FOLLY_ATTR_GNU_COLD]] FOLLY_NOINLINE void throw_exception(Ex&& ex) {
 #if FOLLY_HAS_EXCEPTIONS
   throw static_cast<Ex&&>(ex);
 #else
@@ -50,7 +51,8 @@ template <typename Ex>
 ///
 /// Terminates as if by forwarding to throw_exception but in a noexcept context.
 template <typename Ex>
-[[noreturn]] FOLLY_NOINLINE FOLLY_COLD void terminate_with(Ex&& ex) noexcept {
+[[noreturn, FOLLY_ATTR_GNU_COLD]] FOLLY_NOINLINE void terminate_with(
+    Ex&& ex) noexcept {
   throw_exception(static_cast<Ex&&>(ex));
 }
 
@@ -76,7 +78,7 @@ using throw_exception_arg_ = //
         std::is_array<std::remove_reference_t<R>>::value,
         throw_exception_arg_array_,
         conditional_t<
-            is_trivially_copyable_v<remove_cvref_t<R>>,
+            std::is_trivially_copyable_v<remove_cvref_t<R>>,
             throw_exception_arg_trivial_,
             throw_exception_arg_base_>>;
 template <typename R>
@@ -84,11 +86,12 @@ using throw_exception_arg_t =
     typename throw_exception_arg_<R>::template apply<R>;
 
 template <typename Ex, typename... Args>
-[[noreturn]] FOLLY_NOINLINE FOLLY_COLD void throw_exception_(Args... args) {
+[[noreturn, FOLLY_ATTR_GNU_COLD]] FOLLY_NOINLINE void throw_exception_(
+    Args... args) {
   throw_exception(Ex(static_cast<Args>(args)...));
 }
 template <typename Ex, typename... Args>
-[[noreturn]] FOLLY_NOINLINE FOLLY_COLD void terminate_with_(
+[[noreturn, FOLLY_ATTR_GNU_COLD]] FOLLY_NOINLINE void terminate_with_(
     Args... args) noexcept {
   throw_exception(Ex(static_cast<Args>(args)...));
 }
@@ -150,8 +153,8 @@ template <
     typename... A,
     typename FD = std::remove_pointer_t<std::decay_t<F>>,
     std::enable_if_t<!std::is_function<FD>::value, int> = 0,
-    typename R = decltype(FOLLY_DECLVAL(F &&)(FOLLY_DECLVAL(A &&)...))>
-FOLLY_NOINLINE FOLLY_COLD R invoke_cold(F&& f, A&&... a) //
+    typename R = decltype(FOLLY_DECLVAL(F&&)(FOLLY_DECLVAL(A&&)...))>
+[[FOLLY_ATTR_GNU_COLD]] FOLLY_NOINLINE R invoke_cold(F&& f, A&&... a) //
     noexcept(noexcept(static_cast<F&&>(f)(static_cast<A&&>(a)...))) {
   return static_cast<F&&>(f)(static_cast<A&&>(a)...);
 }
@@ -160,7 +163,7 @@ template <
     typename... A,
     typename FD = std::remove_pointer_t<std::decay_t<F>>,
     std::enable_if_t<std::is_function<FD>::value, int> = 0,
-    typename R = decltype(FOLLY_DECLVAL(F &&)(FOLLY_DECLVAL(A &&)...))>
+    typename R = decltype(FOLLY_DECLVAL(F&&)(FOLLY_DECLVAL(A&&)...))>
 FOLLY_ERASE R invoke_cold(F&& f, A&&... a) //
     noexcept(noexcept(f(static_cast<A&&>(a)...))) {
   return f(static_cast<A&&>(a)...);
@@ -191,7 +194,7 @@ FOLLY_ERASE R invoke_cold(F&& f, A&&... a) //
 ///         i);
 ///   }
 template <typename F, typename... A>
-[[noreturn]] FOLLY_NOINLINE FOLLY_COLD void
+[[noreturn, FOLLY_ATTR_GNU_COLD]] FOLLY_NOINLINE void
 invoke_noreturn_cold(F&& f, A&&... a) noexcept(
     /* formatting */ noexcept(static_cast<F&&>(f)(static_cast<A&&>(a)...))) {
   static_cast<F&&>(f)(static_cast<A&&>(a)...);
@@ -232,8 +235,8 @@ template <
     typename Catch,
     typename... CatchA,
     typename R = std::common_type_t<
-        decltype(FOLLY_DECLVAL(Try &&)()),
-        decltype(FOLLY_DECLVAL(Catch &&)(
+        decltype(FOLLY_DECLVAL(Try&&)()),
+        decltype(FOLLY_DECLVAL(Catch&&)(
             FOLLY_DECLVAL(E&), FOLLY_DECLVAL(CatchA&&)...))>>
 FOLLY_ERASE_TRYCATCH R catch_exception(Try&& t, Catch&& c, CatchA&&... a) {
 #if FOLLY_HAS_EXCEPTIONS
@@ -275,8 +278,8 @@ template <
     typename Catch,
     typename... CatchA,
     typename R = std::common_type_t<
-        decltype(FOLLY_DECLVAL(Try &&)()),
-        decltype(FOLLY_DECLVAL(Catch &&)(FOLLY_DECLVAL(CatchA &&)...))>>
+        decltype(FOLLY_DECLVAL(Try&&)()),
+        decltype(FOLLY_DECLVAL(Catch&&)(FOLLY_DECLVAL(CatchA&&)...))>>
 FOLLY_ERASE_TRYCATCH R
 catch_exception(Try&& t, Catch&& c, CatchA&&... a) noexcept(
     noexcept(static_cast<Catch&&>(c)(static_cast<CatchA&&>(a)...))) {
@@ -308,19 +311,19 @@ catch_exception(Try&& t, Catch&& c, CatchA&&... a) noexcept(
 namespace detail {
 #if FOLLY_APPLE_IOS
 #if __IPHONE_OS_VERSION_MIN_REQUIRED < __IPHONE_12_0
-FOLLY_INLINE_VARIABLE constexpr bool exception_ptr_access_ct = false;
+inline constexpr bool exception_ptr_access_ct = false;
 #else
-FOLLY_INLINE_VARIABLE constexpr bool exception_ptr_access_ct = true;
+inline constexpr bool exception_ptr_access_ct = true;
 #endif
 #else
-FOLLY_INLINE_VARIABLE constexpr bool exception_ptr_access_ct = true;
+inline constexpr bool exception_ptr_access_ct = true;
 #endif
 
 // 0 unknown, 1 true, -1 false
 extern std::atomic<int> exception_ptr_access_rt_cache_;
 
-FOLLY_COLD bool exception_ptr_access_rt_v_() noexcept;
-FOLLY_COLD bool exception_ptr_access_rt_() noexcept;
+[[FOLLY_ATTR_GNU_COLD]] bool exception_ptr_access_rt_v_() noexcept;
+[[FOLLY_ATTR_GNU_COLD]] bool exception_ptr_access_rt_() noexcept;
 
 inline bool exception_ptr_access_rt() noexcept {
   auto const& cache = exception_ptr_access_rt_cache_;
@@ -363,7 +366,7 @@ void* exception_ptr_get_object_(
 //  objects fails. This is likely to do with mismatch between the application
 //  ABI and the system-provided libstdc++/libc++/cxxabi ABI. May falsely return
 //  true on other platforms.
-[[FOLLY_ATTR_PURE]] inline bool exception_ptr_access() noexcept {
+[[FOLLY_ATTR_GNU_PURE]] inline bool exception_ptr_access() noexcept {
   return detail::exception_ptr_access_ct || detail::exception_ptr_access_rt();
 }
 
@@ -450,4 +453,32 @@ class exception_shared_string {
   char const* what() const noexcept;
 };
 
+/**
+ * A wrapper around a given exception type T that allows to store a
+ * static-lifetime string as what() return value, avoiding having to copy it
+ * into dedicated allocated storage on construction as most standard exceptions
+ * do even if the message is static.
+ *
+ * The constructor from the base class can still be used, in which case what()
+ * is delegated to the base class as well.
+ */
+template <class T>
+class static_what_exception : public T {
+ protected:
+  struct static_lifetime {};
+
+ public:
+  using T::T;
+
+  static_what_exception(static_lifetime, const char* msg)
+      : T(std::string{}), msg_(msg) {}
+
+  const char* what() const noexcept override {
+    return msg_ != nullptr ? msg_ : T::what();
+  }
+
+ private:
+  const char* msg_ = nullptr;
+};
+
 } // namespace folly
