@@ -290,102 +290,10 @@ ssize_t recv(NetworkSocket s, void* buf, size_t len, int flags) {
 #endif
 }
 
-ssize_t recvfrom(
-    NetworkSocket s,
-    void* buf,
-    size_t len,
-    int flags,
-    sockaddr* from,
-    socklen_t* fromlen) {
 #ifdef _WIN32
-  if ((flags & MSG_TRUNC) == MSG_TRUNC) {
-    SOCKET h = s.data;
-
-    WSABUF wBuf{};
-    wBuf.buf = (CHAR*)buf;
-    wBuf.len = (ULONG)len;
-    WSAMSG wMsg{};
-    wMsg.dwBufferCount = 1;
-    wMsg.lpBuffers = &wBuf;
-    wMsg.name = from;
-    if (fromlen != nullptr) {
-      wMsg.namelen = *fromlen;
-    }
-
-    // WSARecvMsg is an extension, so we don't get
-    // the convenience of being able to call it directly, even though
-    // WSASendMsg is part of the normal API -_-...
-    LPFN_WSARECVMSG WSARecvMsg;
-    GUID WSARecgMsg_GUID = WSAID_WSARECVMSG;
-    DWORD recMsgBytes;
-    WSAIoctl(
-        h,
-        SIO_GET_EXTENSION_FUNCTION_POINTER,
-        &WSARecgMsg_GUID,
-        sizeof(WSARecgMsg_GUID),
-        &WSARecvMsg,
-        sizeof(WSARecvMsg),
-        &recMsgBytes,
-        nullptr,
-        nullptr);
-
-    // Attempt to disable ICMP behavior which kills the socket.
-    BOOL connReset = false;
-    DWORD bytesReturned = 0;
-    WSAIoctl(
-        h,
-        SIO_UDP_CONNRESET,
-        &connReset,
-        sizeof(connReset),
-        nullptr,
-        0,
-        &bytesReturned,
-        nullptr,
-        nullptr);
-
-    DWORD bytesReceived;
-    int res = WSARecvMsg(h, &wMsg, &bytesReceived, nullptr, nullptr);
-    errno = translate_wsa_error(WSAGetLastError(), s, WSARecvMsg, res);
-    if (res == 0) {
-      return bytesReceived;
-    }
-    if (fromlen != nullptr) {
-      *fromlen = wMsg.namelen;
-    }
-    if ((wMsg.dwFlags & MSG_TRUNC) == MSG_TRUNC) {
-      return wBuf.len + 1;
-    }
-    return -1;
-  }
-  return wrapSocketFunction<ssize_t>(
-      ::recvfrom, s, (char*)buf, (int)len, flags, from, fromlen);
-#elif defined(__EMSCRIPTEN__)
-  throw std::logic_error("Not implemented!");
-#else
-  return wrapSocketFunction<ssize_t>(
-      ::recvfrom, s, buf, len, flags, from, fromlen);
-#endif
-}
-
-ssize_t recvmsg(NetworkSocket s, msghdr* message, int flags) {
-#ifdef _WIN32
-  (void)flags;
+ssize_t wsaRecvMesg(NetworkSocket s, WSAMSG* wsaMsg) {
   SOCKET h = s.data;
 
-  WSAMSG msg;
-  msg.name = (LPSOCKADDR)message->msg_name;
-  msg.namelen = message->msg_namelen;
-  msg.Control.buf = (CHAR*)message->msg_control;
-  msg.Control.len = (ULONG)message->msg_controllen;
-  msg.dwFlags = 0;
-  msg.dwBufferCount = (DWORD)message->msg_iovlen;
-  msg.lpBuffers = new WSABUF[message->msg_iovlen];
-  SCOPE_EXIT { delete[] msg.lpBuffers; };
-  for (size_t i = 0; i < message->msg_iovlen; i++) {
-    msg.lpBuffers[i].buf = (CHAR*)message->msg_iov[i].iov_base;
-    msg.lpBuffers[i].len = (ULONG)message->msg_iov[i].iov_len;
-  }
-
   // WSARecvMsg is an extension, so we don't get
   // the convenience of being able to call it directly, even though
   // WSASendMsg is part of the normal API -_-...
@@ -418,9 +326,72 @@ ssize_t recvmsg(NetworkSocket s, msghdr* message, int flags) {
       nullptr);
 
   DWORD bytesReceived;
-  int res = WSARecvMsg(h, &msg, &bytesReceived, nullptr, nullptr);
+  int res = WSARecvMsg(h, wsaMsg, &bytesReceived, nullptr, nullptr);
   errno = translate_wsa_error(WSAGetLastError(), s, WSARecvMsg, res);
-  return res == 0 ? (ssize_t)bytesReceived : -1;
+
+  if (res == 0) {
+    return bytesReceived;
+  }
+  if ((wsaMsg->dwFlags & MSG_TRUNC) == MSG_TRUNC) {
+    return wsaMsg->lpBuffers[0].len + 1;
+  }
+  return -1;
+}
+#endif
+
+ssize_t recvfrom(
+    NetworkSocket s,
+    void* buf,
+    size_t len,
+    int flags,
+    sockaddr* from,
+    socklen_t* fromlen) {
+#ifdef _WIN32
+  if ((flags & MSG_TRUNC) == MSG_TRUNC) {
+    WSABUF wBuf{};
+    wBuf.buf = (CHAR*)buf;
+    wBuf.len = (ULONG)len;
+    WSAMSG wMsg{};
+    wMsg.dwBufferCount = 1;
+    wMsg.lpBuffers = &wBuf;
+    wMsg.name = from;
+    if (fromlen != nullptr) {
+      wMsg.namelen = *fromlen;
+    }
+
+    return wsaRecvMesg(s, &wMsg);
+  }
+  return wrapSocketFunction<ssize_t>(
+      ::recvfrom, s, (char*)buf, (int)len, flags, from, fromlen);
+#elif defined(__EMSCRIPTEN__)
+  throw std::logic_error("Not implemented!");
+#else
+  return wrapSocketFunction<ssize_t>(
+      ::recvfrom, s, buf, len, flags, from, fromlen);
+#endif
+}
+
+ssize_t recvmsg(NetworkSocket s, msghdr* message, int flags) {
+#ifdef _WIN32
+  (void)flags;
+
+  WSAMSG msg;
+  msg.name = (LPSOCKADDR)message->msg_name;
+  msg.namelen = message->msg_namelen;
+  msg.Control.buf = (CHAR*)message->msg_control;
+  msg.Control.len = (ULONG)message->msg_controllen;
+  msg.dwFlags = 0;
+  msg.dwBufferCount = (DWORD)message->msg_iovlen;
+  msg.lpBuffers = new WSABUF[message->msg_iovlen];
+  SCOPE_EXIT {
+    delete[] msg.lpBuffers;
+  };
+  for (size_t i = 0; i < message->msg_iovlen; i++) {
+    msg.lpBuffers[i].buf = (CHAR*)message->msg_iov[i].iov_base;
+    msg.lpBuffers[i].len = (ULONG)message->msg_iov[i].iov_len;
+  }
+
+  return wsaRecvMesg(s, &msg);
 #elif defined(__EMSCRIPTEN__)
   throw std::logic_error("Not implemented!");
 #else
@@ -470,9 +441,9 @@ ssize_t send(NetworkSocket s, const void* buf, size_t len, int flags) {
 #endif
 }
 
-FOLLY_MAYBE_UNUSED static ssize_t fakeSendmsg(
-    FOLLY_MAYBE_UNUSED NetworkSocket socket,
-    FOLLY_MAYBE_UNUSED const msghdr* message) {
+[[maybe_unused]] static ssize_t fakeSendmsg(
+    [[maybe_unused]] NetworkSocket socket,
+    [[maybe_unused]] const msghdr* message) {
 #ifdef _WIN32
   SOCKET h = socket.data;
   ssize_t bytesSent = 0;
@@ -509,8 +480,8 @@ FOLLY_MAYBE_UNUSED static ssize_t fakeSendmsg(
 }
 
 #ifdef _WIN32
-FOLLY_MAYBE_UNUSED ssize_t wsaSendMsgDirect(
-    FOLLY_MAYBE_UNUSED NetworkSocket socket, FOLLY_MAYBE_UNUSED WSAMSG* msg) {
+[[maybe_unused]] ssize_t wsaSendMsgDirect(
+    [[maybe_unused]] NetworkSocket socket, [[maybe_unused]] WSAMSG* msg) {
   // WSASendMsg freaks out if this pointer is not set to null but length is 0.
   if (msg->Control.len == 0) {
     msg->Control.buf = nullptr;
@@ -523,10 +494,10 @@ FOLLY_MAYBE_UNUSED ssize_t wsaSendMsgDirect(
 }
 #endif
 
-FOLLY_MAYBE_UNUSED static ssize_t wsaSendMsg(
-    FOLLY_MAYBE_UNUSED NetworkSocket socket,
-    FOLLY_MAYBE_UNUSED const msghdr* message,
-    FOLLY_MAYBE_UNUSED int flags) {
+[[maybe_unused]] static ssize_t wsaSendMsg(
+    [[maybe_unused]] NetworkSocket socket,
+    [[maybe_unused]] const msghdr* message,
+    [[maybe_unused]] int flags) {
 #ifdef _WIN32
   // Translate msghdr to WSAMSG.
   WSAMSG msg;
@@ -537,7 +508,9 @@ FOLLY_MAYBE_UNUSED static ssize_t wsaSendMsg(
   msg.dwFlags = flags;
   msg.dwBufferCount = (DWORD)message->msg_iovlen;
   msg.lpBuffers = new WSABUF[message->msg_iovlen];
-  SCOPE_EXIT { delete[] msg.lpBuffers; };
+  SCOPE_EXIT {
+    delete[] msg.lpBuffers;
+  };
   for (size_t i = 0; i < message->msg_iovlen; i++) {
     msg.lpBuffers[i].buf = (CHAR*)message->msg_iov[i].iov_base;
     msg.lpBuffers[i].len = (ULONG)message->msg_iov[i].iov_len;
