@@ -441,7 +441,9 @@ TEST(BucketedTimeSeries, avgTypeConversion) {
     // but the average fits in an int64_t
     BucketedTimeSeries<double> ts(60, seconds(600));
     uint64_t value = 0x3fffffffffffffff;
-    FOR_EACH_RANGE (i, 0, 16) { ts.addValue(seconds(0), value); }
+    FOR_EACH_RANGE (i, 0, 16) {
+      ts.addValue(seconds(0), value);
+    }
 
     EXPECT_DOUBLE_EQ(value, ts.avg());
     EXPECT_DOUBLE_EQ(value, ts.avg<float>());
@@ -454,7 +456,9 @@ TEST(BucketedTimeSeries, avgTypeConversion) {
   {
     // Test BucketedTimeSeries with a smaller integer type
     BucketedTimeSeries<int16_t> ts(60, seconds(600));
-    FOR_EACH_RANGE (i, 0, 101) { ts.addValue(seconds(0), i); }
+    FOR_EACH_RANGE (i, 0, 101) {
+      ts.addValue(seconds(0), i);
+    }
 
     EXPECT_DOUBLE_EQ(50.0, ts.avg());
     EXPECT_DOUBLE_EQ(50.0, ts.avg<float>());
@@ -1101,10 +1105,9 @@ TEST(MinuteHourTimeSeries, QueryByInterval) {
   }
 }
 
-TEST(MultiLevelTimeSeries, Basic) {
-  // using constructor with initializer_list parameter
-  folly::MultiLevelTimeSeries<int> mhts(
-      60, {seconds(60), seconds(3600), seconds(0)});
+namespace {
+
+void runBasicTest(folly::MultiLevelTimeSeries<int>& mhts) {
   EXPECT_EQ(mhts.numLevels(), 3);
 
   EXPECT_EQ(mhts.sum(seconds(60)), 0);
@@ -1217,6 +1220,29 @@ TEST(MultiLevelTimeSeries, Basic) {
   EXPECT_EQ(mhts.sum(seconds(0)), 0);
 }
 
+} // namespace
+
+TEST(MultiLevelTimeSeries, BasicInitializerArray) {
+  std::chrono::seconds levelDurations[] = {
+      seconds(60), seconds(3600), seconds(0)};
+  folly::MultiLevelTimeSeries<int> mhts(60, 3, levelDurations);
+  runBasicTest(mhts);
+}
+
+TEST(MultiLevelTimeSeries, BasicInitializerList) {
+  folly::MultiLevelTimeSeries<int> mhts(
+      60, {seconds(60), seconds(3600), seconds(0)});
+  runBasicTest(mhts);
+}
+
+TEST(MultiLevelTimeSeries, BasicVector) {
+  folly::MultiLevelTimeSeries<int> mhts(
+      60,
+      std::vector<std::chrono::seconds>{
+          seconds(60), seconds(3600), seconds(0)});
+  runBasicTest(mhts);
+}
+
 TEST(MultiLevelTimeSeries, QueryByInterval) {
   folly::MultiLevelTimeSeries<int> mhts(
       60, {seconds(60), seconds(3600), seconds(0)});
