@@ -762,6 +762,12 @@ TEST_F(ConstexprMathTest, constexpr_ceil_integral_round) {
     constexpr auto rounded = folly::constexpr_ceil(roundable, round);
     EXPECT_EQ(-20ll, rounded);
   }
+  {
+    constexpr auto roundable = 0ll;
+    constexpr auto round = std::numeric_limits<long long>::min();
+    constexpr auto rounded = folly::constexpr_ceil(roundable, round);
+    EXPECT_EQ(0ll, rounded);
+  }
 }
 
 TEST_F(ConstexprMathTest, constexpr_mult_floating) {
