@@ -261,7 +261,6 @@ void checkArrayIndex(const T& array, size_t index) {
 ////////////////////////////////////////////////////////////////////////////////
 // Benchmarks for ASCII to int conversion
 ////////////////////////////////////////////////////////////////////////////////
-// @author: Rajat Goel (rajat)
 
 static int64_t handwrittenAtoi(const char* start, const char* end) {
   bool positive = true;
@@ -278,10 +277,10 @@ static int64_t handwrittenAtoi(const char* start, const char* end) {
   switch (*start) {
     case '-':
       positive = false;
-      FOLLY_FALLTHROUGH;
+      [[fallthrough]];
     case '+':
       ++start;
-      FOLLY_FALLTHROUGH;
+      [[fallthrough]];
     default:
       break;
   }
@@ -320,7 +319,9 @@ void follyAtoiMeasure(unsigned int n, unsigned int digits) {
 void clibAtoiMeasure(unsigned int n, unsigned int digits) {
   auto p = pc1.subpiece(pc1.size() - digits, digits);
   assert(*p.end() == 0);
-  FOR_EACH_RANGE (i, 0, n) { doNotOptimizeAway(atoll(p.begin())); }
+  FOR_EACH_RANGE (i, 0, n) {
+    doNotOptimizeAway(atoll(p.begin()));
+  }
 }
 
 void lexicalCastMeasure(unsigned int n, unsigned int digits) {
@@ -476,7 +477,9 @@ struct StringIdenticalToBM {
   StringIdenticalToBM() {}
   void operator()(unsigned int n, size_t len) const {
     String s;
-    BENCHMARK_SUSPEND { s.append(len, '0'); }
+    BENCHMARK_SUSPEND {
+      s.append(len, '0');
+    }
     FOR_EACH_RANGE (i, 0, n) {
       String result = to<String>(s);
       doNotOptimizeAway(result.size());
@@ -489,7 +492,9 @@ struct StringVariadicToBM {
   StringVariadicToBM() {}
   void operator()(unsigned int n, size_t len) const {
     String s;
-    BENCHMARK_SUSPEND { s.append(len, '0'); }
+    BENCHMARK_SUSPEND {
+      s.append(len, '0');
+    }
     FOR_EACH_RANGE (i, 0, n) {
       String result = to<String>(s, nullptr);
       doNotOptimizeAway(result.size());
