@@ -18,6 +18,7 @@
 #include <dlfcn.h>
 #endif
 
+#include <latch>
 #include <thread>
 #include <unordered_set>
 #include <vector>
@@ -59,15 +60,16 @@ TEST(SingletonThreadLocalTest, TryGet) {
 
 TEST(SingletonThreadLocalTest, OneSingletonPerThread) {
   static constexpr std::size_t targetThreadCount{64};
-  std::atomic<std::size_t> completedThreadCount{0};
+  std::latch allSingletonsCreated(targetThreadCount);
   Synchronized<std::unordered_set<Foo*>> fooAddresses{};
   std::vector<std::thread> threads{};
-  auto threadFunction = [&fooAddresses, &completedThreadCount] {
+  auto threadFunction = [&fooAddresses, &allSingletonsCreated] {
     fooAddresses.wlock()->emplace(&FooSingletonTL::get());
-    ++completedThreadCount;
-    while (completedThreadCount < targetThreadCount) {
-      std::this_thread::yield();
-    }
+    // Prevent SingletonThreadLocal's internal cache from re-using
+    // objects across threads by
+    // keeping each thread alive until all the threads have a
+    // SingletonThreadLocal.
+    allSingletonsCreated.arrive_and_wait();
   };
   {
     for (std::size_t threadCount{0}; threadCount < targetThreadCount;
@@ -83,29 +85,13 @@ TEST(SingletonThreadLocalTest, OneSingletonPerThread) {
   EXPECT_EQ(threads.size(), fooDeletedCount);
 }
 
-TEST(SingletonThreadLocalTest, DefaultMakeMoveConstructible) {
-  struct Foo {
-    int a = 4;
-    Foo() = default;
-    Foo(Foo&&) = default;
-    Foo& operator=(Foo&&) = default;
-  };
-  using Real = detail::DefaultMake<Foo>::type;
-  EXPECT_TRUE((std::is_same<Real, Foo>::value));
-  struct Tag {};
-  auto& single = SingletonThreadLocal<Foo, Tag>::get();
-  EXPECT_EQ(4, single.a);
-}
-
-TEST(SingletonThreadLocalTest, DefaultMakeNotMoveConstructible) {
+TEST(SingletonThreadLocalTest, DefaultMake) {
   struct Foo {
     int a = 4;
     Foo() = default;
     Foo(Foo&&) = delete;
     Foo& operator=(Foo&&) = delete;
   };
-  using Real = detail::DefaultMake<Foo>::type;
-  EXPECT_TRUE(((__cplusplus >= 201703ULL) == std::is_same<Real, Foo>::value));
   struct Tag {};
   auto& single = SingletonThreadLocal<Foo, Tag>::get();
   EXPECT_EQ(4, single.a);
@@ -237,8 +223,8 @@ TEST(SingletonThreadLocalDeathTest, Overload) {
 
   auto message = stripLeftMargin(R"MESSAGE(
     Overloaded unique instance over <int, DeathTag, ...> with differing trailing arguments:
-      folly::SingletonThreadLocal<int, DeathTag, Make1, DeathTag>
-      folly::SingletonThreadLocal<int, DeathTag, Make2, DeathTag>
+      folly::SingletonThreadLocal<int, DeathTag, folly::detail::DefaultMake<int>, TLTag1>
+      folly::SingletonThreadLocal<int, DeathTag, folly::detail::DefaultMake<int>, TLTag2>
   )MESSAGE");
   EXPECT_DEATH(dlopen(lib.string().c_str(), RTLD_LAZY), message);
 }
