@@ -60,7 +60,7 @@ bool waitForAnyOutput(Subprocess& proc) {
   ssize_t len;
   do {
     len = ::read(proc.stdoutFd(), &buffer, 1);
-  } while (len == -1 and errno == EINTR);
+  } while (len == -1 && errno == EINTR);
   LOG(INFO) << "Read " << buffer;
   return len == 1;
 }
@@ -231,9 +231,7 @@ TEST(
 
 TEST(SubprocessTest, FatalOnDestroy) {
   EXPECT_DEATH(
-      []() {
-        Subprocess proc(std::vector<std::string>{"/bin/sleep", "10"});
-      }(),
+      []() { Subprocess proc(std::vector<std::string>{"/bin/sleep", "10"}); }(),
       "Subprocess destroyed without reaping child");
 }
 
