@@ -261,7 +261,7 @@ TEST(ThreadLocal, NotDefaultConstructible) {
     explicit Object(int v) : value{v} {}
   };
   std::atomic<int> a{};
-  ThreadLocal<Object> o{[&a] { return new Object(a++); }};
+  ThreadLocal<Object> o{[&a] { return Object(a++); }};
   EXPECT_EQ(0, o->value);
   std::thread([&] { EXPECT_EQ(1, o->value); }).join();
 }
@@ -754,7 +754,7 @@ TEST(ThreadLocal, Fork2) {
 #define SHARED_LIBRARY_TEST_NAME SharedLibrary
 #endif
 
-TEST(ThreadLocal, SHAREDLibraryTestName) {
+TEST(ThreadLocal, SHARED_LIBRARY_TEST_NAME) {
   auto const lib =
       folly::test::find_resource("folly/test/thread_local_test_lib.so");
   auto handle = dlopen(lib.string().c_str(), RTLD_LAZY);
