@@ -23,12 +23,27 @@
 #include <utility>
 #include <vector>
 
+#include <folly/CppAttributes.h>
 #include <folly/ScopeGuard.h>
 #include <folly/portability/GTest.h>
 
 using namespace folly;
 using namespace std;
 
+struct type_identity_test {
+  template <typename A>
+  static A deduce(A, type_identity_t<A>);
+};
+
+TEST(Traits, type_identity) {
+  EXPECT_TRUE((std::is_same_v<int, folly::type_identity_t<int>>));
+  EXPECT_TRUE((std::is_same_v<int, folly::type_identity<int>::type>));
+  EXPECT_TRUE(( //
+      std::is_same_v<int, decltype(type_identity_test::deduce(0, '\0'))>));
+}
+
+namespace {
+
 struct T1 {}; // old-style IsRelocatable, below
 struct T2 {}; // old-style IsRelocatable, below
 struct T3 {
@@ -49,6 +64,52 @@ template <class>
 struct A {};
 struct B {};
 
+struct HashableStruct1 {};
+struct HashableStruct2 {};
+struct UnhashableStruct {};
+
+template <typename X, typename Y>
+struct CompositeStruct {
+  X x;
+  Y y;
+};
+
+} // namespace
+
+namespace std {
+
+template <>
+struct hash<HashableStruct1> {
+  [[maybe_unused]] size_t operator()(const HashableStruct1&) const noexcept {
+    return 0;
+  }
+};
+
+template <>
+struct hash<HashableStruct2> {
+  [[maybe_unused]] size_t operator()(const HashableStruct2&) const noexcept {
+    return 0;
+  }
+};
+
+template <typename X, typename Y>
+struct hash<enable_std_hash_helper<CompositeStruct<X, Y>, X, Y>> {
+  [[maybe_unused]] size_t operator()(
+      const CompositeStruct<X, Y>& value) const noexcept {
+    return std::hash<X>{}(value.x) + std::hash<Y>{}(value.y);
+  }
+};
+
+static_assert(is_hashable_v<HashableStruct1>);
+static_assert(is_hashable_v<HashableStruct2>);
+static_assert(!is_hashable_v<UnhashableStruct>);
+static_assert(is_hashable_v<CompositeStruct<HashableStruct1, HashableStruct1>>);
+static_assert(is_hashable_v<CompositeStruct<HashableStruct1, HashableStruct2>>);
+static_assert(
+    !is_hashable_v<CompositeStruct<HashableStruct1, UnhashableStruct>>);
+
+} // namespace std
+
 namespace folly {
 template <>
 struct IsRelocatable<T1> : std::true_type {};
@@ -239,11 +300,15 @@ void testIsRelocatable(Args&&... args) {
   char vcpy[sizeof(T)];
 
   T* src = new (vsrc) T(std::forward<Args>(args)...);
-  SCOPE_EXIT { src->~T(); };
+  SCOPE_EXIT {
+    src->~T();
+  };
   std::memcpy(vcpy, vsrc, sizeof(T));
   T deep(*src);
   T* dst = new (vdst) T(std::move(*src));
-  SCOPE_EXIT { dst->~T(); };
+  SCOPE_EXIT {
+    dst->~T();
+  };
 
   EXPECT_EQ(deep, *dst);
 #pragma GCC diagnostic push
@@ -277,10 +342,8 @@ TEST(Traits, tag) {
   inspects_tag f;
   EXPECT_FALSE(f.is_char(tag_t<int>{}));
   EXPECT_TRUE(f.is_char(tag_t<char>{}));
-#if __cplusplus >= 201703L
   EXPECT_FALSE(f.is_char(tag<int>));
   EXPECT_TRUE(f.is_char(tag<char>));
-#endif
 }
 
 namespace {
@@ -369,8 +432,8 @@ TEST(Traits, fallbackIsNothrowConvertible) {
   EXPECT_FALSE((folly::fallback::is_nothrow_convertible<int, void>::value));
   EXPECT_TRUE((folly::fallback::is_nothrow_convertible<void, void>::value));
   struct foo {
-    /* implicit */ FOLLY_MAYBE_UNUSED operator std::false_type();
-    /* implicit */ FOLLY_MAYBE_UNUSED operator std::true_type() noexcept;
+    /* implicit */ [[maybe_unused]] operator std::false_type();
+    /* implicit */ [[maybe_unused]] operator std::true_type() noexcept;
   };
   EXPECT_FALSE(
       (folly::fallback::is_nothrow_convertible<foo, std::false_type>::value));
@@ -382,8 +445,8 @@ TEST(Traits, isNothrowConvertible) {
   EXPECT_FALSE((folly::is_nothrow_convertible<int, void>::value));
   EXPECT_TRUE((folly::is_nothrow_convertible<void, void>::value));
   struct foo {
-    /* implicit */ FOLLY_MAYBE_UNUSED operator std::false_type();
-    /* implicit */ FOLLY_MAYBE_UNUSED operator std::true_type() noexcept;
+    /* implicit */ [[maybe_unused]] operator std::false_type();
+    /* implicit */ [[maybe_unused]] operator std::true_type() noexcept;
   };
   EXPECT_FALSE((folly::is_nothrow_convertible<foo, std::false_type>::value));
   EXPECT_TRUE((folly::is_nothrow_convertible<foo, std::true_type>::value));
@@ -534,7 +597,7 @@ TEST(Traits, isConstexprDefaultConstructible) {
   EXPECT_TRUE(is_constexpr_default_constructible<Empty>{});
 
   struct NonTrivialDtor {
-    FOLLY_MAYBE_UNUSED ~NonTrivialDtor() {}
+    [[maybe_unused]] ~NonTrivialDtor() {}
   };
   EXPECT_FALSE(is_constexpr_default_constructible_v<NonTrivialDtor>);
   EXPECT_FALSE(is_constexpr_default_constructible<NonTrivialDtor>{});
@@ -600,6 +663,20 @@ TEST(Traits, intBitsLg) {
 #endif // FOLLY_HAVE_INT128_T
 }
 
+TEST(Traits, isAllocator) {
+  static_assert(is_allocator_v<std::allocator<int>>, "");
+  static_assert(is_allocator<std::allocator<int>>::value, "");
+
+  static_assert(is_allocator_v<std::allocator<std::string>>, "");
+  static_assert(is_allocator<std::allocator<std::string>>::value, "");
+
+  static_assert(!is_allocator_v<int>, "");
+  static_assert(!is_allocator<int>::value, "");
+
+  static_assert(!is_allocator_v<std::string>, "");
+  static_assert(!is_allocator<std::string>::value, "");
+}
+
 struct type_pack_element_test {
   template <size_t I, typename... T>
   using fallback = traits_detail::type_pack_element_fallback<I, T...>;
@@ -612,7 +689,7 @@ struct type_pack_element_test {
   using native_ic = native<IC::value, T...>;
 };
 
-TEST(Traits, typePackElementT) {
+TEST(Traits, type_pack_element) {
   using test = type_pack_element_test;
 
   EXPECT_TRUE(( //
@@ -632,20 +709,6 @@ TEST(Traits, typePackElementT) {
   EXPECT_FALSE((is_detected_v<test::native_ic, index_constant<0>>));
 }
 
-TEST(Traits, isAllocator) {
-  static_assert(is_allocator_v<std::allocator<int>>, "");
-  static_assert(is_allocator<std::allocator<int>>::value, "");
-
-  static_assert(is_allocator_v<std::allocator<std::string>>, "");
-  static_assert(is_allocator<std::allocator<std::string>>::value, "");
-
-  static_assert(!is_allocator_v<int>, "");
-  static_assert(!is_allocator<int>::value, "");
-
-  static_assert(!is_allocator_v<std::string>, "");
-  static_assert(!is_allocator<std::string>::value, "");
-}
-
 TEST(Traits, type_pack_size) {
   EXPECT_EQ(0, (type_pack_size_v<>));
   EXPECT_EQ(1, (type_pack_size_v<int>));
@@ -655,3 +718,27 @@ TEST(Traits, type_pack_size) {
   EXPECT_EQ(1, (type_pack_size_t<int>::value));
   EXPECT_EQ(5, (type_pack_size_t<long long, long, int, short, char>::value));
 }
+
+TEST(Traits, type_list_element) {
+  EXPECT_TRUE(( //
+      std::is_same_v<
+          folly::type_list_element_t<
+              3,
+              folly::tag_t<int, int, int, double, int, int>>, //
+          double>));
+  EXPECT_TRUE(( //
+      std::is_same_v<
+          folly::type_list_element_t<0, folly::tag_t<int[1]>>,
+          int[1]>));
+}
+
+TEST(Traits, type_list_size) {
+  EXPECT_EQ(0, (type_list_size_v<tag_t<>>));
+  EXPECT_EQ(1, (type_list_size_v<tag_t<int>>));
+  EXPECT_EQ(5, (type_list_size_v<tag_t<long long, long, int, short, char>>));
+
+  EXPECT_EQ(0, (type_list_size_t<tag_t<>>::value));
+  EXPECT_EQ(1, (type_list_size_t<tag_t<int>>::value));
+  EXPECT_EQ(
+      5, (type_list_size_t<tag_t<long long, long, int, short, char>>::value));
+}
