@@ -173,18 +173,6 @@ TEST_F(UtilityTest, copy_noexcept_spec) {
   EXPECT_TRUE(noexcept(folly::copy(std::move(thr)))); // note: does not copy
 }
 
-TEST_F(UtilityTest, as_const) {
-  struct S {
-    bool member() { return false; }
-    bool member() const { return true; }
-  };
-  S s;
-  EXPECT_FALSE(s.member());
-  EXPECT_TRUE(folly::as_const(s).member());
-  EXPECT_EQ(&s, &folly::as_const(s));
-  EXPECT_TRUE(noexcept(folly::as_const(s)));
-}
-
 template <typename T>
 static T& as_mutable(T const& t) {
   return const_cast<T&>(t);
@@ -266,6 +254,17 @@ TEST_F(UtilityTest, NonCopyableNonMovable) {
       "Should not be move constructible");
 }
 
+TEST_F(UtilityTest, noop) {
+  auto fn = folly::variadic_noop;
+  EXPECT_TRUE((std::is_nothrow_invocable_r_v<void, decltype(fn)>));
+}
+
+TEST_F(UtilityTest, constant_of) {
+  auto fn = folly::variadic_constant_of<3>;
+  EXPECT_TRUE((std::is_nothrow_invocable_r_v<int, decltype(fn)>));
+  EXPECT_EQ(3, fn());
+}
+
 TEST_F(UtilityTest, to_signed) {
   {
     constexpr auto actual = folly::to_signed(int32_t(-12));
@@ -353,7 +352,7 @@ static constexpr bool is_conv_v = std::is_convertible_v<S, D>;
 
 template <typename S, typename D>
 static constexpr bool is_nx_conv_v = //
-    is_conv_v<S, D>&& std::is_nothrow_constructible_v<D, S>;
+    is_conv_v<S, D> && std::is_nothrow_constructible_v<D, S>;
 
 static_assert(is_conv_v<of<int, 0, 0, 0>&, int>);
 static_assert(!is_conv_v<of<int, 0, 0, 0> const&, int>);
